﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Mvc;

namespace SMP.Common.exception
{
    internal class ApiExceptionAcitonFilterLogAttribute : Attribute, IExceptionFilter
    {
        ILogger<ApiExceptionAcitonFilterLogAttribute> logger;

        public ApiExceptionAcitonFilterLogAttribute(ILogger<ApiExceptionAcitonFilterLogAttribute> logger)
        {
            this.logger = logger;
        }

        public async void OnException(ExceptionContext context)
        {
            string guid = Guid.NewGuid().ToString();
            string errorMessage = context.Exception.Message;
            if (errorMessage.Contains("--->"))
            {
                errorMessage = errorMessage.Substring(errorMessage.IndexOf("--->") + 4);
            }
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(new ApiJsonModel
            {
                code = System.Net.HttpStatusCode.OK,
                data = null,
                success = false,
                message = $"操作失败，请求Id:#{guid.ToString()}" + errorMessage,
                Guid = guid
            });
            if (context.ExceptionHandled == false)
            {
                string msg = json;
                context.Result = new ContentResult
                {
                    Content = msg,
                    StatusCode = StatusCodes.Status500InternalServerError,
                    ContentType = "application/json; charset=utf-8"
                };
            }

            string waringMessage = $"{DateTime.Now.ToString_yyyyMMddHHmmss()} #{guid}" +
                $"\r\n\t {context.HttpContext.Request.Host}{context.HttpContext.Request.Path} {context.HttpContext.Request.Method} 请求失败";
            if (context.HttpContext.Request.QueryString.HasValue)
                waringMessage += $"\r\n\t请求QueryString{context.HttpContext.Request.QueryString.ToString() } ";
            if (context.HttpContext.Request.Body != null && context.HttpContext.Request.Body.Length < 10 * 1024)
            {
                var request = context.HttpContext.Request.Body;
                request.Position = 0;
                waringMessage += $"\r\n\t请求Body { await new StreamReader(request).ReadToEndAsync()} ";
                request.Position = 0;
            }

            try
            {
                if (context.HttpContext.Request.Form != null && context.HttpContext.Request.Form.Count > 0)
                    waringMessage += $"\r\n\t请求FromData {context.HttpContext.Request.Form.Select(x => x.Key.ToString() + "=" + x.Value.ToString() + "&") }";
            }
            catch
            {

            }
            waringMessage += $"\r\n\t错误内容：{ errorMessage }";

            logger.LogError(waringMessage);
        }
    }
}
