﻿using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;

namespace SMP.Data.Finance
{
    public interface IFinance_Logistics_LadingBillRepository : IBaseRepository<Finance_Logistics_LadingBill>
    {
        Task<IEnumerable<Finance_Logistics_LadingBill>> GetPortLadingBillsByNumbers(List<string> LadingBillNumbers);
        Task<Finance_Logistics_LadingBill> GetLadingBill(string LadingBillNumber);
        #region 帐单费项
        /// <summary>
        /// 读取帐单费项
        /// </summary>
        /// <param name="LadingBillId"></param>
        /// <returns></returns>
        Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId);
        /// <summary>
        /// 清空帐单费项
        /// </summary>
        /// <param name="LadingBillId"></param>
        /// <returns></returns>
        Task ClearIncidentals(int LadingBillId);

        Task<IEnumerable<Finance_Logistics_LadingBill>> QueryByLadingBillNumbers(IEnumerable<string> ladingBillNumbers);
        #endregion
        #region 订单成本(作废)
        //Task CreateCostTable(string BillTypeCode);
        //Task<bool> InsertCost(IEnumerable<Finance_Cost> cost_Bases, string BillTypeCode, string oldBillTypeCode);
        #endregion

    }
    public class Finance_Logistics_LadingBillRepository : BaseRepository<Finance_Logistics_LadingBill>, IFinance_Logistics_LadingBillRepository
    {
        public Finance_Logistics_LadingBillRepository(IUnitOfWork unitOfWork) : base(unitOfWork)
        {

        }
        public async Task<IEnumerable<Finance_Logistics_LadingBill>> GetPortLadingBillsByNumbers(List<string> LadingBillNumbers)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(" select * from ( ");
            sb.Append(" select *,ROW_NUMBER() over (partition by  LadingBillNumber,ISNULL(_Destination,'') order by id desc)rowNum  from Finance_Logistics_LadingBill(nolock)  ");
            sb.Append(" where ''!=ISNULL(_Destination,'') and LadingBillNumber in @LadingBillNumbers ");
            sb.Append(@" ) a
where rowNum=1 ");
            var result = await _unitOfWork.QueryBySqlAsync<Finance_Logistics_LadingBill>(sb.ToString(), null, new { LadingBillNumbers = LadingBillNumbers });
            return result;
        }
        public async Task<Finance_Logistics_LadingBill> GetLadingBill(string LadingBillNumber)
        {
            var result =await QueryAsync(x=>x.LadingBillNumber==LadingBillNumber);
            return result.FirstOrDefault();
        }
        #region 帐单费项
        public async Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId)
        {
            string sql = $"select * from Finance_Incidental(nolock) where LadingBillId={LadingBillId} and IsDeleted=0 ";
            return await _unitOfWork.QueryBySqlAsync<Finance_Incidental>(sql);
        }
        public async Task ClearIncidentals(int LadingBillId)
        {
            string sql = $" update Finance_Incidental set IsDeleted=1 where LadingBillId={LadingBillId} and IsDeleted=0";
            await _unitOfWork.ExecuteAsync(sql);
        }

        public async Task<IEnumerable<Finance_Logistics_LadingBill>> QueryByLadingBillNumbers(IEnumerable<string> ladingBillNumbers)
        {
            string sql = "select * from Finance_Logistics_LadingBill(nolock) where LadingBillNumber in @LadingBillNumbers";
            var result = await _unitOfWork.Connection.QueryAsync<Finance_Logistics_LadingBill>(sql, new { LadingBillNumbers = ladingBillNumbers }, transaction: _unitOfWork.Transaction);
            return result;
        }

        #endregion
        #region  订单成本(作废)
        //public async Task CreateCostTable(string tableName)
        //{
        //    StringBuilder sb = new StringBuilder();
        //    sb.Append($" if not exists (select * from sysobjects where name = '{tableName}')");
        //    sb.Append($"select top 1 * into {tableName} from Finance_Cost_Base where 1=2)");
        //    string sql = sb.ToString();
        //    await _unitOfWork.ExecuteAsync(sql);
        //}
        //public async Task<bool> InsertCost(IEnumerable<Finance_Cost> cost_Bases, string BillTypeCode, string oldBillTypeCode)
        //{
        //    string tableName = $"Finance_Cost_{BillTypeCode}";
        //    string sql = string.Empty;
        //    var qty = 800.0;
        //    bool result = true;
        //    try
        //    {
        //        if (oldBillTypeCode != BillTypeCode) { await CreateCostTable(tableName); }
        //        for (int i = 0; i < Math.Ceiling(cost_Bases.Count() / qty); i++)
        //        {
        //            var tempCosts = cost_Bases.Skip(i * (int)qty).Take((int)qty).ToList();
        //            List<string> SystemNos = cost_Bases.Select(x => x.SystemNo).Distinct().ToList();
        //            string SystemNoStr = string.Empty;
        //            foreach (var item in SystemNos)
        //            {
        //                SystemNoStr += $"'{item}',";
        //            }
        //            if (!string.IsNullOrEmpty(oldBillTypeCode) && oldBillTypeCode != BillTypeCode) { sql += $" delete Finance_Cost_{oldBillTypeCode} where SystemNo in ({SystemNoStr.Trim(',')}) "; } else { sql += $" delete {tableName} where SystemNo in ({SystemNoStr.Trim(',')}) "; }
        //            sql += @$"insert into  {tableName} ([SystemNo],[LadingBillNumber],[OrderCost],[CustomerId],[Length],[Width],[Height] ,[OPWeight]) values ";
        //            string values = string.Empty;
        //            foreach (var item in tempCosts)
        //            {
        //                values += $"({item.SystemNo},{item.LadingBillNumber},{item.OrderCost},{item.CustomerId},{item.Length},{item.Width},{item.Height},{item.OPWeight}),";
        //            }
        //            sql += values.Trim(',');
        //            await _unitOfWork.ExecuteAsync(sql);
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        result = false;
        //    }
        //    return result;
        //}
        #endregion

    }
}
