﻿using SMP.Model.Finance;
using XYY.Common.Standard;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
namespace SMP.Data.Finance
{
    public interface IFinance_ServiceBillResolveResultRepository : IBaseRepository<Finance_ServiceBillResolveResult>
    {
        Task<PageResult<Finance_ServiceBillResolveResult>> QueryResolveResultPage(string billName, List<string> numbers, int pageIndex, int pageLength);
        Task<List<string>> GetResolveDataNames(string billName);
        Task<List<string>> GetResolveDataNames(int resolveSolutionId);
    }

    public class Finance_ServiceBillResolveResultRepository : BaseRepository<Finance_ServiceBillResolveResult>, IFinance_ServiceBillResolveResultRepository
    {
        public Finance_ServiceBillResolveResultRepository(IUnitOfWork unitOfWork) : base(unitOfWork)
        {

        }

        public async Task<List<string>> GetResolveDataNames(string billName)
        {
            string sql = "select BillDataName from Finance_ServiceBillResolveResult where BillName=@BillName group by BillDataName";
            var names = await base._unitOfWork.Connection.QueryAsync<string>(sql, new { BillName = billName });
            return names.ToList();

        }

        public async Task<List<string>> GetResolveDataNames(int resolveSolutionId)
        {
            string sql = "select BillDataName from Finance_ServiceBillResolveResult where SolutionId=@SolutionId group by BillDataName";
            var names = await base._unitOfWork.Connection.QueryAsync<string>(sql, new { SolutionId = resolveSolutionId });
            return names.ToList();
        }

        public async Task<PageResult<Finance_ServiceBillResolveResult>> QueryResolveResultPage(string billName, List<string> numbers, int pageIndex, int pageLength)
        {

            PageResult<Finance_ServiceBillResolveResult> pageResult = new PageResult<Finance_ServiceBillResolveResult>();
            pageResult.PageIndex = pageIndex;
            pageResult.PageSize = pageLength;

            if (numbers?.Any() == true)
            {
                string sql = "select * from Finance_ServiceBillResolveResult where BillName=@BillName and Number in @Numbers";
                var result = await base._unitOfWork.Connection.QueryAsync<Finance_ServiceBillResolveResult>(sql,param: new { BillName = billName, Numbers = numbers },transaction:base._unitOfWork.Transaction);
                if (result?.Any() == true)
                {
                    pageResult.TotalCount = result.GroupBy(i => i.Number).Count();
                    pageResult.Result = result;
                    return pageResult;
                }
                else
                {
                    pageResult.TotalCount = 0;
                    pageResult.Result = new List<Finance_ServiceBillResolveResult>();
                    return pageResult;
                }
                    
            }
            else
            {
                if (pageIndex <= 0)
                    pageIndex = 1;
                if (pageLength <= 0)
                    pageLength = 20;

                if (pageLength >= 2000)
                    throw new Exception("最多支持2000条");

                int start = (pageIndex - 1) * pageLength;
                int stop = pageIndex * pageLength;
                string numberSql = @"with t as(
select ROW_NUMBER() over( order by number) rid, number from Finance_ServiceBillResolveResult where BillName=@BillName  group by number )
select number from t
where rid>=@Start and rid<=@Stop";

                var sortedNumbers = await base._unitOfWork.Connection.QueryAsync<string>(numberSql,param: new { BillName = billName, Start = start, Stop = stop },transaction:base._unitOfWork.Transaction);

                if (sortedNumbers?.Any() == true)
                {
                    string sql = "select * from Finance_ServiceBillResolveResult where BillName=@BillName and number in @Numbers";
                    var result = await base._unitOfWork.Connection.QueryAsync<Finance_ServiceBillResolveResult>(sql, param: new { BillName = billName, Numbers = sortedNumbers },transaction:base._unitOfWork.Transaction);

                    string countSql = @"with t as(
select number from Finance_ServiceBillResolveResult where BillName=@BillName group by number
)
select count(1) from t";
                    var totalCount = await base._unitOfWork.Connection.ExecuteScalarAsync<int>(countSql, param: new { BillName = billName }, transaction: base._unitOfWork.Transaction);

                    pageResult.TotalCount = totalCount;
                    pageResult.Result = result;
                    return pageResult;
                }
                else
                {
                    pageResult.TotalCount = 0;
                    pageResult.Result = new List<Finance_ServiceBillResolveResult>();
                    return pageResult;
                }
            }

        }
    }
}
