﻿using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using XYY.Common.Standard;
using SMP.Model.Dto.Reconcile;
using SMP.Model.Enums;

namespace SMP.Data.Finance
{
    public interface IViewServiceBillExpenseRepository : IViewRepository<View_ServiceBillOrderExpense>
    {
        Task<PageResult<View_ServiceBillExpense>> QueryServiceBillExpensePage(string billName,int reconcileSolutionId, ReconcileDiffType diffType, decimal diffAccurate, List<string> numbers, int pageIndex, int pageLength);
        Task<IEnumerable<ReconcileSumAmount>> GetSumAmount(string billName, int reconcileSolutionId);
    }

    public class ViewServiceBillExpenseRepository : ViewRepository<View_ServiceBillOrderExpense>, IViewServiceBillExpenseRepository
    {


        public ViewServiceBillExpenseRepository(IUnitOfWork unitOfWork) : base(unitOfWork)
        {

        }

        public  Task<IEnumerable<ReconcileSumAmount>> GetSumAmount(string billName, int mappingSolutionId)
        {
            string sql = "select BillDataName,sum(BillData) as BillSumAmount,SystemDataName as ExpenseName,sum(SystemData) as ExpenseSumAmount from View_ServiceBillExpense where BillName=@BillName and MappingSolutionId=@MappingSolutionId group by BillDataName,SystemDataName";
            var result =  base._unitOfWork.Connection.QueryAsync<ReconcileSumAmount>(sql, param: new { BillName = billName, MappingSolutionId = mappingSolutionId }, transaction: base._unitOfWork.Transaction);
            return result;
        }

        public async Task<PageResult<View_ServiceBillExpense>> QueryServiceBillExpensePage(string billName, int mappingSolutionId, ReconcileDiffType diffType, decimal diffAccurate, List<string> numbers, int pageIndex, int pageLength)
        {

            PageResult<View_ServiceBillExpense> pageResult = new PageResult<View_ServiceBillExpense>();
            pageResult.PageIndex = pageIndex;
            pageResult.PageSize = pageLength;

            string diffCondition = null;
            if(diffType==ReconcileDiffType.无差异)
            {
                diffCondition = " and abs(Diff) < @DiffAccurate ";
            }
            else if (diffType == ReconcileDiffType.有差异)
            {
                diffCondition = " and abs(Diff) >= @DiffAccurate ";
            }
            else if (diffType == ReconcileDiffType.仅正差异)
            {
                diffCondition = " and Diff >= @DiffAccurate ";
            }
            else if (diffType == ReconcileDiffType.仅负差异)
            {
                diffCondition = " and Diff <= @DiffAccurate ";
            }
            if (numbers?.Any() == true)
            {
                string sql = $"select * from View_ServiceBillExpense where BillName=@BillName and MappingSolutionId=@MappingSolutionId {diffCondition} and Number in @Numbers";
                var result = await base._unitOfWork.Connection.QueryAsync<View_ServiceBillExpense>(sql, param: new { BillName = billName, MappingSolutionId = mappingSolutionId, Numbers = numbers, DiffAccurate=diffAccurate }, transaction: base._unitOfWork.Transaction);
                if (result?.Any() == true)
                {
                    pageResult.TotalCount = result.GroupBy(i => i.Number).Count();
                    pageResult.Result = result;
                    return pageResult;
                }
                else
                {
                    pageResult.TotalCount = 0;
                    pageResult.Result = new List<View_ServiceBillExpense>();
                    return pageResult;
                }

            }
            else
            {
                if (pageIndex <= 0)
                    pageIndex = 1;
                if (pageLength <= 0)
                    pageLength = 20;

                if (pageLength >= 2000)
                    throw new Exception("最多支持2000条");

                int start = (pageIndex - 1) * pageLength;
                int stop = pageIndex * pageLength;
                string numberSql = @"with t as(
select ROW_NUMBER() over( order by number) rid, number from Finance_ServiceBillResolveResult where BillName=@BillName  group by number )
select number from t
where rid>=@Start and rid<=@Stop";

                var sortedNumbers = await base._unitOfWork.Connection.QueryAsync<string>(numberSql, param: new { BillName = billName, Start = start, Stop = stop }, transaction: base._unitOfWork.Transaction);

                if (sortedNumbers?.Any() == true)
                {
                    string sql = $"select * from View_ServiceBillExpense where BillName=@BillName and MappingSolutionId=@MappingSolutionId {diffCondition} and Number in @Numbers";
                    var result = await base._unitOfWork.Connection.QueryAsync<View_ServiceBillExpense>(sql, param: new { BillName = billName, Numbers = sortedNumbers, MappingSolutionId = mappingSolutionId, DiffAccurate = diffAccurate }, transaction: base._unitOfWork.Transaction);

                    string countSql = @$"with t as(
select Number from View_ServiceBillExpense where BillName=@BillName {diffCondition} group by Number
)
select count(1) from t";
                    var totalCount = await base._unitOfWork.Connection.ExecuteScalarAsync<int>(countSql, param: new { BillName = billName, MappingSolutionId = mappingSolutionId, DiffAccurate = diffAccurate }, transaction: base._unitOfWork.Transaction);

                    pageResult.TotalCount = totalCount;
                    pageResult.Result = result;
                    return pageResult;
                }
                else
                {
                    pageResult.TotalCount = 0;
                    pageResult.Result = new List<View_ServiceBillExpense>();
                    return pageResult;
                }
            }

        }
    }
}
