﻿using SMP.Model.Finance;
using SMP.Model.TimeTask;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using SMP.Model.Enums;
using SMP.Model.Dto.Compute;
using Newtonsoft.Json;
using System.Text;
using XYY.Common.Standard;
using SMP.Model.Dto.Finance;

namespace SMP.Data.TimeTask
{
    public interface IFinance_TimeTaskRepository : IBaseRepository<Finance_TimeTask>
    {
        Task<bool> CreateTypeTaskByPort(IEnumerable<Logistisc_Order> orders);
        Task UpdateTaskStatus(FinanceEnum Enum, List<int> Ids);
        Task ClearTypeTask(int taskId);
        Task<Finance_TimeTask> GetFinanceTimeTaskByLading(string LadingBillNumber, TaskTypeEnum taskTypeEnum);
        #region Finance_TimeTypeTaskDetail
        Task<PageResult<Finance_TimeTypeTaskDetail>> QueryDetailPage(QueryModel query);
        Task ClearTypeTaskDetail(int TaskId);
        Task<IEnumerable<Finance_TimeTypeTaskDetail>> GetTaskDetails(int taskId);
        Task<bool> CreateTypeTaskScopeDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes);
        Task<bool> InsTypeTaskDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes);
        Task<bool> CreateTypeTaskByExpense(IEnumerable<Logistisc_Order> orders, List<QueryExpenseByLadingResult> queryExpenseByLadingResults);
        #endregion
    }

    public class Finance_TimeTaskRepository : BaseRepository<Finance_TimeTask>, IFinance_TimeTaskRepository
    {
        public Finance_TimeTaskRepository(IUnitOfWork unitOfWork) : base(unitOfWork)
        {

        }

        public async Task<bool> CreateTypeTaskByPort(IEnumerable<Logistisc_Order> orders)
        {
            bool result = true; int TaskId = 0;
            if (orders == null || orders.Count() == 0) { return result; }
            string LadingBillNumber = orders.FirstOrDefault().LadingBillNumber;
            var dbTaskInfo = _unitOfWork.QueryAsync<Finance_TimeTask>(x => x.LadingBillNumber == LadingBillNumber && x.TaskType == TaskTypeEnum.DetailScope).Result.FirstOrDefault();
            if (dbTaskInfo != null)
            {
                dbTaskInfo.TaskStatus = FinanceEnum.WaitRead; TaskId = dbTaskInfo.Id;
                dbTaskInfo.RetryTotal = 0;//重置执行次数
                dbTaskInfo.RetryCount = 0;
                await _unitOfWork.UpdateAsync(dbTaskInfo);
            }
            else
            {
                Finance_TimeTask task = new Finance_TimeTask()
                {
                    StartTime = DateTime.Now,
                    EndTime = DateTime.Now,
                    OrderCount = orders.Count(),
                    LadingBillCount = 1,
                    LadingBillNumber = LadingBillNumber,
                    TaskType = TaskTypeEnum.DetailScope,
                    Remark = $"提单:[{LadingBillNumber}],存在口岸计费项，需进行末端运算，创建运算任务",
                };
                long dbId = await _unitOfWork.InsertAsync(task);
                TaskId = Convert.ToInt32(dbId);
            }
            await ClearTypeTaskDetail(TaskId);
            var typeTaskDetail = await OrderToTypeTaskDetail(TaskId, orders);
            await _unitOfWork.BulkToDBAsync<Finance_TimeTypeTaskDetail>(typeTaskDetail);
            return result;
        }
        public async Task<bool> CreateTypeTaskByExpense(IEnumerable<Logistisc_Order> orders, List<QueryExpenseByLadingResult> queryExpenseByLadingResults)
        {
            bool result = true; int TaskId = 0;
            if (orders == null || orders.Count() == 0) { return result; }
            string LadingBillNumber = orders.FirstOrDefault().LadingBillNumber;
            var dbTaskInfo = _unitOfWork.QueryAsync<Finance_TimeTask>(x => x.LadingBillNumber == LadingBillNumber && x.TaskType == TaskTypeEnum.OrderSeparately).Result.FirstOrDefault();
            List<string> expenseNames = queryExpenseByLadingResults.Select(x => x.ExpenseName).Distinct().ToList();
            if (dbTaskInfo != null)
            {
                dbTaskInfo.TaskStatus = FinanceEnum.WaitRead; TaskId = dbTaskInfo.Id;
                dbTaskInfo.RetryTotal = 0;//重置执行次数
                dbTaskInfo.RetryCount = 0;
                dbTaskInfo.Remark = $"提单:[{LadingBillNumber}],存在IOSS费用计算，如已删除费用，请忽略此任务！费用名：【{string.Join("|", expenseNames)}】";
                dbTaskInfo.ExpenseResultsJson = JsonConvert.SerializeObject(queryExpenseByLadingResults);
                await _unitOfWork.UpdateAsync(dbTaskInfo);
            }
            else
            {
                Finance_TimeTask task = new Finance_TimeTask()
                {
                    StartTime = DateTime.Now,
                    EndTime = DateTime.Now,
                    OrderCount = orders.Count(),
                    LadingBillCount = 1,
                    LadingBillNumber = LadingBillNumber,
                    TaskType = TaskTypeEnum.OrderSeparately,
                    ExpenseResultsJson = JsonConvert.SerializeObject(queryExpenseByLadingResults),
                    Remark = $"提单:[{LadingBillNumber}],存在IOSS费用计算，如已删除费用，请忽略此任务！费用名：【{string.Join("|", expenseNames)}】",
                };
                long dbId = await _unitOfWork.InsertAsync(task);
                TaskId = Convert.ToInt32(dbId);
            }
            await ClearTypeTaskDetail(TaskId);
            var typeTaskDetail = await OrderToTypeTaskDetail(TaskId, orders);
            await _unitOfWork.BulkToDBAsync<Finance_TimeTypeTaskDetail>(typeTaskDetail);
            return result;
        }
        public async Task UpdateTaskStatus(FinanceEnum Enum, List<int> Ids)
        {
            string sql = $"update Finance_TimeTask set TaskStatus={(int)Enum} where Id in ({string.Join(',', Ids)}) ";
            await _unitOfWork.ExecuteAsync(sql);
        }
        public async Task ClearTypeTask(int Id)
        {
            await _unitOfWork.ExecuteAsync($"delete Finance_TimeTypeTask where Id={Id}");
            await ClearTypeTaskDetail(Id);
        }
        public async Task<Finance_TimeTask> GetFinanceTimeTaskByLading(string LadingBillNumber, TaskTypeEnum taskTypeEnum)
        {
            var result = await QueryAsync(x => x.LadingBillNumber == LadingBillNumber && x.TaskType == taskTypeEnum);
            return result.FirstOrDefault();
        }
        #region Finance_TimeTypeTaskDetail
        public async Task<PageResult<Finance_TimeTypeTaskDetail>> QueryDetailPage(QueryModel query)
        {
            return await _unitOfWork.GetPagingListAsync<Finance_TimeTypeTaskDetail>(query);
        }
        public async Task ClearTypeTaskDetail(int TaskId)
        {
            await _unitOfWork.ExecuteAsync($"delete Finance_TimeTypeTaskDetail where TaskId={TaskId}");
        }
        private async Task<List<Finance_TimeTypeTaskDetail>> OrderToTypeTaskDetail(int TaskId, IEnumerable<Logistisc_Order> orders)
        {
            List<Finance_TimeTypeTaskDetail> result = new List<Finance_TimeTypeTaskDetail>();
            foreach (var item in orders)
            {
                result.Add(new Finance_TimeTypeTaskDetail()
                {
                    TaskId = TaskId,
                    SystemNo = item.SystemNo,
                    LadingBillNumber = item.LadingBillNumber,
                    BoxNumber = item.BoxNumber,
                    DetailStatus = FinanceEnum.Fail,
                });
            }
            return result;
        }
        public async Task<IEnumerable<Finance_TimeTypeTaskDetail>> GetTaskDetails(int taskId)
        {
            return await _unitOfWork.QueryBySqlAsync<Finance_TimeTypeTaskDetail>($"select *from Finance_TimeTypeTaskDetail(nolock) where TaskId={taskId}");
        }
        /// <summary>
        /// 末端预估错误数据记录
        /// 预估执行会分批，每次最多5K
        /// </summary>
        /// <param name="TaskId"></param>
        /// <param name="computes"></param>
        /// <returns></returns>
        public async Task<bool> CreateTypeTaskScopeDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes)
        {
            if (computes == null || computes.Count == 0) { return true; }
            var allSystemNo = computes.Select(x => x.BillNumber).Distinct().ToList();
            var errorSystemNo = computes.Where(x => !x.IsSuccess).Select(x => x.BillNumber).Distinct().ToList();
            if (string.IsNullOrEmpty(TaskInfo.LadingBillNumber))//时间拉取任务
            {
                return await TimeTaskDetail(TaskInfo, computes, allSystemNo,errorSystemNo);
            }
            else//口岸末端重算任务
            {
                return await LadingPortTaskDetail(TaskInfo, computes, allSystemNo,errorSystemNo);
            }

        }
        public async Task<bool> InsTypeTaskDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes)
        {
            if (computes == null || computes.Count == 0) { return true; }
            List<string> systemNo = computes.Select(x => x.BillNumber).Distinct().ToList();
            if (string.IsNullOrEmpty(TaskInfo.LadingBillNumber))//时间拉取任务
            {
                return await InsTimeTaskDetail(TaskInfo, systemNo);
            }
            else//重算任务（口岸末端；独立计算费项；）
            {
                return await InsLadingPortTaskDetail(TaskInfo, systemNo);
            }
        }
        #region  TypeTaskDetail private
        private async Task<bool> InsTimeTaskDetail(Finance_TimeTask TaskInfo, List<string> SystemNo)
        {
            var xml = string.Join("\r\n", SystemNo.Select(x => $"<NumberList><Number>{x}</Number></NumberList>"));
            string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_searchs') and type='U')
    drop table #temp_searchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_searchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";


            string sql = $"delete a from Finance_TimeTypeTaskDetail(nolock) a join #temp_searchs(nolock)b on a.SystemNo=b.Number where TaskId= 1 ";
            string executeSql = systemNoSql.Formater(xml) + sql;//软删除
            await _unitOfWork.ExecuteAsync(executeSql, new { SystemNo = SystemNo });
            return true;
        }
        private async Task<bool> TimeTaskDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes, List<string> allSystemNo, List<string> errorSystemNo)
        {
            List<Finance_TimeTypeTaskDetail> result = new List<Finance_TimeTypeTaskDetail>();
            await InsTimeTaskDetail(TaskInfo, allSystemNo);
            decimal batchQty = 5000;
            for (int i = 0; i < Math.Ceiling(errorSystemNo.Count() / batchQty); i++)
            {
                var tempErrorSystemNo = errorSystemNo.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                foreach (var item in tempErrorSystemNo)
                {
                    result.Add(new Finance_TimeTypeTaskDetail()
                    {
                        TaskId = TaskInfo.Id,
                        SystemNo = item,
                        DetailStatus = SMP.Model.Enums.FinanceEnum.Fail,
                        ErrorDetailJson = JsonConvert.SerializeObject(computes.Where(x => x.BillNumber == item)),
                    });
                }
                await _unitOfWork.BulkToDBAsync<Finance_TimeTypeTaskDetail>(result);
            }
            return true;
        }
        private async Task<bool> LadingPortTaskDetail(Finance_TimeTask TaskInfo, List<ComputeAmountResult> computes, List<string> allSystemNo, List<string> errorSystemNo)
        {
            await InsLadingPortTaskDetail(TaskInfo, allSystemNo);
            decimal batchQty = 50;
            //尽量都做分批处理
            for (int i = 0; i < Math.Ceiling(errorSystemNo.Count() / batchQty); i++)
            {
                var tempErrorSystemNo = errorSystemNo.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                StringBuilder sb = new StringBuilder();
                foreach (var item in errorSystemNo)
                {
                    sb.Append($" update Finance_TimeTypeTaskDetail set ErrorDetailJson='{JsonConvert.SerializeObject(computes.Where(x => x.BillNumber == item))}', DetailStatus={(int)SMP.Model.Enums.FinanceEnum.Fail} where TaskId={TaskInfo.Id} and SystemNo='{item}' ");
                }
                await _unitOfWork.ExecuteAsync(sb.ToString());
            }
            return true;
        }
        /// <summary>
        /// 还原数
        /// </summary>
        /// <returns></returns>
        private async Task<bool> InsLadingPortTaskDetail(Finance_TimeTask TaskInfo, List<string> SystemNo)
        {
            int pageLen = 2000;
            int totalCount = SystemNo.Count();
            int totalPage = (int)Math.Ceiling(totalCount / (decimal)pageLen);
            for (int i = 0; i < totalPage; i++)
            {
                var pageNumbers = SystemNo.Skip(pageLen * i).Take(pageLen);
                StringBuilder sb = new StringBuilder();
                sb.Append($" update Finance_TimeTypeTaskDetail set ErrorDetailJson='', DetailStatus={(int)SMP.Model.Enums.FinanceEnum.Succeed} where TaskId={TaskInfo.Id} and SystemNo in @SystemNo ");
                await _unitOfWork.ExecuteAsync(sb.ToString(), new { SystemNo = pageNumbers });
            }
            return true;
        }
        #endregion

        #endregion
    }
}
