﻿using Microsoft.Extensions.Caching.Distributed;
using RestSharp;
using SMP.Service.BaseInfo.Dto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SMP.Common.Extensions;

namespace SMP.Service.BaseInfo
{
    public interface IBaseInfoService
    {

        Task<List<CustomerDto>> GetAllCustomers();
        Task<List<PublicChannelDto>> GetAllPublicChannels();
        Task<List<DeliveryChannelDto>> GetAllDeliveryChannels();

        Task<List<CustomerDto>> CacheGetAllCustomers();
        Task<List<PublicChannelDto>> CacheGetAllPublicChannels();
        Task<List<DeliveryChannelDto>> CacheGetAllDeliveryChannels();

    }

    public class BaseInfoService : IBaseInfoService
    {
        private string token = "132A7468DE079C6CEB59F383A661E612";
        private string host = "http://wms.xingyunyi.cn";

        const string CustomerCacheKey = "%Cache_CustomerCache%";
        const string DeliveryChannelCacheKey = "%Cache_DeliveryChannel%";
        const string PublicChannelCacheKey = "%Cache_PublicChannel%";

        IDistributedCache _cache;

        public BaseInfoService(IDistributedCache cache)
        {
            _cache = cache;
        }

        public async Task<List<CustomerDto>> GetAllCustomers()
        {
            var resp = await Request<ApiResult<List<CustomerDto>>>("/api/BaseInfo/GetCustomers");
            return resp.Data;
        }

        public async Task<T> Request<T>(string api)
        {
            RestClient client = new RestClient(host);
            RestRequest request = new RestRequest(api);

            Parameter tokenParam = new HeaderParameter("Token", token);
            request.AddParameter(tokenParam);

            var resp = await client.GetAsync<T>(request);
            return resp;
        }

        public async Task<List<DeliveryChannelDto>> GetAllDeliveryChannels()
        {
            var resp = await Request<ApiResult<List<DeliveryChannelDto>>>("/api/BaseInfo/GetDeliverChannel");
            return resp.Data;
        }

        public async Task<List<PublicChannelDto>> GetAllPublicChannels()
        {
            var resp = await Request<ApiResult<List<PublicChannelDto>>>("/api/BaseInfo/GetAllPublic");
            return resp.Data;
        }

        public async Task<List<CustomerDto>> CacheGetAllCustomers()
        {
            var cacheData = await _cache.GetAsync<List<CustomerDto>>(CustomerCacheKey);
            if (cacheData == null)
            {
                var data = await GetAllCustomers();
                await _cache.SetAsync(CustomerCacheKey, data, options: new DistributedCacheEntryOptions { AbsoluteExpirationRelativeToNow = new TimeSpan(1, 0, 0) });
                return data;
            }
            else return cacheData;
        }

        public async Task<List<PublicChannelDto>> CacheGetAllPublicChannels()
        {
            var cacheData = await _cache.GetAsync<List<PublicChannelDto>>(PublicChannelCacheKey);
            if (cacheData == null)
            {
                var data = await GetAllPublicChannels();
                await _cache.SetAsync(PublicChannelCacheKey, data, options: new DistributedCacheEntryOptions { AbsoluteExpirationRelativeToNow = new TimeSpan(1, 0, 0) });
                return data;
            }
            else return cacheData;
        }

        public async Task<List<DeliveryChannelDto>> CacheGetAllDeliveryChannels()
        {
            var cacheData = await _cache.GetAsync<List<DeliveryChannelDto>>(DeliveryChannelCacheKey);
            if (cacheData == null)
            {
                var data = await GetAllDeliveryChannels();
                await _cache.SetAsync(DeliveryChannelCacheKey, data, options: new DistributedCacheEntryOptions { AbsoluteExpirationRelativeToNow = new TimeSpan(1, 0, 0) });
                return data;
            }
            else return cacheData;
        }
    }
}
