﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Model.Dto.Compute;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SMP.Model.Compute;
using NCalc;
using System.IO.Compression;
using System.Collections.Concurrent;
using SMP.Model.Enums;
using SMP.Common.Unit;
using XYY.Core.Standard.Mvc;
using System.Diagnostics;

namespace SMP.Service.Compute
{
    public interface IComputeAmountService
    {
        Task<List<ComputeAmountResult>> MultipleCompute(List<ComputeAmountSouce> souces);
    }

    public class ComputeAmountService : IComputeAmountService
    {
        public IDistributedCache _cache;
        public IComputeExpenseItemService ItemService;
        public ISysRateService RateService;

        public ComputeAmountService(IDistributedCache cache, IComputeExpenseItemService itemService, ISysRateService rateService)
        {
            _cache = cache;
            ItemService = itemService;
            RateService = rateService;
        }

        public async Task<List<ComputeAmountResult>> MultipleCompute(List<ComputeAmountSouce> souces)
        {
            //每次调用方法，自身做一次缓存处理，防止取缓存与解压缩所带来的性能丢失
            ConcurrentDictionary<int, Compute_ExpenseItem> dict = new ConcurrentDictionary<int, Compute_ExpenseItem>();
            ConcurrentDictionary<string, Model.Sys_Rate> rates = new ConcurrentDictionary<string, Model.Sys_Rate>();
            ConcurrentBag<ComputeAmountResult> results = new ConcurrentBag<ComputeAmountResult>();

            Parallel.ForEach(souces, async item =>
            {
                try
                {
                    if (item.ExpenseItemId == 0)
                    {
                        results.Add(new ComputeAmountResult
                        {
                            BillNumber = item.BillNumber,
                            ExpenseId = 0,
                            ExpenseItemId = item.ExpenseItemId,
                            Message = "订单无渠道，或渠道未设置计算费项，请设置后重试！",
                            IsSuccess = false,
                        });
                        return;
                    }
                    Compute_ExpenseItem expresnse = null;
                    lock (dict)
                    {
                        if (!dict.TryGetValue(item.ExpenseItemId, out expresnse))
                        {
                            expresnse = GetExpense(item.ExpenseItemId).Result;
                            dict.TryAdd(item.ExpenseItemId, expresnse);
                        }
                    }
                    if (expresnse != null && expresnse.ItemDetails != null)
                    {
                        ConcurrentDictionary<int, ComputeAmountResult> itemFees = new ConcurrentDictionary<int, ComputeAmountResult>();
                        int findex = 0;
                        foreach (var detail in expresnse.ItemDetails)
                        {
                            ComputeAmountResult result = new ComputeAmountResult
                            {
                                BillNumber = item.BillNumber,
                                ExpenseId = detail.ExpenseId,
                                ExpenseItemId = item.ExpenseItemId
                            };

                            if (expresnse.Name == "IOSS关税" && !string.IsNullOrEmpty(item.IOSS))
                            {
                                result.Message = "客户提供了IOSS，无须计费";
                                result.IsSuccess = true;
                                results.Add(result);
                                continue;
                            }
                            Compute_Expense version = null;
                            if (item.BindBillId > 0)
                            {
                                //有绑定过提单数据
                                var versionList = detail.DetailVersions.Where(x => x.BindBillId == item.BindBillId || x.VersionDate <= item.ComputeDate).OrderByDescending(x => x.VersionDate);
                                if (versionList.Count() > 0)
                                {
                                    if (versionList.Any(x => x.BindBillId == item.BindBillId))
                                    {
                                        //有绑定提单数据取提单数据
                                        version = versionList.First(x => x.BindBillId == item.BindBillId);
                                    }
                                    else
                                    {
                                        ///没有取第一个版本
                                        version = versionList.First();
                                    }
                                }
                            }
                            else
                            {
                                //非绑定提单数据
                                version =
                                    detail.DetailVersions.Where(x => x.VersionDate <= item.ComputeDate && x.BindBillId == 0).OrderByDescending(x => x.VersionDate).FirstOrDefault();
                            }
                            if (version != null && version.Details != null)
                            {
                                string key = item.ComputeDate.ToString("yyyy-MM-dd_") + version.CurrenyId;
                                Model.Sys_Rate rate = null;
                                result.ExpenseName = version.Name;
                                lock (rates)
                                {
                                    if (!rates.TryGetValue(key, out rate))
                                    {
                                        rate = RateService.QueryRate(item.ComputeDate, version.CurrenyId).Result;
                                        rates.TryAdd(key, rate);
                                    }
                                }


                                if (rate == null)
                                {
                                    result.Message = "未找到汇率";
                                    result.IsSuccess = false;
                                    results.Add(result);
                                    continue;
                                }

                                result.BillCurrency = rate.Currency;
                                result.SettlementRate = rate.ToCNY;
                                ///重量处理
                                if (item.Weight <= version.Min)
                                {
                                    item.Weight = version.Min;
                                }
                                //if (version.Stepping > 0)
                                //    item.Weight = Math.Ceiling(item.Weight / version.Stepping) * version.Stepping;
                                //

                                //取运费
                                var detailItem =
                                    version.Details.OrderBy(x => x.ToG).FirstOrDefault(x =>
                                      (item.Weight <= x.ToG && x.WeightType == Model.Enums.WeightType.运费 && x.WeightUnit != WeightUnit.M3) ||
                                      (item.M3 <= x.Weight && x.WeightType == WeightType.运费 && x.WeightUnit == WeightUnit.M3)
                                    );
                                if (detailItem != null)
                                {
                                    //取操作费
                                    var opDetailItem =
                                    version.Details.OrderBy(x => x.ToG).FirstOrDefault(x =>
                                        ((item.Weight <= x.ToG && x.WeightUnit != WeightUnit.M3) ||
                                    (item.M3 <= x.Weight && x.WeightUnit == WeightUnit.M3))
                                        && x.WeightType == Model.Enums.WeightType.操作费
                                        && x.FormulaId == detailItem.FormulaId);
                                    if (version.Zones != null)
                                    {
                                        List<Compute_Zone> mathZones = new List<Compute_Zone>();
                                        foreach (var zone in version.Zones)
                                        {
                                            bool isMath = false;
                                            List<bool> AllMath = new List<bool>();
                                            foreach (var method in zone.ZoneMethods)
                                            {
                                                bool methodMath = false;
                                                if (method.Mode == Model.Enums.ZoneMode.邮编明细列表 && item.ZipCode != null)
                                                {
                                                    //去空格，去无效字符
                                                    item.ZipCode = item.ZipCode.Trim().Replace(" ", "");
                                                    string[] mathZips = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                    int zipCodeLength = mathZips.Max(x => x.Length);
                                                    if (item.ZipCode.Length > zipCodeLength)
                                                    {
                                                        //如果是邮编明细的话，邮编长度必须固定
                                                        //邮编从左开始截取
                                                        item.ZipCode = item.ZipCode.Substring(0, zipCodeLength);
                                                    }
                                                    methodMath = mathZips.Contains(item.ZipCode);
                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.邮编起止列表 && item.ZipCode != null)
                                                {
                                                    //去空格，去无效字符
                                                    item.ZipCode = item.ZipCode.Trim().Replace(" ", "");
                                                    //获取组
                                                    string[] mathZips = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                    foreach (var mathZip in mathZips)
                                                    {
                                                        //zip3
                                                        if (mathZip.Contains('-'))
                                                        {
                                                            var m2 = mathZip.Split('-');
                                                            int zipCodeLenght = Math.Max(m2[0].Length, m2[1].Length);
                                                            item.ZipCode = item.ZipCode.Substring(0, zipCodeLenght);
                                                            if (int.TryParse(m2[0], out var start)
                                                                && int.TryParse(m2[1], out var end)
                                                                && int.TryParse(item.ZipCode, out var zip))
                                                            {
                                                                methodMath = zip >= start && zip <= end;
                                                                if (methodMath)
                                                                    break;
                                                            }
                                                        }
                                                        else
                                                        {
                                                            if (!string.IsNullOrEmpty(mathZip))
                                                            {

                                                                methodMath = item.ZipCode.Substring(0, mathZip.Length) == mathZip;
                                                                if (methodMath)
                                                                    break;
                                                            }
                                                        }
                                                    }
                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.地址正则 && item.Stree != null)
                                                {
                                                    if (method.OriginalContent == ".*" || method.OriginalContent == "*")
                                                    {
                                                        methodMath = true;
                                                    }
                                                    else
                                                    {

                                                        item.Stree = item.Stree.Trim();
                                                        System.Text.RegularExpressions.Regex regex
                                                            = new System.Text.RegularExpressions.Regex(method.OriginalContent);

                                                        methodMath = regex.IsMatch(item.Stree);
                                                    }
                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.国家列表 && item.CountryCode != null)
                                                {
                                                    item.CountryCode = item.CountryCode.Trim();
                                                    string[] mathCoutryCodes = method.OriginalContent.Replace("\r\n", "\r").Split('\r', '\n', ',', ';');
                                                    methodMath = mathCoutryCodes.Contains(item.CountryCode);

                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.跟踪号正则 && item.TrackingNumber != null)
                                                {
                                                    item.TrackingNumber = item.TrackingNumber.Trim();
                                                    System.Text.RegularExpressions.Regex regex =
                                                        new System.Text.RegularExpressions.Regex(method.OriginalContent);
                                                    methodMath = regex.IsMatch(item.TrackingNumber);
                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.API接口)
                                                {
                                                    try
                                                    {

                                                        string url = "http://localhost:7130/RemoteService";
                                                        var req = new
                                                        {
                                                            ApiName = method.OriginalContent,
                                                            ReceiveName = string.Empty,
                                                            ReceiveCountryCode = item.CountryCode,
                                                            ReceiveState = item.State,
                                                            ReceiveCity = item.City,
                                                            ReceiveZipCode = item.ZipCode,
                                                            ReceiveStree = item.Stree
                                                        };
                                                        string str = Newtonsoft.Json.JsonConvert.SerializeObject(req);
                                                        HttpContent content = new StringContent(str);
                                                        content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue("application/json");
                                                        System.Net.Http.HttpClient client = new HttpClient();

                                                        var response = await client.PostAsync(url, content);
                                                        response.EnsureSuccessStatusCode();//用来抛异常的
                                                        string responseBody = await response.Content.ReadAsStringAsync();
                                                        var r = Newtonsoft.Json.JsonConvert.DeserializeObject<ApiJsonModel<bool>>(responseBody);
                                                        methodMath = r.data;
                                                    }
                                                    catch (Exception ex)
                                                    {
                                                        Console.Write(ex.Message);
                                                        methodMath = false;
                                                    }


                                                }
                                                else if (method.Mode == Model.Enums.ZoneMode.提单落地口岸 && item.Port != null)
                                                {
                                                    item.Port = item.Port.Trim();
                                                    methodMath = item.Port == method.OriginalContent.Trim();
                                                }
                                                else
                                                {
                                                    methodMath = false;
                                                }
                                                AllMath.Add(methodMath);
                                            }
                                            //不包含匹配失败的规则
                                            isMath = !AllMath.Any(x => !x);
                                            if (isMath)
                                            {
                                                mathZones.Add(zone);
                                            }
                                        }

                                        if (mathZones.Count == 0)
                                        {
                                            result.Message = $"取明细价格失败，分区未匹配，请检查订单信息与相关费用[ExpenseItemId]{item.ExpenseItemId}设置";
                                        }
                                        else if (mathZones.Count > 1)
                                        {
                                            result.Message = $"取明细价格失败，分区存在匹配冲突{string.Join(",", mathZones.Select(x => x.Name).ToArray())}" +
                                                $"，请检查订单信息与相关费用[ExpenseItemId]{item.ExpenseItemId}设置";
                                        }
                                        else
                                        {
                                            result.IsSuccess = true;
                                            result.ExpenseVersion = version.VersionDate.Value;

                                            decimal fee = detailItem.ZoneValue.Where(x => x.ZoneId == mathZones[0].Id).FirstOrDefault()?.Fee ?? 0;
                                            result.Amount = fee;
                                            result.Currency = "CNY";
                                            result.Zones = String.Join(";", mathZones[0].Name);
                                            string Formula = detailItem.Formula.Formula;
                                            Formula = ReplaterFormula(Formula, "运费", fee.ToString());
                                            if (version.ComputeType == Model.Enums.ComputeType.运费加操作费)
                                            {
                                                if (opDetailItem == null)
                                                {
                                                    result.Message = $"当前计费模式需要操作费，当前重量找不到匹配的操作费";
                                                    continue;
                                                }
                                                decimal operFee = opDetailItem?.ZoneValue?.Where(x => x.ZoneId == mathZones[0].Id).FirstOrDefault()?.Fee ?? 0;
                                                Formula = ReplaterFormula(Formula, "操作费", operFee.ToString());

                                            }
                                            //是否(最大值(体重,称重)>首重,首重价格+向上取整((最大值(称重,体重)-首重)÷续重)×续重价格,首重价格)

                                            decimal toWeight = item.Weight;


                                            if (detailItem.WeightUnit == WeightUnit.OZ)
                                            {
                                                toWeight = item.Weight / ConvertWeightHelper.OZToG;
                                            }
                                            else if (detailItem.WeightUnit == WeightUnit.LBS)
                                            {
                                                toWeight = item.Weight / ConvertWeightHelper.LBSToG;
                                            }
                                            else if (detailItem.WeightUnit == WeightUnit.KG)
                                            {
                                                toWeight = item.Weight / ConvertWeightHelper.KGToG;
                                            }
                                            else if (detailItem.WeightUnit == WeightUnit.M3)
                                            {
                                                toWeight = item.Weight / ConvertWeightHelper.KGToG;
                                            }
                                            Formula = ReplaterFormula(Formula, "票数", item.OrderQty.ToString());
                                            Formula = ReplaterFormula(Formula, "称重", toWeight.ToString());
                                            Formula = ReplaterFormula(Formula, "立方米", item.M3.ToString());
                                            Formula = ReplaterFormula(Formula, "体积", item.Volume.ToString());
                                            Formula = ReplaterFormula(Formula, "步进", version.Stepping.ToString());
                                            Formula = ReplaterFormula(Formula, "取整", "Ceiling");
                                            Formula = ReplaterFormula(Formula, "四舍五入", "Round");
                                            Formula = ReplaterFormula(Formula, "是否", "if");
                                            Formula = ReplaterFormula(Formula, "取大值", "Max");
                                            Formula = ReplaterFormula(Formula, "最大值", "Max");
                                            Formula = ReplaterFormula(Formula, "申报价值", item.GoodsDeclareValue.ToString());
                                            Formula = ReplaterFormula(Formula, "÷", "/");
                                            Formula = ReplaterFormula(Formula, "×", "*");
                                            Formula = ReplaterFormula(Formula, "+", "+");
                                            Formula = ReplaterFormula(Formula, "≥", ">=");
                                            Formula = ReplaterFormula(Formula, "≤", "<=");
                                            Formula = ReplaterFormula(Formula, "。", ".");
                                            Formula = ReplaterFormula(Formula, "０", "0");
                                            Formula = ReplaterFormula(Formula, "３", "3");
                                            Formula = ReplaterFormula(Formula, "２", "2");
                                            Formula = ReplaterFormula(Formula, "１", "1");
                                            Formula = ReplaterFormula(Formula, "６", "6");
                                            Formula = ReplaterFormula(Formula, "５", "5");
                                            Formula = ReplaterFormula(Formula, "４", "4");
                                            Formula = ReplaterFormula(Formula, "９", "9");
                                            Formula = ReplaterFormula(Formula, "８", "8");
                                            Formula = ReplaterFormula(Formula, "７", "7");
                                            for (int i = 0; i < itemFees.Count; i++)
                                            {
                                                Formula = ReplaterFormula(Formula, $"第{i}步", itemFees[i].Amount.ToString());
                                            }
                                            if (itemFees.Count > 0)
                                            {
                                                Formula = ReplaterFormula(Formula, $"上一步", itemFees[findex - 1].AmountinOriginalCurrency.ToString());
                                            }
                                            if (Formula == "0")
                                                Formula = "0.000";
                                            Expression expression = new Expression(Formula);
                                            Trace.WriteLine(Formula);
                                            try
                                            {
                                                result.AmountinOriginalCurrency = Convert.ToDecimal(expression.Evaluate().ToString());

                                            }
                                            catch (Exception ex)
                                            {
                                                result.Message = $"公式执行失败{Formula}";
                                                result.IsSuccess = false;
                                            }
                                            result.Amount = Math.Round(result.AmountinOriginalCurrency * rate.ToCNY, 2, MidpointRounding.AwayFromZero);
                                            result.VersionId = version.VersionId ?? 0;
                                            result.DetailId = version.Id;
                                            result.ExpenseName = version.Name;
                                            result.ApportionMethod = version.ApportionMethod;
                                        }
                                    }
                                    else
                                    {
                                        result.Message = "按重量查找，未找到相关的有效计费版本";
                                    }
                                }
                                else
                                {
                                    result.Message = "按重量查找，未找到相关的有效计费区间";
                                }
                            }
                            else
                            {
                                result.Message = $"按时间({item.ComputeDate.ToString_yyyyMMdd()})查找，未找到相关的有效计费版本";
                            }
                            itemFees.TryAdd(findex, result);
                            findex++;
                            results.Add(result);
                        }
                    }
                }
                catch (Exception ex)
                {
                    results.Add(new ComputeAmountResult
                    {
                        BillNumber = item.BillNumber,
                        Message = ex.Message,
                        IsSuccess = false
                    });
                }
            });
            return results.ToList();
        }

        private static string ReplaterFormula(string Formula, string name, string fee)
        {
            return Formula.Replace(name, fee);
        }

        public string @computeItemKey => "expense#";
        private async Task<Compute_ExpenseItem> GetExpense(int ComputeItemId)
        {
            string key = @computeItemKey + ComputeItemId;
            var cacheV = _cache.Get(key);
            if (cacheV == null)
            {
                //缓存压缩
                var expenseItem = await ItemService.GetExpenseItem(ComputeItemId);
                string josnValue = Newtonsoft.Json.JsonConvert.SerializeObject(expenseItem);
                //var bytes = System.Text.UTF8Encoding.UTF8.GetBytes(josnValue);
                //using var originalFileStream = new MemoryStream(bytes);
                //using var compressedFileStream = new MemoryStream();
                //using BrotliStream compressor = new BrotliStream(compressedFileStream, CompressionLevel.Fastest);
                //originalFileStream.CopyTo(compressor);

                //var compressedFileArray = compressedFileStream.ToArray();

                _cache.SetString(key, josnValue);
                return expenseItem;
            }
            else
            {
                //缓存解压
                //using var originalFileStream = new MemoryStream(cacheV);
                //using var outputFileStream = new MemoryStream();
                //using var decompressor = new BrotliStream(originalFileStream, CompressionMode.Decompress);
                //decompressor.CopyTo(outputFileStream);

                //var decompressorArray = outputFileStream.ToArray();
                string orijson = System.Text.UTF8Encoding.UTF8.GetString(cacheV);
                if (!orijson.EndsWith("}"))
                    orijson = orijson + "}";
                return Newtonsoft.Json.JsonConvert.DeserializeObject<Compute_ExpenseItem>(orijson);
            }
        }
    }
}
