﻿using SMP.Data.Compute;
using SMP.Model.Compute;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Compute
{
    public interface IComputeExpenseItemTemplateService
    {
        Task<PageResult<Compute_ExpenseItemTemplate>> QueryComputeExpenseItemTemplate(QueryModel queryModel);
        Task UpdateExpenseItemTemplate(Compute_ExpenseItemTemplate Country);
        Task InsertExpenseItemTemplate(Compute_ExpenseItemTemplate Country);
        Task DeleteExpenseItemTemplate(Compute_ExpenseItemTemplate Country);
        Task<IEnumerable<Compute_ExpenseItemTemplate>> GetComputeExpenseItemTemplate();

        Task<Compute_ExpenseItemTemplate> Get(int TemplateId);
    }

    public class ComputeExpenseItemTemplateService : IComputeExpenseItemTemplateService
    {
        private ICompute_ExpenseItemTemplateRepository _expenseItemTemplateRepository;

        private ICompute_ExpenseItemTemplateDetailRepository _expenseItemTemplateDetailRepository;

        public ComputeExpenseItemTemplateService(ICompute_ExpenseItemTemplateRepository expenseItemTemplateRepository, ICompute_ExpenseItemTemplateDetailRepository expenseItemTemplateDetailRepository)
        {
            _expenseItemTemplateRepository = expenseItemTemplateRepository;
            _expenseItemTemplateDetailRepository = expenseItemTemplateDetailRepository;
        }


        public async Task DeleteExpenseItemTemplate(Compute_ExpenseItemTemplate Country)
        {
            await _expenseItemTemplateRepository.UpdateByConditionAsync(x => x.Id == Country.Id, new { IsDelete = true });
        }

        public async Task<Compute_ExpenseItemTemplate> Get(int TemplateId)
        {
            var baseInfo = await _expenseItemTemplateRepository.GetAsync(TemplateId);
            baseInfo.Details = (await _expenseItemTemplateDetailRepository.QueryAsync(x => x.TemplateId == TemplateId && x.IsDelete == false)).ToList();
            return baseInfo;
        }

        public async Task<IEnumerable<Compute_ExpenseItemTemplate>> GetComputeExpenseItemTemplate()
        {
            return await _expenseItemTemplateRepository.QueryAsync();
        }


        public async Task InsertExpenseItemTemplate(Compute_ExpenseItemTemplate template)
        { 
            template.Qty = template.Details.Count;
            var Id = (int)await (_expenseItemTemplateRepository.InsertAsync(template));
            template.Details.ForEach(x =>
            {
                x.TemplateId = Id;
                x.CreateTime = DateTime.Now;
            });  

            await _expenseItemTemplateDetailRepository.BulkToDBAsync(template.Details);
        }

        public async Task<PageResult<Compute_ExpenseItemTemplate>> QueryComputeExpenseItemTemplate(QueryModel queryModel)
        {
            if (queryModel.QueryParamer == null)
                queryModel.QueryParamer = new List<QueryParamer>();
            queryModel.QueryParamer.Add(new QueryParamer { Filed = "IsDelete", Method = "Eq", Value = "0" });
            return await _expenseItemTemplateRepository.QueryPagerAsync(queryModel);
        }

        public async Task UpdateExpenseItemTemplate(Compute_ExpenseItemTemplate template)
        {
            //清理之前的数据
            await _expenseItemTemplateDetailRepository.UpdateByConditionAsync(x => x.TemplateId == template.Id, new
            {
                IsDelete = true
            });

            template.Qty = template.Details.Count;
            await _expenseItemTemplateRepository.UpdateAsync(template);
            template.Details.ForEach(x =>
            {
                x.TemplateId = template.Id;
                x.CreateTime = DateTime.Now;
            });

            await _expenseItemTemplateDetailRepository.BulkToDBAsync(template.Details);
        }
    }
}
