﻿using SMP.Data.Compute;
using SMP.Model.Compute;
using XYY.Common.Standard;

namespace SMP.Service.Compute
{
    public interface IComputeFormulaService
    {

        Task<PageResult<Compute_Formula>> QueryComputeFormula(QueryModel queryModel);
        Task UpdateComputeFormula(Compute_Formula computeFormula);
        Task InsertComputeFormula(Compute_Formula computeFormula);
        Task DeleteComputeFormula(Compute_Formula computeFormula);
        Task<List<Compute_Formula>> GetAll();
    }

    public class ComputeFormulaService : IComputeFormulaService
    {
        private readonly ICompute_FormulaRepository formulaRepository;
        private readonly ICompute_ExpenseDetailRepository detailRepository;
        private readonly IComputeExpenseService computeExpenseService;

        public ComputeFormulaService(ICompute_FormulaRepository formulaRepository, ICompute_ExpenseDetailRepository detailRepository, IComputeExpenseService computeExpenseService)
        {
            this.formulaRepository = formulaRepository;
            this.detailRepository = detailRepository;
            this.computeExpenseService = computeExpenseService;
        }

        public async Task DeleteComputeFormula(Compute_Formula formula)
        {
            await this.formulaRepository.DeleteAsync(formula);
        }

        public async Task<List<Compute_Formula>> GetAll()
        {
            return (await this.formulaRepository.QueryAsync()).ToList();
        }

        public async Task InsertComputeFormula(Compute_Formula formula)
        {
            await this.formulaRepository.InsertAsync(formula);

        }

        public async Task<PageResult<Compute_Formula>> QueryComputeFormula(QueryModel formula)
        {
            return await this.formulaRepository.QueryPagerAsync(formula);
        }

        public async Task UpdateComputeFormula(Compute_Formula computeFormula)
        {
            await this.formulaRepository.UpdateAsync(computeFormula);

            var expenseDetails = await this.detailRepository.QueryAsync(x => x.FormulaId == computeFormula.Id);
            int[] details = expenseDetails.Select(x => x.ExpenseID).ToArray();
            await computeExpenseService.RefreshItemCache(details);
        }
    }
}
