﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Data.Compute;
using SMP.Model.Compute;
using XYY.Common.Standard;

namespace SMP.Service.Compute
{
    public interface IComputeZoneService
    {
        Task<IEnumerable<Compute_Zone>> QueryComputeZone();
        Task<PageResult<Compute_Zone>> QueryComputeZone(QueryModel queryModel);
        Task UpdateComputeZone(Compute_Zone computeZone);
        Task InsertComputeZone(Compute_Zone computeZone);
        Task DeleteComputeZone(Compute_Zone computeZone);
        Task<Compute_Zone> GetDetails(int ZoneId);
    }

    public class ComputeZoneService : IComputeZoneService
    {
        private readonly ICompute_ZoneRepository ZoneRepository;
        private readonly ICompute_ZoneMethodRepository MethodRepository;
        private readonly IDistributedCache _cache;
        public string @computeItemKey => "expense#";

        public void RefreshCache(int itemId)
        {
            this._cache.Remove(@computeItemKey + itemId.ToString());
        }

        public ComputeZoneService(
            ICompute_ZoneRepository ZoneRepository,
            ICompute_ZoneMethodRepository MethodRepository, IDistributedCache cache)
        {
            this.ZoneRepository = ZoneRepository;
            this.MethodRepository = MethodRepository;
            _cache = cache;
        }

        public async Task DeleteComputeZone(Compute_Zone Zone)
        {
            await this.ZoneRepository.DeleteAsync(Zone);
        }

        public async Task<Compute_Zone> GetDetails(int ZoneId)
        {
            var cz = await this.ZoneRepository.GetAsync(ZoneId);
            cz.ZoneMethods =
                (await this.MethodRepository.QueryAsync(x => x.ZoneId == ZoneId)).ToList();

            return cz;
        }

        public async Task InsertComputeZone(Compute_Zone Zone)
        {
            var id = (int)await this.ZoneRepository.InsertAsync(Zone);
            if (Zone.ZoneMethods == null || Zone.ZoneMethods.Count == 0)
                throw new Exception("至少传入一个分区类型");

            Zone.ZoneMethods.ForEach(x =>
            {
                x.ZoneId = id;
                x.CreateTime = DateTime.Now;
            });
            //foreach(var item in Zone.ZoneMethods)
            //{
            //    await this.MethodRepository.InsertAsync(item);
            //}
            await this.MethodRepository.BulkToDBAsync(Zone.ZoneMethods);
            //ClearZoneCache(Zone);

        }

        public async Task<PageResult<Compute_Zone>> QueryComputeZone(QueryModel Zone)
        {
            return await this.ZoneRepository.QueryPagerAsync(Zone);
        }

        public async Task<IEnumerable<Compute_Zone>> QueryComputeZone()
        {
            return await this.ZoneRepository.QueryAsync();
        }

        public async Task UpdateComputeZone(Compute_Zone Zone)
        {
            await this.ZoneRepository.UpdateAsync(Zone);

            await this.MethodRepository.DeleteAsync(x => x.ZoneId == Zone.Id);
            if (Zone.ZoneMethods == null || Zone.ZoneMethods.Count == 0)
                throw new Exception("至少传入一个分区类型");

            Zone.ZoneMethods.ForEach(x =>
            {
                x.ZoneId = Zone.Id;
                x.CreateTime = DateTime.Now;
            });

            await this.MethodRepository.BulkToDBAsync(Zone.ZoneMethods);
            ClearZoneCache(Zone);

        }

        public async Task ClearZoneCache(Compute_Zone Zone)
        { 
            //清理关联线路的计费缓存
            string sql = @"select distinct b.ItemId  from Compute_ExpenseZone(nolock) a join Compute_ExpenseItemDetail(nolock)b on a.ExpenseId=b.ExpenseId   where ZoneId = 51 and a.IsDelete=0  ";
            var expressIds = await this.ZoneRepository.QueryAsync<int>(sql);
            foreach (var expressId in expressIds)
            {
                RefreshCache(expressId);
            }
        }
    }
}
