﻿using SMP.Data.Finance;
using SMP.Model.Dto.DataInterconnection;
using SMP.Model.Dto.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.Compute;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using SMP.Model.TimeTask;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface IFinance_Logistics_LadingBillService
    {
        #region 帐单
        Task<PageResult<Finance_Logistics_LadingBill>> QueryPage(QueryModel query);
        Task<long> Create(LadingBillDto dto);
        Task Edit(LadingBillDto dto);
        Task<long> ScopeBillCreate(int OrderQuerySettingId, string BillName);
        Task ScopeBillEdit(LadingBillDto dto);
        Task RemoveLadingBill(int LadingBillId);
        Task RemoveScopeBill(int LadingBillId);
        Task<long> CustomerLadingBillCreate(LadingBillDto dto);
        Task CustomerLadingBillEdit(LadingBillDto dto);
        Task RemoveCustomerLadingBill(int LadingBillId);
        Task<Finance_Logistics_LadingBill> InsBill(int Id);
        Task<bool> IsExistsAsync(string LadingBillNumber, int OrderQuerySettingId = 0);
        Task ScopeBillEditByScop(string name, int OrderQuerySettingId);
        /// <summary>
        /// 批量重算提单
        /// </summary>
        /// <returns></returns>
        Task<bool> BatchRecompute();
        Task<bool> BatchRecomputeSeparateOrders(int startId, int endId);
        #endregion

        #region 帐单费项
        Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId);
        #endregion

    }
    #endregion
    #region services 
    public class Finance_Logistics_LadingBillService : IFinance_Logistics_LadingBillService
    {
        private readonly IUnitOfWork _unitOfWork;
        IFinance_Logistics_LadingBillRepository _repository;
        private readonly IWMSDataInterconnectionService _wMSDataInterconnection;
        private readonly IOrderCostService _orderCostService;
        private readonly IComputeExpenseService _computeExpenseService;
        public Finance_Logistics_LadingBillService(IFinance_Logistics_LadingBillRepository repository, IWMSDataInterconnectionService wMSDataInterconnection, IOrderCostService orderCostService
            , IComputeExpenseService computeExpenseService
            , IUnitOfWork unitOfWork)
        {
            _repository = repository;
            this._wMSDataInterconnection = wMSDataInterconnection;
            this._orderCostService = orderCostService;
            this._computeExpenseService = computeExpenseService;
            _unitOfWork = unitOfWork;
        }

        public async Task<PageResult<Finance_Logistics_LadingBill>> QueryPage(QueryModel query)
        {
            LadingBillDto returnResult = new LadingBillDto();
            if (query.QueryParamer == null) { query.QueryParamer = new List<QueryParamer>(); }
            query.QueryParamer.Add(new QueryParamer() { Filed = "IsDelete", Value = "0", Method = "Eq" });

            var result = await _repository.QueryPager2Async(query);

            return result;
        }
        public async Task<Finance_Logistics_LadingBill> InsBill(int Id)
        {
            var result = await _repository.QueryAsync(x => x.Id == Id && x.IsDelete == false);
            return result.FirstOrDefault();
        }
        public async Task<bool> IsExistsAsync(string LadingBillNumber, int OrderQuerySettingId = 0)
        {
            if (OrderQuerySettingId > 0) { return await _repository.IsExistsAsync(x => x.LadingBillNumber == LadingBillNumber && x.OrderQuerySettingId != OrderQuerySettingId && x.IsDelete == false); }
            else { return await _repository.IsExistsAsync(x => x.LadingBillNumber == LadingBillNumber && x.IsDelete == false); }

        }
        #region 人工干预，进行重算
        #region 头程
        public async Task<bool> BatchRecompute()
        {
            string sql = @"  select * from Finance_Logistics_LadingBill where  IsDelete=0 and Id in (select LadingBillId from Finance_Incidental where ExpenseId=129 and IsDeleted=0) ";// LadingBillNumber='784-34070455'
            var ladingBills = await _repository.QueryBySqlAsync(sql);
            foreach (var item in ladingBills)
            {
                var orders = await _orderCostService.GetOrdersByLadingBillNumber(item.LadingBillNumber);
                var finances = await InsLadingBillPageExpense(item.LadingBillNumber, item.Id);
                await _repository.ClearIncidentals(item.Id);
                IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = item.Id, LadingBillNumber = item.LadingBillNumber, IncidentalFees = finances, Weight = ((item.ServicesWeight ?? 0) * 1000) > 0 ? ((item.ServicesWeight ?? 0) * 1000) : item.Weight, Pieces = item.Pieces ?? 0, orders = orders, BillTime = item.BillTime, Currency = item.BillCurrency, _Destination = item._Destination, M3 = (item.M3 ?? 0.0M) };

                var tuple = await _orderCostService.AddIncidentals(compute);
                item.Cost = tuple.Item1;
                await _repository.UpdateAsync(item);
                //计算订单均摊成本
                await _orderCostService.ApportionCompute(item.LadingBillNumber, item.LadingBillNumber, orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.LadingBill);
            }
            return true;
        }
        private async Task<IEnumerable<finances>> InsLadingBillPageExpense(string? LadingBillNumber, int? BindBillId)
        {
            List<finances> finances = new List<finances>();
            if (string.IsNullOrEmpty(LadingBillNumber)) { return finances; }
            QueryExpenseByLadingInput input = new QueryExpenseByLadingInput() { LadingBillNumber = LadingBillNumber, BindBillId = BindBillId ?? 0 };
            var result = await _computeExpenseService.GetExpenseWithVersionByLading(input);
            if (result != null && result.Count() > 0)
            {
                var itemIds = result.Select(x => x.ExpenseItemId).Distinct().ToList();
                foreach (var item in itemIds)
                {
                    finances finance = new finances();
                    finance.IncidentalType = item;
                    finance.ExpenseResult = result.Where(x => x.ExpenseItemId == item);
                    finances.Add(finance);
                }
            }
            return finances;
        }
        #endregion
        #region 末端
        public async Task<bool> BatchRecomputeSeparateOrders(int startId,int endId)
        {
            //int startId = 0; int endId = 10000;
            //选取一个任务，存放异常信息   默认使用ID:179
            string taskSql = "select *from Finance_TimeTask(nolock) where Id=179";
            var taskInfo = await _unitOfWork.QueryBySqlAsync<Finance_TimeTask>(taskSql);
            var tempTaskInfo = taskInfo.FirstOrDefault();
            try
            {
                Console.WriteLine($"开始取数：起始ID：{startId}，截止ID：{endId}，取数时间：{DateTime.Now}");
                string ordersSql = @$"select top 100000 a.* from Logistisc_Order(nolock) a 
left join Logistics_Channel(nolock) b on b.ThirdPartyUniqueId=a.OrderChannelUniqueId
where 0!=ISNULL(b.ExpenseItemId,0) and a.Id>={startId} and a.Id<{endId} order by a.Id ";

                ordersSql = @$" 
   select  top 40000 b.* from Logistics_Channel(nolock) a join Logistisc_Order(nolock)b
                                        on a.ThirdPartyUniqueId = b.OrderChannelUniqueId		 
										 left join Compute_ExpenseAmount(nolock) c on c.Number=b.SystemNo and c.IsDelete=0 and c.TransactionTime>'2022-06-16 18:00' and c.BillTypeCode='SystemEndBill'	
										 join Finance_Logistics_LadingBill(nolock)d on d.LadingBillNumber=b.LadingBillNumber
                                        where     b.ReceiveTime>'2022-06-01' and b.ReceiveTime<'2022-06-16' and c.Id is   null and a.ExpenseItemId is not null
										and ThirdPartyUniqueId in 
										(
222,
221,
340,
223,
216,
210,
218,
219,
211,
217)";

                var dbOrders = await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(ordersSql);
                var orders = dbOrders.ToList();

                if (orders.Count > 0)
                {
                    await _orderCostService.EestimateOrderEndCost(tempTaskInfo, orders);
                }
            }
            catch (Exception ex)
            {
                //消化异常，不做处理
                return false;
            }
            return true;
        }
        #endregion
        #endregion
        #region 自飞头程帐单增改删

        public async Task<long> Create(LadingBillDto dto)
        {
            if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber && x.IsDelete == false))
            {
                throw new Exception("已存在相同类型帐单" + dto.LadingBillNumber + "不允许创建新的同类型帐单");
            }
            ThirdPartyData_Input input = new ThirdPartyData_Input() { LadingBillNumber = dto.LadingBillNumber };
            var ThirdPartyLadingBillDto = await _wMSDataInterconnection.PullOrderInfo(input);
            if (ThirdPartyLadingBillDto == null || ThirdPartyLadingBillDto?.orders?.FirstOrDefault()?.LadingBillNumber == null) { throw new Exception("提单数据不存在，请确认提单号是否正确！"); }
            await _orderCostService.SyncOrderInfo(ThirdPartyLadingBillDto.orders);
            int BoxQty = ThirdPartyLadingBillDto.orders.Where(x => !string.IsNullOrEmpty(x.BoxNumber)).Select(x => x.BoxNumber).Distinct().Count();
            dto.BoxQty = BoxQty;
            dto.Pieces = ThirdPartyLadingBillDto.orders.Count();
            dto.ChargeWeight = dto.Weight = ThirdPartyLadingBillDto.TotalWeight ?? 0;
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            int Id = (int)await _repository.InsertAsync(db);

            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = ((dto?.ServicesWeight ?? 0) * 1000) > 0 ? ((dto?.ServicesWeight ?? 0) * 1000) : dto.Weight, Pieces = dto.Pieces ?? 0, orders = ThirdPartyLadingBillDto.orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            await _repository.UpdateByPrimaryKeyAsync(Id, "Cost", tuple.Item1);
            //计算订单均摊成本
            await _orderCostService.ApportionCompute("",
                dto.LadingBillNumber, ThirdPartyLadingBillDto.orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.LadingBill);

            return Id;
        }

        public async Task Edit(LadingBillDto dto)
        {
            if (dto.Prepaid > 0) { throw new Exception("帐单已付款不可修改！"); }
            var oldDb = await _repository.GetAsync(dto.Id);
            IEnumerable<Logistisc_Order> orders = null;
            dto.ChargeWeight = dto.Weight = dto.Weight * 1000;//页面默认kg
            if (oldDb.LadingBillNumber != dto.LadingBillNumber)
            {
                if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber && x.IsDelete == false && x.Id != dto.Id))
                {
                    throw new Exception("提单号重复，请确认填写是否正确！");
                }
                ThirdPartyData_Input input = new ThirdPartyData_Input() { LadingBillNumber = dto.LadingBillNumber };
                var ThirdPartyLadingBillDto = await _wMSDataInterconnection.PullOrderInfo(input);
                if (ThirdPartyLadingBillDto == null || ThirdPartyLadingBillDto?.orders?.FirstOrDefault()?.LadingBillNumber == null) { throw new Exception("提单数据不存在，请确认提单号是否正确！"); }
                int BoxQty = ThirdPartyLadingBillDto.orders.Where(x => !string.IsNullOrEmpty(x.BoxNumber)).Select(x => x.BoxNumber).Distinct().Count();
                dto.BoxQty = BoxQty;
                dto.Pieces = ThirdPartyLadingBillDto.orders.Count();
                dto.ChargeWeight = dto.Weight = ThirdPartyLadingBillDto.TotalWeight ?? 0;
                orders = ThirdPartyLadingBillDto.orders;
                await _orderCostService.SyncOrderInfo(ThirdPartyLadingBillDto.orders);
            }
            else
            {
                orders = await _orderCostService.GetOrdersByLadingBillNumber(dto.LadingBillNumber);
            }
            await _repository.ClearIncidentals(dto.Id);
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            var serviceWeight = ((dto?.ServicesWeight ?? 0) * 1000);
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = ((dto?.ServicesWeight ?? 0) * 1000) > 0 ? ((dto?.ServicesWeight ?? 0) * 1000) : dto.Weight, Pieces = dto.Pieces ?? 0, orders = orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            db.Cost = tuple.Item1;
            await _repository.UpdateAsync(db);
            //计算订单均摊成本
            await _orderCostService.ApportionCompute(oldDb.LadingBillNumber, dto.LadingBillNumber, orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.LadingBill);
        }
        public async Task RemoveLadingBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($@" update a set IsDelete=1 
from Compute_ExpenseAmount(nolock) a
left join Logistisc_Order(nolock) b on b.SystemNo=a.Number
left join Finance_Logistics_LadingBill(nolock) c on c.LadingBillNumber=b.LadingBillNumber
where a.BillTypeCode=c.BillTypeCode and a.SystemBillType=c.SystemBillType and c.Id={LadingBillId} and a.CostType={(int)CostType.预估成本}  ");
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }

        #endregion
        #region  帐单费项
        public async Task<IEnumerable<Finance_Incidental>> GetIncidentals(int LadingBillId)
        {
            return await _repository.GetIncidentals(LadingBillId);
        }
        #endregion
        #region 范围帐单 增改删
        public async Task<long> ScopeBillCreate(int OrderQuerySettingId, string BillName)
        {
            //获取执行数据
            var orders = await _orderCostService.GetOrdersByScopeBill(OrderQuerySettingId: OrderQuerySettingId);
            //简单创建，不做任何特殊处理
            decimal weight = orders.Sum(x => x.OPWeight);
            Finance_Logistics_LadingBill toDb = new Finance_Logistics_LadingBill()
            {
                LadingBillNumber = BillName,
                BoxQty = orders?.Where(x => !string.IsNullOrEmpty(x.BoxNumber))?.Select(x => x.BoxNumber)?.Distinct()?.Count() ?? 0,
                Pieces = orders?.Count() ?? 0,
                ChargeWeight = weight,
                Weight = weight,
                OrderQuerySettingId = OrderQuerySettingId,
                BillTime = DateTime.Now,
                BillTypeCode = "",
                SystemBillType = Model.Enums.SystemBillTypeEnum.ScopeBill,
            };
            return await _repository.InsertAsync(toDb);
        }
        public async Task ScopeBillEdit(LadingBillDto dto)
        {
            var oldDb = await _repository.GetAsync(dto.Id);
            //获取执行数据
            var orders = await _orderCostService.GetOrdersByScopeBill(dto.LadingBillNumber);
            await _repository.ClearIncidentals(dto.Id);
            dto.BoxQty = orders?.Where(x => !string.IsNullOrEmpty(x.BoxNumber))?.Select(x => x.BoxNumber)?.Distinct()?.Count() ?? 0;
            dto.Pieces = orders?.Count() ?? 0;
            decimal weight = orders?.Sum(x => x.OPWeight) ?? 0;
            dto.ChargeWeight = dto.Weight = weight;
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            decimal costValue = 0;
            //提前做费用版本验证
            dto.finances = await _orderCostService.FinancesVerify(oldDb.Id,dto.finances);
            //验证是否有相同费项
            var tupleVerify = await _orderCostService.ScopeBillVerify(oldDb.OrderQuerySettingId,oldDb.Id,dto.finances);
            //外部清除数据
            if (!await _orderCostService.ClearApportionComputeRecord(new List<string>() { oldDb.LadingBillNumber }, (int)CostType.预估成本)) { throw new Exception("范围帐单，旧数据清除失败，请重新保存！"); }
            //计算基础数据
            var financesList = dto.finances.ToList();
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = financesList, Weight = ((dto?.ServicesWeight ?? 0) * 1000) > 0 ? ((dto?.ServicesWeight ?? 0) * 1000) : dto.Weight, Pieces = dto.Pieces ?? 0, orders = orders, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            if (tupleVerify.Item1)
            {
                var sameExpenseItemIds = tupleVerify.Item2.Select(x=>x.ExpenseItemId).Distinct().ToList();
                var sameFinances = dto.finances.Where(x => sameExpenseItemIds.Contains(x.IncidentalType));
                financesList.RemoveAll(x=>sameExpenseItemIds.Contains(x.IncidentalType));//移除非正常流程费项数据
                //更改费项
                compute.IncidentalFees = sameFinances;
                  var tuple = await _orderCostService.CombinedComputeByScopeBill(compute, tupleVerify.Item2);
                //计算订单均摊成本
                await _orderCostService.ApportionComputeByScopeBill(dto.LadingBillNumber,compute, tuple.Item1, tuple.Item2, tupleVerify.Item2);
            }
            //正常费项流程
            if (financesList != null&& financesList.Count()>0) 
            {
                var tuple = await _orderCostService.AddIncidentals(compute);
                costValue += tuple.Item1;
                //计算订单均摊成本
                await _orderCostService.ApportionCompute(oldDb.LadingBillNumber, dto.LadingBillNumber, orders, tuple.Item2, Model.Enums.SystemBillTypeEnum.ScopeBill);
                db.Cost = costValue;
            }
            await _repository.UpdateAsync(db);
            if (tupleVerify.Item1) {
                //更新后，涉及的提单全部重算总额
                List<int> ladingBillIds = tupleVerify.Item2.SelectMany(x => x.LadingBillIds).Distinct().ToList();
                ladingBillIds.Add(oldDb.Id);
                foreach (var item in ladingBillIds)
                {
                    string sql = @$"	 update Finance_Logistics_LadingBill set Cost=a.cost
from ( select SUM(b.SettlementAmount) cost from Finance_Logistics_LadingBill(nolock) a
left join Compute_ExpenseAmount(nolock) b on b.LadingBillNumber=a.LadingBillNumber
where a.Id={item} and b.IsDelete=0 and a.IsDelete=0
) a
where Id={item} ";
                    await _unitOfWork.ExecuteAsync(sql);
                }
            }
        }
        public async Task ScopeBillEditByScop(string name, int OrderQuerySettingId)
        {
            var result = await _repository.QueryAsync(x => x.OrderQuerySettingId == OrderQuerySettingId);
            if (result.FirstOrDefault().LadingBillNumber != name)
            {
                await _orderCostService.ClearApportionComputeRecord(new List<string>() { result.FirstOrDefault().LadingBillNumber }, (int)CostType.预估成本);
            }
            await _repository.UpdateByPrimaryKeyAsync(result.FirstOrDefault().Id, "LadingBillNumber", name);
        }
        public async Task RemoveScopeBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($@" update a set IsDelete=1 
from Compute_ExpenseAmount(nolock) a
left join Finance_OrderRangeSearchResult(nolock) b1 on b1.SystemNo=a.Number
left join Finance_Logistics_LadingBill(nolock) c on c.OrderQuerySettingId=b1.RangeSettingId
where a.BillTypeCode=c.BillTypeCode and a.SystemBillType=c.SystemBillType and c.Id={LadingBillId} and a.CostType={(int)CostType.预估成本}  ");
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            sb.Append($" update Finance_OrderQuerySetting set Deleted=1 where Id=(select OrderQuerySettingId from Finance_Logistics_LadingBill where Id={LadingBillId})");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }
        #endregion
        #region 客户头程帐单增删改
        public async Task<long> CustomerLadingBillCreate(LadingBillDto dto)
        {
            if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber && x.IsDelete == false))
            {
                throw new Exception("已存在相同类型帐单" + dto.LadingBillNumber + "不允许创建新的同类型帐单");
            }
            dto.ChargeWeight = dto.Weight = dto.Weight * 1000;//页面录入默认kg
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            int Id = (int)await _repository.InsertAsync(db);
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = ((dto?.ServicesWeight ?? 0) * 1000) > 0 ? ((dto?.ServicesWeight ?? 0) * 1000) : dto.Weight, Pieces = dto.Pieces ?? 0, orders = null, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            await _repository.UpdateByPrimaryKeyAsync(Id, "Cost", tuple.Item1);
            //成本写入
            await _orderCostService.ApportionComputeOnlyLading(dto.LadingBillNumber, tuple.Item2, Model.Enums.SystemBillTypeEnum.CustomerLadingBill);
            return Id;
        }
        public async Task CustomerLadingBillEdit(LadingBillDto dto)
        {
            if (dto.Prepaid > 0) { throw new Exception("帐单已付款不可修改！"); }
            var oldDb = await _repository.GetAsync(dto.Id);
            if (oldDb.LadingBillNumber != dto.LadingBillNumber)
            {
                if (await _repository.IsExistsAsync(x => x.LadingBillNumber == dto.LadingBillNumber && x.IsDelete == false && x.Id != dto.Id))
                {
                    throw new Exception("提单号重复，请确认填写是否正确！");
                }
            }
            await _repository.ClearIncidentals(dto.Id);
            dto.ChargeWeight = dto.Weight = dto.Weight * 1000;//页面录入默认kg
            var db = dto.Copy<Finance_Logistics_LadingBill>();
            IncidentalsCompute compute = new IncidentalsCompute() { LadingBillId = dto.Id, LadingBillNumber = dto.LadingBillNumber, IncidentalFees = dto.finances, Weight = ((dto?.ServicesWeight ?? 0) * 1000) > 0 ? ((dto?.ServicesWeight ?? 0) * 1000) : dto.Weight, Pieces = dto.Pieces ?? 0, orders = null, BillTime = dto.BillTime, Currency = dto.BillCurrency, _Destination = dto._Destination, M3 = (dto.M3 ?? 0.0M) };
            var tuple = await _orderCostService.AddIncidentals(compute);
            db.Cost = tuple.Item1;
            await _repository.UpdateAsync(db);
            //成本写入
            await _orderCostService.ApportionComputeOnlyLading(dto.LadingBillNumber, tuple.Item2, Model.Enums.SystemBillTypeEnum.CustomerLadingBill);
        }
        public async Task RemoveCustomerLadingBill(int LadingBillId)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($" update Finance_Logistics_LadingBill set IsDelete=1 where Id={LadingBillId} ");
            await _repository.ExecuteSqlAsync(sb.ToString());
        }
        #endregion
    }
    #endregion
}
