﻿using SMP.Data.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using XYY.Common.Standard;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface ILogisticsFirstPortService
    {
        Task<PageResult<Logistics_FirstPort>> QueryPage(QueryModel query);


        Task<long> Create(Logistics_FirstPort entity);

        Task<Logistics_FirstPort> GetById(int id);
        Task<Logistics_FirstPort> GetByName(string port);
        Task Edit(Logistics_FirstPort entity);

        Task<bool> Delete(int id);

        Task<IEnumerable<Logistics_FirstPort>> QueryAll();

        Task<bool> IsExists(string port, PortType portType, int Id);
    }
    #endregion
    #region services 
    public class LogisticsFirstPortService : ILogisticsFirstPortService
    {
        ILogistics_FirstPortRepository _repository;

        public LogisticsFirstPortService(ILogistics_FirstPortRepository repository)
        {
            _repository = repository;
        }

        public async Task<PageResult<Logistics_FirstPort>> QueryPage(QueryModel query)
        {
            var result = await _repository.QueryPagerAsync(query);
            return result;
        }

        public async Task Edit(Logistics_FirstPort entity)
        {
            if (await _repository.IsExistsAsync(x => x.Id == entity.Id))
            {
               await _repository.UpdateAsync(entity);
            }
            else
            {
                throw new Exception("未找到口岸记录，无法进行更新");
            }
        }

        public async Task<bool> Delete(int id)
        {
            bool ok = await _repository.DeleteByIdAsync(id);
            return ok;
        }


        public async Task<long> Create(Logistics_FirstPort entity)
        {
            return await _repository.InsertAsync(entity);
        }

        public async Task<Logistics_FirstPort> GetById(int id)
        {
            return await _repository.GetAsync(id);
        }

        public async Task<Logistics_FirstPort> GetByName(string port)
        {
            var result = await _repository.QueryAsync(i => i.Port == port);
            return result.FirstOrDefault();
        }

        public async Task<bool> IsExists(string port, PortType portType, int Id)
        {
            bool result = true;
            if (Id < 1)
            {
                result = await _repository.IsExistsAsync(x => x.Port == port && x.PortType == portType);
            }
            else
            {
                result = await _repository.IsExistsAsync(x => x.Port == port && x.PortType == portType && x.Id != Id);
            }
            return result;
        }



        public async Task<IEnumerable<Logistics_FirstPort>> QueryAll()
        {
            return await _repository.QueryAsync();
        }
    }
    #endregion
}
