﻿using SMP.Data.Finance;
using SMP.Model.Finance;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface ILogistics_ChannelService
    {
        Task<PageResult<Logistics_Channel>> QueryPage(QueryModel query);
        Task Edit(Logistics_Channel entity);
        Task SyncChannel();
        Task<IEnumerable<Logistics_Channel>> GetAllChannelList(List<string> ChannelUniqueIds);
    }
    #endregion
    #region services
    public class Logistics_ChannelService : ILogistics_ChannelService
    {
        ILogistics_ChannelRepository _repository;
        IWMSDataInterconnectionService _wMSDataInterconnection;

        public Logistics_ChannelService(ILogistics_ChannelRepository repository, IWMSDataInterconnectionService wMSDataInterconnection)
        {
            _repository = repository;
            this._wMSDataInterconnection = wMSDataInterconnection;
        }
        public async Task<PageResult<Logistics_Channel>> QueryPage(QueryModel query)
        {
            return await _repository.QueryPagerAsync(query);
        }
        public async Task Edit(Logistics_Channel entity)
        {
            await _repository.UpdateAsync(entity);
        }
        public async Task SyncChannel()
        {
            var thirdPartyData = await _wMSDataInterconnection.PullChannel();
            if (thirdPartyData == null || thirdPartyData.Count() == 0) { throw new Exception("未拉取到渠道信息！"); }
            var dbData = await _repository.QueryAsync();
            if (dbData == null || dbData.Count() == 0) { await _repository.BulkToDBAsync(thirdPartyData.ToList()); }
            else
            {
                var ThirdPartyUniqueIds = thirdPartyData.Select(x => x.ThirdPartyUniqueId).Distinct().ToList();
                var dbUniqueIds = dbData.Select(x => x.ThirdPartyUniqueId).Distinct().ToList();
                var dbEnable = dbData.Where(x => !ThirdPartyUniqueIds.Contains(x.ThirdPartyUniqueId));//已经没有的数据
                var thirdPartyDataEdit = thirdPartyData.Where(x => dbUniqueIds.Contains(x.ThirdPartyUniqueId));//修改的数据
                var thirdPartyDataAdd = thirdPartyData.Where(x => !dbUniqueIds.Contains(x.ThirdPartyUniqueId));//新增的数据
                if (thirdPartyDataAdd != null && thirdPartyDataAdd.Count() > 0) { await _repository.BulkToDBAsync(thirdPartyDataAdd.ToList()); }
                if (dbEnable != null && dbEnable.Count() > 0)
                {
                    var dbEnableListIds = dbEnable.Select(x => x.ThirdPartyUniqueId);
                    string Ids = string.Empty;
                    foreach (var item in dbEnable)
                    {
                        Ids += "'" + item + "',";
                    }
                    string sql = $"update Logistics_Channel set IsEnable=0 where ThirdPartyUniqueId in ({Ids.TrimEnd(',')}) ";
                    await _repository.ExecuteSqlAsync(sql);
                }
                if (thirdPartyDataEdit != null && thirdPartyDataEdit.Count() > 0)
                {
                    List<Logistics_Channel> edit = new List<Logistics_Channel>();
                    foreach (var item in thirdPartyDataEdit)
                    {
                        var temp = dbData.Where(x => x.ThirdPartyUniqueId == item.ThirdPartyUniqueId).FirstOrDefault();
                        if (item.PublicName != temp.PublicName || item.IsEnable != temp.IsEnable || item.PublicCode != temp.PublicCode)
                        {
                            temp.PublicName = item.PublicName;
                            temp.PublicCode = item.PublicCode;
                            temp.IsEnable = item.IsEnable;
                            edit.Add(temp);
                        }
                        //await _repository.UpdateAsync(temp);
                    }
                    if (edit.Count > 0) { await _repository.BatchUpdateEntiy(edit); }
                }
            }
        }
        public async Task<IEnumerable<Logistics_Channel>> GetAllChannelList(List<string> ChannelUniqueIds)
        {
            if (ChannelUniqueIds != null && ChannelUniqueIds.Count > 0) {
                string Ids = string.Empty;
                foreach (var item in ChannelUniqueIds)
                {
                    Ids += "'" +item+ "',";
                }
                return await _repository.QueryBySqlAsync($"select * from Logistics_Channel where ThirdPartyUniqueId in ({Ids.TrimEnd(',')})");
            }
            else { return await _repository.QueryAsync(); }
        }
    }
    #endregion
}
