﻿using SMP.Data.Finance;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface ILogistisc_BillTypeService
    {
        Task<PageResult<Logistisc_BillType>> QueryPage(QueryModel query);


        Task<long> Create(Logistisc_BillType entity);

        Task<Logistisc_BillType> GetById(int id);
        Task<Logistisc_BillType> GetByName(string port);
        Task<IEnumerable<Logistisc_BillType>> GetByCodes(IEnumerable<string> billTypeCodes);
        Task Edit(Logistisc_BillType entity);

        Task<bool> Delete(int id);

        Task<IEnumerable<Logistisc_BillType>> QueryAll();

        Task<bool> IsExists(string billTypeCode, int Id = 0);
    }
    #endregion
    #region services 
    public class Logistisc_BillTypeService : ILogistisc_BillTypeService
    {
        ILogistisc_BillTypeRepository _repository;

        public Logistisc_BillTypeService(ILogistisc_BillTypeRepository repository)
        {
            _repository = repository;
        }

        public async Task<PageResult<Logistisc_BillType>> QueryPage(QueryModel query)
        {
            var result = await _repository.QueryPagerAsync(query);
            return result;
        }

        public async Task Edit(Logistisc_BillType entity)
        {
            if (await _repository.IsExistsAsync(x => x.Id == entity.Id))
            {
                if (await IsExists(entity.BillTypeCode, entity.Id)) { throw new Exception("当前标识已存在，请重新输入唯一标识！"); }
                await _repository.UpdateAsync(entity);
            }
            else
            {
                throw new Exception("未找到帐单类型，无法进行更新");
            }
        }

        public async Task<bool> Delete(int id)
        {
            bool ok = await _repository.DeleteByIdAsync(id);
            return ok;
        }


        public async Task<long> Create(Logistisc_BillType entity)
        {
            if (await IsExists(entity.BillTypeCode)) { throw new Exception("当前标识已存在，请重新输入唯一标识！"); }
            return await _repository.InsertAsync(entity);
        }

        public async Task<Logistisc_BillType> GetById(int id)
        {
            return await _repository.GetAsync(id);
        }

        public async Task<Logistisc_BillType> GetByName(string BillTypeName)
        {
            var result = await _repository.QueryAsync(i => i.BillTypeName == BillTypeName);
            return result.FirstOrDefault();
        }

        public async Task<bool> IsExists(string billTypeCode, int Id = 0)
        {
            return await _repository.IsExistsAsync(x => x.BillTypeCode == billTypeCode && x.Id != Id);
        }


        public async Task<IEnumerable<Logistisc_BillType>> QueryAll()
        {
            return await _repository.QueryAsync();
        }

        public async Task<IEnumerable<Logistisc_BillType>> GetByCodes(IEnumerable<string> billTypeCodes)
        {
            var result = await _repository.QueryAsync(i => billTypeCodes.Contains(i.BillTypeCode));
            return result;
        }
    }
    #endregion
}
