﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;
using XYY.Common.Standard;
using SMP.Model.Dto.Finance;
using SMP.Model.Finance;
using SMP.Data.Finance;
using SMP.Service.Compute;
using SMP.Model.Dto.Compute;
using SMP.Model.Compute;
using SMP.Model.Enums;
using SMP.Data.Compute;
using SMP.Data.TimeTask;
using SMP.Model.TimeTask;
using Newtonsoft.Json;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface IOrderCostService
    {
        #region 数据查询与同步
        Task<PageResult<Compute_ExpenseAmount>> QueryPage(QueryModel qm);
        Task<IEnumerable<Logistisc_Order>> GetOrdersByLadingBillNumber(string LadingBillNumber);
        Task<IEnumerable<Logistisc_Order>> GetOrdersBySystemNos(IEnumerable<string> SystemNos);
        Task<IEnumerable<Logistisc_Order>> GetOrdersByScopeBill(string LadingBillNumber = "", int OrderQuerySettingId = 0);
        /// <summary>
        /// 同步订单信息保存
        /// </summary>
        /// <param name="orders"></param>
        /// <returns></returns>
        Task<bool> SyncOrderInfo(IEnumerable<Logistisc_Order> orders);
        #endregion
        #region 提单成本计算
        /// <summary>
        /// 计算费用
        /// </summary>
        /// <param name="compute"></param>
        /// <returns></returns>
        Task<Tuple<decimal, List<ComputeAmountResult>>> AddIncidentals(IncidentalsCompute compute);
        /// <summary>
        /// 特殊范围帐单计算费用
        /// </summary>
        /// <returns></returns>
        Task<Tuple<List<ComputeAmountResult>, List<SameScopeBill>>> CombinedComputeByScopeBill(IncidentalsCompute compute, List<ScopeSameInfo> scopeSameInfos);
        /// <summary>
        /// 摊分费用
        /// </summary>
        /// <param name="SpreadlogicId"></param>
        /// <param name="computes"></param>
        /// <returns></returns>
        Task<bool> ApportionCompute(string OldLadingBillNumber, string LadingBillNumber, IEnumerable<Logistisc_Order> orders, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum);
        /// <summary>
        /// 费用记录
        /// </summary>
        /// <returns></returns>
        Task<bool> ApportionComputeOnlyLading(string billNumber, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum);
        /// <summary>
        /// 提单费用清理
        /// 可选 按费项
        /// </summary>
        /// <returns></returns>
        Task<bool> ClearApportionComputeRecord(IEnumerable<string> billNumber, int costType, List<int> ExpenseItemIds = null);
        Task<bool> ApportionComputeByScopeBill(string LadingBillNumber, IncidentalsCompute compute, List<ComputeAmountResult> computes, List<SameScopeBill> baseData, List<ScopeSameInfo> scopeSameInfos);
        #endregion
        #region 订单成本计算
        Task<bool> EestimateOrderEndCost(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders);
        /// <summary>
        /// 根据费项特殊计算订单
        /// 归属提单调整
        /// </summary>
        /// <returns></returns>
        Task<bool> EestimateOrderSeparately(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders);
        List<Compute_ExpenseAmount> ApportionComputeRecord(ApportionComputeRecordInput input, List<SameScopeBill> baseData = null);
        Task<bool> InsertApportionComputeRecord(List<Compute_ExpenseAmount> allApportionComputeAmounts);
        Task<bool> ClearApportionComputeRecord(IEnumerable<string> systemNos, CostType costType);
        #endregion
        #region 数据校验
        Task<Tuple<bool, List<ScopeSameInfo>>> ScopeBillVerify(int OrderQuerySettingId, int LadingBillId, IEnumerable<finances> originalFinances);
        Task<IEnumerable<finances>> FinancesVerify(int BillId, IEnumerable<finances> finances);
        #endregion
    }
    #endregion
    #region services
    public class OrderCostService : IOrderCostService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly ICompute_ExpenseAmountRepository _repository;
        private readonly IFinance_Logistics_LadingBillRepository _LadingBillRepository;
        private readonly IFinance_TimeTaskRepository _TimeTaskRepository;

        private readonly IComputeAmountService _computeAmountService;
        private readonly ISpreadlogicService _spreadlogicService;
        private readonly ILogistics_ChannelService _ChannelService;
        private readonly IComputeExpenseService _computeExpenseService;
        private readonly IComputeExpenseItemService _computeExpenseItemService;
        public OrderCostService(IUnitOfWork unitOfWork, ICompute_ExpenseAmountRepository repository, IComputeAmountService computeAmountService, ISpreadlogicService spreadlogicService, ILogistics_ChannelService ChannelService, IComputeExpenseService computeExpenseService
            , IComputeExpenseItemService computeExpenseItemService
            , IFinance_Logistics_LadingBillRepository LadingBillRepository
            , IFinance_TimeTaskRepository TimeTaskRepository
            )
        {
            _unitOfWork = unitOfWork;
            _repository = repository;
            _computeAmountService = computeAmountService;
            _spreadlogicService = spreadlogicService;
            _ChannelService = ChannelService;
            _computeExpenseService = computeExpenseService;
            _computeExpenseItemService = computeExpenseItemService;
            _LadingBillRepository = LadingBillRepository;
            _TimeTaskRepository = TimeTaskRepository;
        }
        #region 数据查询与同步
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersByLadingBillNumber(string LadingBillNumber)
        {
            string sql = $"select * from Logistisc_Order where LadingBillNumber='{LadingBillNumber}' ";
            return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(sql);
        }
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersByScopeBill(string LadingBillNumber = "", int OrderQuerySettingId = 0)
        {
            StringBuilder sb = new StringBuilder();
            string sqlFrom = string.Empty; string sqlLeftJoin = string.Empty; string sqlOn = string.Empty;
            if (!string.IsNullOrEmpty(LadingBillNumber))
            {
                sqlFrom = " Finance_Logistics_LadingBill(nolock) a ";
                sqlLeftJoin = " left join ";
                sqlOn = " on b.RangeSettingId=a.OrderQuerySettingId ";
            }
            sb.Append(@$" select c.*
from {sqlFrom}
{sqlLeftJoin} Finance_OrderRangeSearchResult(nolock) b {sqlOn}
left join Logistisc_Order(nolock) c on c.SystemNo=b.SystemNo ");
            sb.Append($" where 1=1  ");

            if (!string.IsNullOrEmpty(LadingBillNumber)) { sb.Append($" and a.SystemBillType={(int)SystemBillTypeEnum.ScopeBill} and a.LadingBillNumber='{LadingBillNumber}' and a.IsDelete=0 "); }
            if (OrderQuerySettingId > 0) { sb.Append($" and b.RangeSettingId={OrderQuerySettingId} "); }
            return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(sb.ToString());
        }
        public async Task<IEnumerable<Logistisc_Order>> GetOrdersBySystemNos(IEnumerable<string> SystemNos)
        {
            try
            {
                var xml = string.Join("\r\n", SystemNos.Select(x => $"<NumberList><Number>{x}</Number></NumberList>"));
                string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_getordersbysystemnos') and type='U')
    drop table #temp_getordersbysystemnos
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_getordersbysystemnos
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                string selectSql = " select * from Logistisc_Order(nolock) where SystemNo in (select * from #temp_getordersbysystemnos) ";
                string executeSql = systemNoSql.Formater(xml) + selectSql;
                return await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(executeSql);
            }
            catch (Exception ex)
            {
                return null;
            }
        }
        public async Task<List<SameScopeBill>> GetSameScopeBills(List<int> billIds)
        {
            List<SameScopeBill> sameScopeBills = new List<SameScopeBill>();
            string billSql = $" select * from Finance_Logistics_LadingBill(nolock) where Id in ({string.Join(',', billIds)});";
            var bills = await _unitOfWork.QueryBySqlAsync<Finance_Logistics_LadingBill>(billSql);
            foreach (var item in billIds)
            {
                SameScopeBill sameScopeBill = new SameScopeBill() { LadingBillId = item };
                sameScopeBill.bill = bills.Where(x => x.Id == item).FirstOrDefault();
                string orderSql = $@"select d.*  from Finance_Logistics_LadingBill(nolock) a 
left join Finance_OrderQuerySetting(nolock) b on b.Id=a.OrderQuerySettingId
left join Finance_OrderRangeSearchResult(nolock) c on c.RangeSettingId=b.Id
left join Logistisc_Order(nolock) d on d.SystemNo=c.SystemNo
where a.Id ={item} and a.IsDelete=0 ";
                var orders = await _unitOfWork.QueryBySqlAsync<Logistisc_Order>(orderSql);
                sameScopeBill.Orders = orders.ToList();
                if (sameScopeBill.Orders.Any()) { sameScopeBills.Add(sameScopeBill); }
            }
            return sameScopeBills;
        }

        public async Task<PageResult<Compute_ExpenseAmount>> QueryPage(QueryModel qm)
        {
            if (qm.QueryParamer == null) { qm.QueryParamer = new List<QueryParamer>(); }
            qm.QueryParamer.Add(new QueryParamer() { Filed = "IsDelete", Value = "0", Method = "Eq" });

            return await _repository.QueryPagerAsync(qm);
            //#region 基础sql
            //StringBuilder sb = new StringBuilder();
            //var tableNams = await GetFinanceCostTableName("");
            //string filed = string.Empty; string leftJoin = string.Empty; int index = 0;
            //if (tableNams.Count() > 0)
            //{
            //    filed = ",(";
            //    foreach (var item in tableNams)
            //    {
            //        filed += $"b{index}.OrderCost"; leftJoin += $" left join {item} b{index} on b{index}.SystemNo=a.SystemNo ";
            //        index++;
            //    }
            //    filed += ") TotalCost";
            //}
            //sb.Append($"select a.*{filed} from Finance_Cost_End a ");
            //sb.Append($" {leftJoin} ");
            //#endregion
            //#region 分页
            ////排序
            //string sortString = string.Empty;
            //foreach (var item in qm.QuerySort)
            //{
            //    sortString += " " + item.Field + " " + (item.IsDesc ? "DESC" : "") + ",";
            //}
            //sortString = sortString.TrimEnd(',');
            //string sql1 = sb.ToString();
            //StringBuilder pager = new StringBuilder();
            //string where = string.Empty;
            //if (qm.QueryParamer != null) { where = _unitOfWork.GetWhere(qm.QueryParamer); }
            //pager.Append($@"select * from (select Row_Number()over(order by  {sortString})rowId,*
            //from ({sql1})a where 1=1 {where}) T where rowId between {(qm.PageIndex - 1) * qm.PageSize + 1} and {qm.PageSize * qm.PageIndex};");
            //pager.Append(@$"select count(0)TotalCount from (select Row_Number()over(order by  {sortString})rowId,*
            //from({ sql1})a where 1 = 1 { where}) T where 1=1");
            //#endregion
            //#region 取值
            //string sql = pager.ToString();
            //var dr = await _unitOfWork.Connection.QueryMultipleAsync(sql);
            ////需要注意读取顺序，依据语句执行顺序进行读取
            //var pageResult = new PageResult<OrderCost>()
            //{
            //    Result = dr.Read<OrderCost>(),
            //    PageIndex = qm.PageIndex,
            //    PageSize = qm.PageIndex,
            //    TotalCount = dr.Read<int>().FirstOrDefault()
            //};
            //#endregion
            //return pageResult;
        }

        public async Task<bool> SyncOrderInfo(IEnumerable<Logistisc_Order> orders)
        {
            var batchQty = 5000.0;
            try
            {
                for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
                {
                    var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    var xml = string.Join("\r\n", tempOrders.Select(x => $"<NumberList><Number>{x.SystemNo}</Number></NumberList>"));
                    string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_smpsyncordersearchs') and type='U')
    drop table #temp_smpsyncordersearchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_smpsyncordersearchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                    string deleteSql = " delete Logistisc_Order where SystemNo in (select * from #temp_smpsyncordersearchs) ";
                    string executeSql = systemNoSql.Formater(xml) + deleteSql;
                    await _unitOfWork.ExecuteAsync(executeSql);
                    await _unitOfWork.BulkToDBAsync<Logistisc_Order>(tempOrders.ToList());
                }
            }
            catch (Exception ex)
            {

                return false;
            }
            return true;

        }
        #endregion
        #region 提单成本计算
        /// <summary>
        /// 提单成本合计
        /// </summary>
        /// <returns></returns>
        public async Task<Tuple<decimal, List<ComputeAmountResult>>> AddIncidentals(IncidentalsCompute compute)
        {
            decimal sumIncidentalFee = 0;
            List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
            List<Finance_Incidental> toDb = new List<Finance_Incidental>();
            compute.IncidentalFees = await FinancesVerify(compute.LadingBillId, compute.IncidentalFees);
            if (compute.IncidentalFees != null)
            {
                var temp = compute.IncidentalFees.ToList();
                temp.RemoveAll(x => x.IncidentalType == 0);
                List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
                foreach (var fa in temp)
                {
                    var tempList = fa.ExpenseResult.Where(x => x.num == 1).Select(x => { x.IncidentalFee = 0; return x; }).CopyList<Finance_Incidental>();//重置数值,去重
                    //重置数值,去重,获取此分摊模式的数据
                    var tempOrderSeparately = fa.ExpenseResult.Where(x => x.num == 1 && x.ApportionMethod == ApportionMethodEnum.OrderSeparately).Select(x => { x.IncidentalFee = 0; return x; }).ToList();
                    foreach (var item in tempList)
                    {
                        item.ExpenseItemId = fa.IncidentalType;
                        item.LadingBillId = compute.LadingBillId;
                    }
                    toDb.AddRange(tempList);
                    //按整个费项来设置。不参与提单计算，写入详情表
                    if (tempOrderSeparately != null && tempOrderSeparately.Count > 0) { continue; }
                    souces.Add(new ComputeAmountSouce()
                    {
                        ComputeDate = compute.BillTime,
                        BindBillId = compute.LadingBillId,
                        Weight = compute.Weight,
                        ExpenseItemId = fa.IncidentalType,
                        Currency = compute.Currency,
                        BillNumber = "",
                        CountryCode = "",
                        State = "",
                        City = "",
                        ZipCode = "",
                        Stree = "",
                        TrackingNumber = "",
                        Port = compute._Destination,
                        OrderQty = compute.Pieces,
                        M3 = compute.M3,
                    });
                }
                computes = await _computeAmountService.MultipleCompute(souces);
                //总值计算
                if (computes.Any(x => !x.IsSuccess))
                {
                    var errorInfo = computes.Where(x => !x.IsSuccess).Select(x => x.ExpenseName + ":" + x.Message).Distinct();
                    throw new Exception(string.Join(',', errorInfo));
                }
                //独立计费项判断，并创建异步任务
                if (!await ApportionMethodVerify(compute.orders, compute.IncidentalFees, compute.LadingBillNumber)) { throw new Exception("存在独立计费项，任务创建失败！请联系技术人员！"); }
                sumIncidentalFee = computes.Sum(x => x.Amount);
                if (toDb.Count > 0) { await _unitOfWork.BulkToDBAsync<Finance_Incidental>(toDb); }
            }
            return Tuple.Create(sumIncidentalFee, computes);
        }
        public async Task<bool> ApportionCompute(string OldLadingBillNumber, string LadingBillNumber, IEnumerable<Logistisc_Order> orders, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum)
        {
            //基础数据判断 start
            if (billTypeEnum != SystemBillTypeEnum.ScopeBill)//非范围帐单才会在这里清除旧数据
            {
                if (!await ClearApportionComputeRecord(new List<string>() { OldLadingBillNumber }, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }
            }
            var channelId = orders.Select(x => x.OrderChannelUniqueId).Distinct().ToList();
            var channels = await _ChannelService.GetAllChannelList(channelId);
            if (channels == null || channels.Count() == 0) { throw new Exception("未获取到订单相应的渠道！"); }
            var logicDetails = computes.Select(x => x.ApportionMethod).Distinct().ToList();
            //基础数据判断 end
            var TotalWegith = orders.Sum(x => x.OPWeight);
            orders.ToList().ForEach(x => x.WeightRate = decimal.Round(Convert.ToDecimal(x.OPWeight) / Convert.ToDecimal(TotalWegith), 6));//重量占比保留六位
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            foreach (var item in computes)
            {
                ApportionComputeRecordInput input = new ApportionComputeRecordInput()
                {
                    orders = orders,
                    LadingBillNumber = LadingBillNumber,
                    amountResult = item,
                    SystemBillType = billTypeEnum,
                };
                var computeAmounts = ApportionComputeRecord(input);
                allApportionComputeAmounts.AddRange(computeAmounts);
            }
            //创建异步任务
            if (billTypeEnum != SystemBillTypeEnum.ScopeBill)
            {
                orders = await PortVerify(orders, channels);
                await _TimeTaskRepository.CreateTypeTaskByPort(orders.Where(x => !string.IsNullOrEmpty(x.Port)));
            }

            return await InsertApportionComputeRecord(allApportionComputeAmounts);
        }
        /// <summary>
        /// 客户头程计算
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ApportionComputeOnlyLading(string billNumber, List<ComputeAmountResult> computes, SystemBillTypeEnum billTypeEnum)
        {
            bool result = true;
            List<string> billNumbers = new List<string>() { billNumber };
            //基础数据判断 start
            if (!await ClearApportionComputeRecord(billNumbers, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            foreach (var compute in computes)
            {
                DateTime TransactionTime = DateTime.Now;
                allApportionComputeAmounts.Add(new Compute_ExpenseAmount()
                {
                    Number = billNumber,
                    ExpenseId = compute.ExpenseId,
                    ExpenseDetailId = compute.DetailId,
                    VersionId = compute.VersionId,
                    ExpenseItemId = compute.ExpenseItemId,
                    TransactionTime = TransactionTime,
                    Currency = compute.BillCurrency,
                    AmountinOriginalCurrency = compute.AmountinOriginalCurrency,
                    SettlementRate = compute.SettlementRate,
                    SettlementCurrency = compute.Currency,
                    SettlementAmount = compute.Amount,
                    FeeDescription = compute.ExpenseName,
                    LadingBillNumber = billNumber,
                    CostType = CostType.预估成本,
                    CreateTime = DateTime.Now,
                    Zone = compute.Zones,
                    IsDelete = false,
                    SystemBillType = billTypeEnum,
                });
            }
            await InsertApportionComputeRecord(allApportionComputeAmounts);
            return result;
        }
        /// <summary>
        /// 范围帐单 相同费项计算
        /// 合计计算
        /// </summary>
        /// <returns></returns>
        public async Task<Tuple<List<ComputeAmountResult>, List<SameScopeBill>>> CombinedComputeByScopeBill(IncidentalsCompute compute, List<ScopeSameInfo> scopeSameInfos)
        {
            List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
            List<Finance_Incidental> toDb = new List<Finance_Incidental>();
            List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
            //取出所有基础数据
            List<int> billIds = scopeSameInfos.SelectMany(x => x.LadingBillIds).Distinct().ToList();
            var baseData = await GetSameScopeBills(billIds);
            foreach (var fa in compute.IncidentalFees)
            {
                if (fa.IncidentalType == 0) { continue; }
                var tempList = fa.ExpenseResult.Where(x => x.num == 1).Select(x => { x.IncidentalFee = 0; return x; }).CopyList<Finance_Incidental>();//重置数值,去重
                foreach (var item in tempList)
                {
                    item.ExpenseItemId = fa.IncidentalType;
                    item.LadingBillId = compute.LadingBillId;
                }
                toDb.AddRange(tempList);
                //取合计数值 start  每个费项重叠的订单都不一定相同，循环取数
                var tempScope = scopeSameInfos.Where(x => x.ExpenseItemId == fa.IncidentalType).FirstOrDefault();
                var tempBaseData = baseData.Where(x => tempScope.LadingBillIds.Contains(x.LadingBillId)).ToList();
                decimal tempWeight = tempBaseData.Sum(x => ((x.bill.ServicesWeight ?? 0) * 1000) > 0 ? ((x.bill?.ServicesWeight ?? 0) * 1000) : x.bill.Weight)+compute.Weight;
                decimal tempM3 = tempBaseData.Sum(x => (x.bill.M3 ?? 0))+compute.M3;
                int tempOrderQty = tempBaseData.Sum(x => x.Orders.Count())+compute.orders.Count();
                //取合计数值 end
                souces.Add(new ComputeAmountSouce()
                {
                    ComputeDate = compute.BillTime,
                    BindBillId = compute.LadingBillId,
                    Weight = tempWeight,
                    ExpenseItemId = fa.IncidentalType,
                    Currency = compute.Currency,
                    BillNumber = "",
                    CountryCode = "",
                    State = "",
                    City = "",
                    ZipCode = "",
                    Stree = "",
                    TrackingNumber = "",
                    Port = compute._Destination,
                    OrderQty = tempOrderQty,
                    M3 = tempM3,
                });
            }
            computes = await _computeAmountService.MultipleCompute(souces);
            //总值计算
            if (computes.Any(x => !x.IsSuccess))
            {
                var errorInfo = computes.Where(x => !x.IsSuccess).Select(x => x.ExpenseName + ":" + x.Message).Distinct();
                throw new Exception(string.Join(',', errorInfo));
            }
          
            if (toDb.Count > 0) { await _unitOfWork.BulkToDBAsync<Finance_Incidental>(toDb); }
            return Tuple.Create(computes, baseData);
        }
        /// <summary>
        /// 范围帐单 相同费项计算
        /// 分摊计算
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ApportionComputeByScopeBill(string LadingBillNumber, IncidentalsCompute compute, List<ComputeAmountResult> computes, List<SameScopeBill> baseData, List<ScopeSameInfo> scopeSameInfos)
        {
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            //按费项清一次数据，费项多可能会出连接数的错误。
            foreach (var item in scopeSameInfos)
            {
                bool result = await ClearApportionComputeRecord(item.LadingBillIds,(int)CostType.预估成本,new List<int>() { item.ExpenseItemId});
                if (!result) { throw new Exception("旧费项数据清理失败！请重新保存！"); }
            }
            //取数
            foreach (var item in computes)
            {
                var tempScope = scopeSameInfos.Where(x => x.ExpenseItemId == item.ExpenseItemId).FirstOrDefault();
                var tempBaseData = baseData.Where(x => tempScope.LadingBillIds.Contains(x.LadingBillId)).ToList();
                var orders = tempBaseData.SelectMany(x => x.Orders).ToList();
                orders.AddRange(compute.orders);
                var TotalWegith = orders.Sum(x => x.OPWeight);
                orders.ForEach(x => x.WeightRate = decimal.Round(Convert.ToDecimal(x.OPWeight) / Convert.ToDecimal(TotalWegith), 6));//重量占比保留六位
                ApportionComputeRecordInput input = new ApportionComputeRecordInput()
                {
                    orders = orders,
                    LadingBillNumber = LadingBillNumber,
                    amountResult = item,
                    SystemBillType = SystemBillTypeEnum.ScopeBill,
                };
                var computeAmounts = ApportionComputeRecord(input, tempBaseData);
                allApportionComputeAmounts.AddRange(computeAmounts);
            }
            bool returnResult= await InsertApportionComputeRecord(allApportionComputeAmounts);
            return returnResult;
        }
        #endregion
        #region 订单成本计算
        public async Task<bool> EestimateOrderEndCost(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders)
        {
            bool result = true;
            //计算的基础数据获取 start
            var channelId = orders.Select(x => x.OrderChannelUniqueId).Distinct().ToList();
            var channels = await _ChannelService.GetAllChannelList(channelId);
            var batchQty = 5000.0;
            //计算的基础数据获取 end
            if (channels == null || channels.Count() == 0) { throw new Exception("未获取到订单相应的渠道！"); }
            orders = await PortVerify(orders, channels);
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
            {
                var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
                List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
                if (!await ClearApportionComputeRecord(tempOrders, SystemBillTypeEnum.OrderBill, (int)CostType.预估成本)) { throw new Exception("旧数据清除失败！"); }//独立订单计算 固定类型
                foreach (var item in tempOrders)
                {
                    var tempChannel = channels.Where(x => x.ThirdPartyUniqueId == item.OrderChannelUniqueId)?.FirstOrDefault();
                    //总费用计算
                    souces.Add(new ComputeAmountSouce()
                    {
                        ComputeDate = item.ReceiveTime ?? item.OrderTime,
                        Port = item.Port,
                        Weight = item.OPWeight,
                        Currency = tempChannel?.BillCurrency ?? "",
                        Volume = (item.Length * item.Width * item.Height) ?? 0,
                        ExpenseItemId = tempChannel?.ExpenseItemId ?? 0,
                        BillNumber = item.SystemNo,
                        CountryCode = item.ReceiverCountryCode,
                        State = item.ReceiverState,
                        City = item.ReceiverCity,
                        ZipCode = item.ReceiverZipCode,
                        Stree = item.ReceiverStreet,
                        TrackingNumber = item.TrackingNumber,
                        GoodsDeclareValue = item.DeclareFee ?? 0,
                    });
                    item.WeightRate = 1;//单个订单预估重量占比始终为1
                }
                computes = await _computeAmountService.MultipleCompute(souces);
                //计算数据处理
                if (computes.Any(x => !x.IsSuccess))
                {
                    result = false;
                    await _TimeTaskRepository.CreateTypeTaskScopeDetail(TaskInfo, computes.ToList());
                    computes.RemoveAll(x => !x.IsSuccess);
                }
                else
                {
                    await _TimeTaskRepository.InsTypeTaskDetail(TaskInfo, computes.ToList());
                }
                //处理后数据写入
                foreach (var compute in computes)
                {
                    DateTime TransactionTime = DateTime.Now;
                    allApportionComputeAmounts.Add(new Compute_ExpenseAmount()
                    {
                        Number = compute.BillNumber,
                        ExpenseId = compute.ExpenseId,
                        ExpenseDetailId = compute.DetailId,
                        VersionId = compute.VersionId,
                        ExpenseItemId = compute.ExpenseItemId,
                        TransactionTime = TransactionTime,
                        Currency = compute.BillCurrency,
                        AmountinOriginalCurrency = compute.AmountinOriginalCurrency,
                        SettlementRate = compute.SettlementRate,
                        SettlementCurrency = compute.Currency,
                        SettlementAmount = compute.Amount,
                        FeeDescription = compute.ExpenseName,
                        BillTypeCode = "SystemEndBill",
                        CostType = CostType.预估成本,
                        CreateTime = DateTime.Now,
                        Zone = compute.Zones,
                        IsDelete = false,
                        SystemBillType = SystemBillTypeEnum.OrderBill,
                    });
                }
            }
            if (allApportionComputeAmounts.Count > 0) { await InsertApportionComputeRecord(allApportionComputeAmounts); }
            return result;
        }
        public async Task<bool> EestimateOrderSeparately(Finance_TimeTask TaskInfo, IEnumerable<Logistisc_Order> orders)
        {
            bool result = true;
            var ladingBillInfo = _LadingBillRepository.GetLadingBill(TaskInfo.LadingBillNumber);
            var ExpenseResults = JsonConvert.DeserializeObject<List<QueryExpenseByLadingResult>>(TaskInfo.ExpenseResultsJson);
            if (ladingBillInfo == null || ExpenseResults == null || ExpenseResults.Count() == 0) { return false; }
            var batchQty = 5000.0;
            List<int> ExpenseItemIds = ExpenseResults.Select(x => x.ExpenseItemId).Distinct().ToList();
            List<Compute_ExpenseAmount> allApportionComputeAmounts = new List<Compute_ExpenseAmount>();
            //独立订单计算不判断口岸
            for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
            {
                var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                List<ComputeAmountResult> computes = new List<ComputeAmountResult>();
                List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
                //增加清数据过程，按费项清理
                if (!await ClearApportionComputeRecord(tempOrders, (int)CostType.预估成本, ExpenseItemIds)) { throw new Exception("旧数据清除失败！"); }
                foreach (var expenseItemId in ExpenseItemIds)
                {
                    foreach (var item in tempOrders)
                    {
                        souces.Add(new ComputeAmountSouce()
                        {
                            ComputeDate = item.ReceiveTime ?? item.OrderTime,
                            Port = "",
                            Weight = item.OPWeight,
                            Currency = "",
                            Volume = (item.Length * item.Width * item.Height) ?? 0,
                            ExpenseItemId = expenseItemId,
                            BillNumber = item.SystemNo,
                            CountryCode = item.ReceiverCountryCode,
                            State = item.ReceiverState,
                            City = item.ReceiverCity,
                            ZipCode = item.ReceiverZipCode,
                            Stree = item.ReceiverStreet,
                            TrackingNumber = item.TrackingNumber,
                            IOSS = item.IOSS,
                            GoodsDeclareValue = item.DeclareFee ?? 0,
                        });
                    }
                }
                computes = await _computeAmountService.MultipleCompute(souces);
                //计算数据处理
                if (computes.Any(x => !x.IsSuccess))
                {
                    result = false;
                    await _TimeTaskRepository.CreateTypeTaskScopeDetail(TaskInfo, computes.ToList());
                    computes.RemoveAll(x => !x.IsSuccess);
                }
                else
                {
                    await _TimeTaskRepository.InsTypeTaskDetail(TaskInfo, computes.ToList());
                }
                //处理后数据写入
                foreach (var compute in computes)
                {
                    DateTime TransactionTime = DateTime.Now;
                    allApportionComputeAmounts.Add(new Compute_ExpenseAmount()
                    {
                        Number = compute.BillNumber,
                        ExpenseId = compute.ExpenseId,
                        ExpenseDetailId = compute.DetailId,
                        VersionId = compute.VersionId,
                        ExpenseItemId = compute.ExpenseItemId,
                        TransactionTime = TransactionTime,
                        Currency = compute.BillCurrency,
                        AmountinOriginalCurrency = compute.AmountinOriginalCurrency,
                        SettlementRate = compute.SettlementRate,
                        SettlementCurrency = compute.Currency,
                        SettlementAmount = compute.Amount,
                        FeeDescription = compute.ExpenseName,
                        CostType = CostType.预估成本,
                        CreateTime = DateTime.Now,
                        Zone = compute.Zones,
                        IsDelete = false,
                        LadingBillNumber = TaskInfo.LadingBillNumber,
                        SystemBillType = SystemBillTypeEnum.LadingBill,
                    });
                }
            }
            if (allApportionComputeAmounts.Count > 0) { await InsertApportionComputeRecord(allApportionComputeAmounts); }
            return result;
        }
        #endregion
        #region 数据落地
        /// <summary>
        /// 摊分数据运算
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public List<Compute_ExpenseAmount> ApportionComputeRecord(ApportionComputeRecordInput input, List<SameScopeBill> baseData = null)
        {
            bool result = true; bool isWeightRate = false;
            decimal ApportionValue = 0.0M;//结算币种分摊
            decimal AmountinOriginalCurrency = 0.0M;//原币种分
            switch (input.amountResult.ApportionMethod)
            {
                case Model.Enums.ApportionMethodEnum.OrderCount:
                    ApportionValue = input.amountResult.Amount / input.orders.Count();
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency / input.orders.Count();
                    break;
                case Model.Enums.ApportionMethodEnum.Actually:
                    ApportionValue = input.amountResult.Amount;
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency;
                    break;
                case Model.Enums.ApportionMethodEnum.WeightRate:
                    ApportionValue = input.amountResult.Amount;
                    AmountinOriginalCurrency = input.amountResult.AmountinOriginalCurrency;
                    isWeightRate = true;
                    break;
                case Model.Enums.ApportionMethodEnum.None:
                    //无需分摊，记一笔为0的账
                    break;
            }
            List<Compute_ExpenseAmount> toDbModel = new List<Compute_ExpenseAmount>();
            DateTime TransactionTime = DateTime.Now;// Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:ffff"));
            string ladingNumber = string.Empty;
            //特殊范围分摊数据获取正确的提单号
            if (baseData != null && baseData.Count() > 0)
            {
                //重复项处理
                foreach (var sameScopeBill in baseData)
                {
                    foreach (var item in sameScopeBill.Orders)
                    {
                        toDbModel.Add(new Compute_ExpenseAmount()
                        {
                            Number = item.SystemNo,
                            ExpenseId = input.amountResult.ExpenseId,
                            ExpenseDetailId = input.amountResult.DetailId,
                            VersionId = input.amountResult.VersionId,
                            ExpenseItemId = input.amountResult.ExpenseItemId,
                            TransactionTime = TransactionTime,
                            Currency = input.amountResult.BillCurrency,
                            AmountinOriginalCurrency = !isWeightRate ? AmountinOriginalCurrency : AmountinOriginalCurrency * (decimal)item.WeightRate,
                            SettlementRate = input.amountResult.SettlementRate,
                            SettlementCurrency = input.amountResult.Currency,
                            SettlementAmount = !isWeightRate ? ApportionValue : ApportionValue * (decimal)item.WeightRate,
                            FeeDescription = input.amountResult.ExpenseName,
                            LadingBillNumber = sameScopeBill.bill.LadingBillNumber,
                            CostType = input.CostType,
                            CreateTime = DateTime.Now,
                            SystemBillType = input.SystemBillType,
                        });
                    }
                }
                List<string> systemNos = baseData.SelectMany(x => x.Orders).Select(x => x.SystemNo).Distinct().ToList();
                var orderNow = input.orders.Where(x => !systemNos.Contains(x.SystemNo)).ToList();
                //过滤出当前帐单的订单
                foreach (var item in orderNow)
                {
                    toDbModel.Add(new Compute_ExpenseAmount()
                    {
                        Number = item.SystemNo,
                        ExpenseId = input.amountResult.ExpenseId,
                        ExpenseDetailId = input.amountResult.DetailId,
                        VersionId = input.amountResult.VersionId,
                        ExpenseItemId = input.amountResult.ExpenseItemId,
                        TransactionTime = TransactionTime,
                        Currency = input.amountResult.BillCurrency,
                        AmountinOriginalCurrency = !isWeightRate ? AmountinOriginalCurrency : AmountinOriginalCurrency * (decimal)item.WeightRate,
                        SettlementRate = input.amountResult.SettlementRate,
                        SettlementCurrency = input.amountResult.Currency,
                        SettlementAmount = !isWeightRate ? ApportionValue : ApportionValue * (decimal)item.WeightRate,
                        FeeDescription = input.amountResult.ExpenseName,
                        LadingBillNumber = input.LadingBillNumber,
                        CostType = input.CostType,
                        CreateTime = DateTime.Now,
                        SystemBillType = input.SystemBillType,
                    });
                }
             }
            else//正常流程的分摊
            {
                foreach (var item in input.orders)
                {
                    toDbModel.Add(new Compute_ExpenseAmount()
                    {
                        Number = item.SystemNo,
                        ExpenseId = input.amountResult.ExpenseId,
                        ExpenseDetailId = input.amountResult.DetailId,
                        VersionId = input.amountResult.VersionId,
                        ExpenseItemId = input.amountResult.ExpenseItemId,
                        TransactionTime = TransactionTime,
                        Currency = input.amountResult.BillCurrency,
                        AmountinOriginalCurrency = !isWeightRate ? AmountinOriginalCurrency : AmountinOriginalCurrency * (decimal)item.WeightRate,
                        SettlementRate = input.amountResult.SettlementRate,
                        SettlementCurrency = input.amountResult.Currency,
                        SettlementAmount = !isWeightRate ? ApportionValue : ApportionValue * (decimal)item.WeightRate,
                        FeeDescription = input.amountResult.ExpenseName,
                        LadingBillNumber = input.LadingBillNumber,
                        CostType = input.CostType,
                        CreateTime = DateTime.Now,
                        SystemBillType = input.SystemBillType,
                    });
                }
            }
            return toDbModel;
        }
        public async Task<bool> InsertApportionComputeRecord(List<Compute_ExpenseAmount> allApportionComputeAmounts)
        {
            bool insertOk = false;
            if (allApportionComputeAmounts?.Any() == true)
            {
                decimal batchQty = 5000.0M;//5000条一批
                for (int i = 0; i < Math.Ceiling(allApportionComputeAmounts.Count() / batchQty); i++)
                {
                    var temp = allApportionComputeAmounts.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    insertOk = await _unitOfWork.BulkToDBAsync<Compute_ExpenseAmount>(temp);
                }
            }
            return insertOk;
        }
        /// <summary>
        /// 末端成本数据记录清理
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<Logistisc_Order> orders, SystemBillTypeEnum billTypeEnum, int costType)
        {
            if (orders == null || orders.Count() == 0) { return true; }
            var batchQty = 10000.0;
            try
            {
                for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
                {
                    var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    var xml = string.Join("\r\n", tempOrders.Select(x => $"<NumberList><Number>{x.SystemNo}</Number></NumberList>"));
                    string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_smpclearsearchs') and type='U')
    drop table #temp_smpclearsearchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_smpclearsearchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";
                    //string deleteSql = $" delete Compute_ExpenseAmount where Number in (select * from #temp_searchs)  and SystemBillType={(int)billTypeEnum} and costType={costType}";
                    string updateToDeleteSql = $"		 UPDATE a set a.IsDelete=1  from  Compute_ExpenseAmount(nolock)a join #temp_smpclearsearchs(nolock)b on a.Number=b.Number where SystemBillType = { (int)billTypeEnum } and costType = {costType} ";

                    string executeSql = systemNoSql.Formater(xml) + updateToDeleteSql;//软删除
                    await _unitOfWork.ExecuteAsync(executeSql);
                }
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }
        /// <summary>
        /// 提单成本数据清理
        /// 可选按费项
        /// </summary>
        /// <returns></returns>
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<string> billNumber, int costType, List<int> ExpenseItemIds = null)
        {
            if (billNumber == null || billNumber.Count() == 0) { return true; }
            try
            {
                string updateToDeleteSql = $"UPDATE Compute_ExpenseAmount set IsDelete=1 where LadingBillNumber in @billNumber  and costType={costType}";
                if (ExpenseItemIds != null && ExpenseItemIds.Count > 0) { updateToDeleteSql += $" and ExpenseItemId in ({string.Join(',', ExpenseItemIds)})"; }
                string executeSql = updateToDeleteSql;//软删除
                await _unitOfWork.ExecuteAsync(executeSql, new { billNumber = billNumber });
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<int> billIds, int costType, List<int> ExpenseItemIds = null)
        {
            if (billIds == null || billIds.Count() == 0) { return true; }
            try
            {
                string updateToDeleteSql = $"UPDATE Compute_ExpenseAmount set IsDelete=1 where LadingBillNumber in (select LadingBillNumber from Finance_Logistics_LadingBill(nolock) where Id in @billIds)  and costType={costType}";
                if (ExpenseItemIds != null && ExpenseItemIds.Count > 0) { updateToDeleteSql += $" and ExpenseItemId in ({string.Join(',', ExpenseItemIds)})"; }
                string executeSql = updateToDeleteSql;//软删除
                await _unitOfWork.ExecuteAsync(executeSql, new { billIds = billIds});
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }
        /// <summary>
        /// 按费项清理数据
        /// </summary>
        public async Task<bool> ClearApportionComputeRecord(IEnumerable<Logistisc_Order> orders, int costType, List<int> ExpenseItemIds)
        {
            if (orders == null || orders.Count() == 0) { return true; }
            var batchQty = 5000.0;
            try
            {
                for (int i = 0; i < Math.Ceiling(orders.Count() / batchQty); i++)
                {
                    var tempOrders = orders.Skip(i * (int)batchQty).Take((int)batchQty).ToList();
                    var xml = string.Join("\r\n", tempOrders.Select(x => $"<NumberList><Number>{x.SystemNo}</Number></NumberList>"));
                    string systemNoSql = @"
if exists (select * from tempdb.dbo.sysobjects where id = object_id(N'tempdb..#temp_smpclearexpenseitemsearchs') and type='U')
    drop table #temp_smpclearexpenseitemsearchs
                                            DECLARE @idoc int
                                            DECLARE @doc varchar(max)
                                            SET @doc ='
                                            <ROOT>
                                            {0}
                                            </ROOT>'
                                            --Create an internal representation of the XML document.
                                            EXEC sp_xml_preparedocument @idoc OUTPUT, @doc
                                            -- SELECT stmt using OPENXML rowset provider
                                            SELECT * into #temp_smpclearexpenseitemsearchs
                                            FROM   OPENXML (@idoc, '/ROOT/NumberList',2)
                                                     WITH (Number nvarchar(500)) ";

                    string updateToDeleteSql = $"		 UPDATE a set a.IsDelete=1  from  Compute_ExpenseAmount(nolock)a join #temp_smpclearexpenseitemsearchs(nolock)b on a.Number=b.Number where costType = {costType} and ExpenseItemId in ({string.Join(',', ExpenseItemIds)})";

                    string executeSql = systemNoSql.Formater(xml) + updateToDeleteSql;//软删除
                    await _unitOfWork.ExecuteAsync(executeSql);
                }
            }
            catch (Exception ex)
            {
                return false;
            }
            return true;
        }

        public async Task<bool> ClearApportionComputeRecord(IEnumerable<string> systemNos, CostType costType)
        {
            if (systemNos == null || systemNos.Count() == 0) { return true; }
            {
                try
                {
                    string updateToDeleteSql = $"UPDATE Compute_ExpenseAmount set IsDelete=1 where Number in @SystemNo  and costType=@CostType";

                    int pageLen = 2000;
                    int totalCount = systemNos.Count();
                    int totalPage = (int)Math.Ceiling(totalCount / (decimal)pageLen);
                    for (var p = 0; p < totalPage; p++)
                    {
                        var pageNumbers = systemNos.Skip(pageLen * p).Take(pageLen);
                        await _unitOfWork.ExecuteAsync(updateToDeleteSql, new { SystemNo = systemNos, CostType = costType });
                    }
                }
                catch (Exception ex)
                {
                    return false;
                }
            }
            return true;
        }
        #endregion
        #region 校验或扩展
        /// <summary>
        /// 费用数据验证
        /// </summary>
        /// <param name="BillId"></param>
        /// <param name="finances"></param>
        /// <returns></returns>
        public async Task<IEnumerable<finances>> FinancesVerify(int BillId, IEnumerable<finances> finances)
        {
            //获取所有待修改数据
            List<Compute_Expense> computes = new List<Compute_Expense>();
            foreach (var item in finances)
            {
                //独立计算费项不做验证
                if (item.ExpenseResult.Any(x => x.ApportionMethod == ApportionMethodEnum.OrderSeparately)) { continue; }
                var tempResult = item.ExpenseResult.Where(x => x.num == 1).Select(x => new { x.ExpenseId, x.VersionId }).ToList();
                foreach (var result in tempResult)
                {
                    var Compute_Expense = await _computeExpenseService.GetVersion(result.VersionId);
                    var detailZoon = Compute_Expense.Details.Where(x => x.ExpenseID == result.ExpenseId).FirstOrDefault();
                    if (detailZoon.ZoneValue.Count != 1) { throw new Exception(" 当前费用无分区值，或分区数量大于1！"); }
                    bool isChange = false;
                    foreach (var detail in item.ExpenseResult.Where(x => x.ExpenseId == result.ExpenseId))
                    {
                        var tempDetail = Compute_Expense.Details.Where(x => x.Id == detail.ExpenseDetailId)?.FirstOrDefault();
                        if (tempDetail != null
                            && tempDetail.ZoneValue.FirstOrDefault().Fee != detail.IncidentalFee)
                        {
                            isChange = true; break;
                        }
                    }
                    if (isChange)
                    {
                        Compute_Expense.BindBillId = BillId;
                        foreach (var detail in Compute_Expense.Details)
                        {
                            detail.ZoneValue.FirstOrDefault().Fee = item.ExpenseResult.Where(x => x.ExpenseDetailId == detail.Id).FirstOrDefault().IncidentalFee;
                        }
                        computes.Add(Compute_Expense);
                    }
                }
            }
            //有修改项进行处理
            if (computes.Count > 0)
            {
                Dictionary<int, int> expen = new Dictionary<int, int>();
                foreach (var item in computes)
                {
                    int versionId = await _computeExpenseService.AddComputeVersionExpense(item);
                    expen.Add(item.Id, versionId);
                }
                //将新的数据记录返回 VersionId
                List<int> ExpenseId = computes.Select(x => x.Id).Distinct().ToList();
                foreach (var item in finances)
                {
                    var tempExpenseResult = item.ExpenseResult.Where(x => ExpenseId.Contains(x.ExpenseId));
                    foreach (var result in tempExpenseResult)
                    {
                        result.VersionId = expen.Where(y => y.Key == result.ExpenseId).FirstOrDefault().Value;
                    }
                }
            }
            return finances;
        }
        /// <summary>
        /// 订单口岸验证
        /// </summary>
        /// <param name="originalData"></param>
        /// <returns></returns>
        public async Task<IEnumerable<Logistisc_Order>> PortVerify(IEnumerable<Logistisc_Order> originalData, IEnumerable<Logistics_Channel> channels)
        {
            List<int> expenseItemIds = channels.Select(x => x.ExpenseItemId ?? 0).Distinct().ToList();
            var portExpenseItems = await _computeExpenseItemService.GetExpenseItemByZoneMethodMode(ZoneMode.提单落地口岸, expenseItemIds);
            if (portExpenseItems == null || portExpenseItems.Count() == 0) { return originalData; }
            List<int> portExpenseItemIds = portExpenseItems.Select(x => x.Id).Distinct().ToList();
            var tempChannel = channels.Where(x => portExpenseItemIds.Contains(x.ExpenseItemId ?? 0)).ToList();
            foreach (var item in tempChannel)
            {
                var tempOrders = originalData.Where(x => x.OrderChannelUniqueId == item.ThirdPartyUniqueId);
                //考虑放到循环外取数
                List<string> LadingBillNumbers = tempOrders.Select(x => x.LadingBillNumber ?? "").Distinct().ToList();
                var ladingBills = await _LadingBillRepository.GetPortLadingBillsByNumbers(LadingBillNumbers);

                foreach (var order in tempOrders)
                {
                    order.Port = ladingBills.Where(x => x.LadingBillNumber == order.LadingBillNumber)?.FirstOrDefault()?._Destination ?? "";
                }
            }
            return originalData;
        }
        /// <summary>
        /// 计费项分摊方式验证
        /// 订单独立计算验证
        /// </summary>
        /// <returns></returns>
        private async Task<bool> ApportionMethodVerify(IEnumerable<Logistisc_Order> originalData, IEnumerable<finances> finances, string LadingBillNumber)
        {
            bool result = true;
            #region 基础数据处理
            List<QueryExpenseByLadingResult> queryExpenseByLadingResults = new List<QueryExpenseByLadingResult>();
            foreach (var item in finances)
            {
                var temp = item.ExpenseResult.Where(x => x.num == 1 && x.ApportionMethod == ApportionMethodEnum.OrderSeparately).ToList();
                if (temp != null && temp.Count() > 0) { queryExpenseByLadingResults.AddRange(temp); }
            }
            #endregion  
            List<ComputeAmountSouce> souces = new List<ComputeAmountSouce>();
            if (queryExpenseByLadingResults.Count > 0)
            {
                await _TimeTaskRepository.CreateTypeTaskByExpense(originalData, queryExpenseByLadingResults);

            }
            else
            {
                var task = await _TimeTaskRepository.GetFinanceTimeTaskByLading(LadingBillNumber, TaskTypeEnum.OrderSeparately);
                if (task != null) { await _TimeTaskRepository.ClearTypeTask(task.Id); }
            }
            return result;
        }
        /// <summary>
        /// 校验范围订单是否有相同数据
        /// </summary>
        /// <param name="OrderQuerySettingId"></param>
        /// <param name="LadingBillId"></param>
        /// <param name="originalFinances"></param>
        /// <returns></returns>
        public async Task<Tuple<bool, List<ScopeSameInfo>>> ScopeBillVerify(int OrderQuerySettingId, int LadingBillId, IEnumerable<finances> originalFinances)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append($@" select a.* from Finance_Incidental(nolock) a 
	where LadingBillId in 
	(
		select a.Id from Finance_Logistics_LadingBill(nolock) a
		left join Finance_OrderQuerySetting(nolock) b on b.Id=a.OrderQuerySettingId and b.Deleted=0
		where b.NumberType={(int)NumberType.提单号} and b.Numbers=(select Numbers from Finance_OrderQuerySetting(nolock) where Id={OrderQuerySettingId}) and a.IsDelete=0
	) and LadingBillId!={LadingBillId} and a.IsDeleted=0 ");
            var finances = await _unitOfWork.QueryBySqlAsync<Finance_Incidental>(sb.ToString());
            if (finances == null || finances.Count()==0)
            {
                return new Tuple<bool, List<ScopeSameInfo>>(false, new List<ScopeSameInfo>());
            }
            List<int> ExpenseItemIds = originalFinances.Select(x => x.IncidentalType).Distinct().ToList();
            List<ScopeSameInfo> scopeSameInfos = new List<ScopeSameInfo>();
            foreach (var item in ExpenseItemIds)
            {
                //排除独立计算费用 所在的费项
                var originalFinancesVerify = originalFinances.Where(x => x.IncidentalType == item).FirstOrDefault();
                if (originalFinancesVerify.ExpenseResult.Any(x => x.ApportionMethod == ApportionMethodEnum.OrderSeparately)) { continue; }
                //重复数据取数
                var repeat = from a in originalFinancesVerify.ExpenseResult
                             join b in finances on new {a.ExpenseId,a.VersionId } equals new { b.ExpenseId,b.VersionId }
                             where b.ExpenseItemId==item
                             select b;
                var repeatList = repeat.ToList();
                if (repeatList == null || repeatList.Count == 0) { continue; }
                ScopeSameInfo scopeSameInfo = new ScopeSameInfo() { ExpenseItemId = item };
                var LadingBillIds = repeat.Select(x => x.LadingBillId).Distinct().ToList();
                foreach (var billId in LadingBillIds)
                {
                    var tempBill = repeatList.Where(x => x.LadingBillId == billId).ToList();
                    if (tempBill.Count == originalFinancesVerify.ExpenseResult.Count())//数据一致证明费项一样
                    {
                        scopeSameInfo.LadingBillIds.Add(billId);
                    }
                }
                if (scopeSameInfo.LadingBillIds.Any()) { scopeSameInfos.Add(scopeSameInfo); }
            }
            //最终数据返回
            if (scopeSameInfos.Count > 0)
            {
                return new Tuple<bool, List<ScopeSameInfo>>(true, scopeSameInfos);
            }
            else
            {
                return new Tuple<bool, List<ScopeSameInfo>>(false, new List<ScopeSameInfo>());
            }
        }
        #endregion
    }
    #endregion

}
