﻿using SMP.Data.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SMP.Service.Finance
{
    public interface IServiceBillSpreadApplyService
    {
        Task<long> CheckAndAddNewApply(string billName, int? orderRangeId = null);

        Task<bool> UpdateApplySuccess(int id);

        Task<bool> UpdateApplyError(int id, string message);
    }

    public class ServiceBillSpreadApplyService : IServiceBillSpreadApplyService
    {
        IFinance_ServiceBillSpreadApplyRepository _billSpreadApplyRepository;
        public ServiceBillSpreadApplyService(IFinance_ServiceBillSpreadApplyRepository billSpreadApplyRepository)
        {
            _billSpreadApplyRepository = billSpreadApplyRepository;
        }
        public async Task<long> CheckAndAddNewApply(string billName, int? orderRangeId = null)
        {
            var exists = await _billSpreadApplyRepository.IsExistsAsync(i => i.BillName == billName && i.Status == ServiceBillSpreadStatus.成功);
            if (exists)
            {
                throw new Exception($"账单【{billName}】已存在成功的分摊记录");
            }
            Finance_ServiceBillSpreadApply apply = new Finance_ServiceBillSpreadApply
            {
                BillName = billName,
                CreateTime = DateTime.Now,
                OrderRangeId = orderRangeId,
                Status = ServiceBillSpreadStatus.未开始,
            };

            var id = await _billSpreadApplyRepository.InsertAsync(apply);
            return id;
        }

        public async Task<bool> UpdateApplySuccess(int id)
        {
            bool ok = await _billSpreadApplyRepository.UpdateByConditionAsync(i => i.Id == id, new { Status = ServiceBillSpreadStatus.成功, Message = string.Empty });
            return ok;
        }

        public async Task<bool> UpdateApplyError(int id,string message)
        {
            bool ok = await _billSpreadApplyRepository.UpdateByConditionAsync(i => i.Id == id, new { Status = ServiceBillSpreadStatus.失败, Message = message });
            return ok;
        }
    }
}
