﻿using SMP.Data;
using SMP.Data.Compute;
using SMP.Data.Finance;
using SMP.Data.Logistisc;
using SMP.Model;
using SMP.Model.Compute;
using SMP.Model.Dto.Compute;
using SMP.Model.Dto.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.Finance.Dto;
using SMP.Service.OrderRangeQuery;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SMP.Service.Finance
{
    public interface IServiceBillSpreadService
    {
        Task SpreadServiceBill(SpreadServiceBillParam param);
        Task<bool> IsServiceBillOrderRangeSpread(string billName);
    }
    public class ServiceBillSpreadService : IServiceBillSpreadService
    {
        IOrderRangeQueryService _orderRangeQueryService;
        IFinance_ServiceBillResolveSolutionRepository _ServiceBillResolveSolutionRepository;
        IFinance_OrderQuerySettingRepository _orderQuerySettingRepository;
        IFinance_OrderRangeSearchResultRepository _OrderRangeSearchResultRepository;
        IFinance_ServiceBillResolveResultRepository _ServiceBillResolveResultRepository;
        IFinance_ServiceBillShareAmountRepository _ServiceBillShareAmountRepository;
        IFinance_ServiceBillExpenseMappingSolutionRepository _expenseMappingSolutionRepository;
        IFinance_ServiceBillExpenseMappingRepository _expenseMappingRepository;
        IOrderCostService _orderCostService;
        ISysRateService _rateService;
        IView_ServiceBillResultOrderInfoRepository _serviceBillResultOrderInfoRepository;
        ICompute_ExpenseRepository _expenseRepository;
        IFinance_Logistics_LadingBillRepository _ladingBillRepository;
        IView_OrderRangeOrderInfoRepository _orderRangeOrderInfoRepository;
        IFinance_ServiceBillResolveApplyRepository _serviceBillResolveApplyRepository;
        IServiceBillSpreadApplyService _serviceBillSpreadApplyService;
        ILogistisc_OrderRepository _orderRepository;

        public ServiceBillSpreadService(IOrderRangeQueryService orderRangeQueryService, IFinance_OrderQuerySettingRepository orderQuerySettingRepository,
            IFinance_OrderRangeSearchResultRepository orderRangeSearchResultRepository, IFinance_ServiceBillResolveResultRepository ServiceBillResolveResultRepository
            , IFinance_ServiceBillShareAmountRepository serviceBillShareAmountRepository,
            IFinance_ServiceBillExpenseMappingSolutionRepository expenseMappingSolutionRepository, IFinance_ServiceBillExpenseMappingRepository expenseMappingRepository,
            IOrderCostService orderCostService, ISysRateService rateService, IView_ServiceBillResultOrderInfoRepository serviceBillResultOrderInfoRepository,
            ICompute_ExpenseRepository expenseRepository, IFinance_Logistics_LadingBillRepository ladingBillRepository,
            IView_OrderRangeOrderInfoRepository orderRangeOrderInfoRepository, IFinance_ServiceBillResolveApplyRepository serviceBillResolveApplyRepository, IServiceBillSpreadApplyService serviceBillSpreadApplyService,
             ILogistisc_OrderRepository orderRepository
            )
        {
            _orderRangeQueryService = orderRangeQueryService;
            _orderQuerySettingRepository = orderQuerySettingRepository;
            _OrderRangeSearchResultRepository = orderRangeSearchResultRepository;
            _ServiceBillResolveResultRepository = ServiceBillResolveResultRepository;
            _ServiceBillShareAmountRepository = serviceBillShareAmountRepository;
            _expenseMappingSolutionRepository = expenseMappingSolutionRepository;
            _expenseMappingRepository = expenseMappingRepository;
            _orderCostService = orderCostService;
            _rateService = rateService;
            _serviceBillResultOrderInfoRepository = serviceBillResultOrderInfoRepository;
            _expenseRepository = expenseRepository;
            _ladingBillRepository = ladingBillRepository;
            _orderRangeOrderInfoRepository = orderRangeOrderInfoRepository;
            _serviceBillResolveApplyRepository = serviceBillResolveApplyRepository;
            _serviceBillSpreadApplyService = serviceBillSpreadApplyService;
            _orderRepository = orderRepository;
        }

        private async Task<IEnumerable<Finance_ServiceBillExpenseMapping>> GetMappings(int serviceProviderId, string billTypeCode)
        {
            var mappingSolutions = await _expenseMappingSolutionRepository.QueryAsync(m => m.ServiceProviderId == serviceProviderId && m.BillTypeCode == billTypeCode && m.Enabled == true && m.Deleted == false);
            if (mappingSolutions?.Any() != true)
            {
                throw new Exception($"找不到该账单类型(供应商代码:{serviceProviderId},账单类型代码:{billTypeCode})对应的费用映射方案");
            }
            if (mappingSolutions.Count() > 1)
            {
                throw new Exception($"账单类型(供应商代码:{serviceProviderId},账单类型代码:{billTypeCode})存在多个可用的费用映射方案");
            }
            var mappingSolution = mappingSolutions.First();

            var mappings = await _expenseMappingRepository.QueryAsync(m => m.MappingSolutionId == mappingSolution.Id);
            if (mappings?.Any() != true)
                throw new Exception($"找不到费用映射明细,ServiceProviderId:{serviceProviderId},BillTypeCode:{billTypeCode}");

            return mappings;
        }
        public async Task SpreadServiceBill(SpreadServiceBillParam param)
        {
            //账单摊分思路:
            //1.如果账单解析结果的单号类型为跟踪号/系统单号/转单号，则视为末端派送账单，
            //末端派送账单的摊分逻辑已经和渠道关联，所以这部分账单数据通过订单渠道获取摊分逻辑进行摊分。
            //2.如果账单解析结果的单号类型为提单，则视为非末端账单(提单账单)，这类账单的摊分逻辑是关联在提单上的，
            //因此直接在提单上获取摊分方案进行摊分
            //3.如果是无单号类型的账单，则需要指定订单范围搜索设置,并通过指定摊分逻辑进行摊分。

            ValidateParam(param);
            var spreadApplyId = await _serviceBillSpreadApplyService.CheckAndAddNewApply(param.BillName, param.OrderRangeSettingId);
            try
            {
                var billResolveResults = await _serviceBillResultOrderInfoRepository.QueryAsync(i => i.BillName == param.BillName && i.DataType == ServiceBillDataType.金额);
                if ((param.OrderRangeSettingId == null || param.OrderRangeSettingId == 0) && billResolveResults?.Any() != true)
                {
                    throw new Exception($"找不到账单解析结果");
                }


                //1.以订单号的每种种费用类型做摊分,无需均摊逻辑，直接转换为费用金额
                var orderNumberResults = billResolveResults.Where(i => i.NumberType == NumberType.系统单号 || i.NumberType == NumberType.跟踪号 || i.NumberType == NumberType.转单号);
                if (orderNumberResults?.Any() == true)
                {
                    await SpreadServiceBillByOrderNumber(orderNumberResults, (int)spreadApplyId);
                }

                //2.以提单号的每种费用类型做摊分
                var ladingBillNumberResults = billResolveResults.Where(i => i.NumberType == NumberType.提单号);
                if (ladingBillNumberResults?.Any() == true)
                {
                    await SpreadServiceBillByLadingBill(ladingBillNumberResults, (int)spreadApplyId);
                }

                //3.以订单范围均摊
                var noneNumberResults = await _ServiceBillResolveResultRepository.QueryAsync(i => i.BillName == param.BillName && i.NumberType == NumberType.无);
                if (noneNumberResults?.Any() == true)
                {
                    if (param.OrderRangeSettingId == null || param.OrderRangeSettingId.Value == 0)
                    {
                        throw new Exception("该账单包含无单号的汇总费用，但未指定订单搜索范围方案");
                    }
                    await SpreadServiceBillByOrderRange(noneNumberResults, param.OrderRangeSettingId.Value, (int)spreadApplyId);
                }

                var boxNumberResult = billResolveResults.Where(i => i.NumberType == NumberType.箱号);
                if (boxNumberResult?.Any() == true)
                {
                    throw new Exception("暂不支持账单号码为箱号的均摊");
                }

                await _serviceBillSpreadApplyService.UpdateApplySuccess((int)spreadApplyId);
            }
            catch (Exception ex)
            {
                await _serviceBillSpreadApplyService.UpdateApplyError((int)spreadApplyId, ex.Message);
                throw;
            }
        }

        private async Task SpreadServiceBillByOrderRange(IEnumerable<Finance_ServiceBillResolveResult> noneNumberResults, int orderRangeSettingId, int spreadApplyId)
        {
            var currencys = noneNumberResults.Select(i => i.Currency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);


            Finance_OrderQuerySetting orderRangeSetting = await _orderQuerySettingRepository.GetAsync(orderRangeSettingId);
            if (orderRangeSetting == null)
            {
                throw new Exception($"找不到设置信息,Id:{orderRangeSettingId}");
            }

            if (orderRangeSetting.IsExecuted == null || orderRangeSetting.IsExecuted.Value == false)
            {
                int orderCount = await _orderRangeQueryService.InsertOrderRange(orderRangeSetting);
                if (orderCount <= 0)
                {
                    throw new Exception($"订单范围搜索结果为0,Id:{orderRangeSettingId}");
                }
            }
            var rangeOrderInfos = await _orderRangeOrderInfoRepository.QueryAsync(i => i.RangeSettingId == orderRangeSettingId);
            if (rangeOrderInfos?.Any() != true)
                throw new Exception($"订单范围搜索关联的订单数为零");

            var allSystemNos = rangeOrderInfos.Select(i => i.SystemNo);
            bool clearDataOk = await _orderCostService.ClearApportionComputeRecord(allSystemNos, CostType.账单成本);
            if (!clearDataOk)
            {
                throw new Exception($"清除旧数据失败");
            }

            var dataNameGroups = noneNumberResults.GroupBy(i => i.BillDataName);

            List<View_ServiceBillResultOrderInfo> allBillResoveOrderInfos = new List<View_ServiceBillResultOrderInfo>();
            foreach (var dataNameGroup in dataNameGroups)
            {
                int dataNameGroupCount = dataNameGroup.Count();
                if (dataNameGroupCount > 1)
                {
                    throw new Exception($"账单解析结果中存在{dataNameGroupCount}条名为【{dataNameGroup.Key}】的费用");
                }
                foreach (var order in rangeOrderInfos)
                {
                    var first = dataNameGroup.First();
                    View_ServiceBillResultOrderInfo billResultOrderInfo = GetResolveOrderInfo(first, order);
                    allBillResoveOrderInfos.Add(billResultOrderInfo);
                }
            }

            var billProviderId = noneNumberResults.First().ServiceProviderId;
            var billTypeCode = noneNumberResults.First().BillTypeCode;
            var mappings = await GetMappings(billProviderId, billTypeCode);
            checkBillDataName(mappings);

            var expenseIds = mappings.Select(i => i.ExpenseId);
            var allExpenses = await _expenseRepository.QueryAsync(i => i.Id.In(expenseIds));

            var expenseMappingGroups = mappings.GroupBy(i => i.ExpenseId);

            foreach (var expenseMappingGroup in expenseMappingGroups)
            {
                int expenseItemId = expenseMappingGroup.First().ExpenseItemId;
                if (!expenseMappingGroup.All(i => i.ExpenseItemId == expenseItemId))
                {
                    throw new Exception($"费用映射包含{expenseMappingGroup.Count()}个相同系统费用(Id:{expenseMappingGroup.Key})，但费用的费项ID不一致！");
                }

                var billDataNames = expenseMappingGroup.Select(i => i.BillDataName);
                var billDataCurrencys = noneNumberResults.Where(r => billDataNames.Contains(r.BillDataName)).GroupBy(i => i.Currency).Select(g => g.Key).Distinct();

                var expense = allExpenses.First(i => i.Id == expenseMappingGroup.Key);
                if (billDataCurrencys.Count() > 1)
                {
                    string billDataNamesStr = string.Join(",", billDataNames);
                    throw new Exception($"系统费用(${expense.Name})由账单费用({billDataNames})组成，但费用直接货币不一致！");
                }


                var expenseResults = noneNumberResults.Where(i => billDataNames.Contains(i.BillDataName));
                var first = expenseResults.First();
                DateTime? billTime = first.BillTime;
                if (!billTime.HasValue)
                {
                    throw new Exception("账单时间不能为空");
                }
                var currency = billDataCurrencys.First();
                var rate = sysRates?.FirstOrDefault(i => i.Currency == currency && (i.StartDate <= first.BillTime && i.EndDate >= first.BillTime));

                decimal originAmount = 0m;
                foreach (var dataName in billDataNames)
                {
                    var firstResult = expenseResults.FirstOrDefault(i => i.BillDataName == dataName);
                    originAmount += firstResult == null ? 0 : firstResult.Data;
                }

                await SpreadExpenseByLadingBill(rangeOrderInfos, originAmount, currency, billTime.Value, expense, expenseItemId, rate, spreadApplyId);
            }

        }

        private View_ServiceBillResultOrderInfo GetResolveOrderInfo(Finance_ServiceBillResolveResult first, View_OrderRangeOrderInfo order)
        {
            var info = new View_ServiceBillResultOrderInfo
            {
                BillCurrency = first.Currency,
                BillData = first.Data,
                BillDataName = first.BillDataName,
                BillName = first.BillName,
                BillNumber = first.Number,
                BillTime = first.BillTime,
                BillTypeCode = first.BillTypeCode,
                BoxNumber = order.BoxNumber,
                CustomerId = order.CustomerId,
                CustomerName = order.CustomerName,
                Height = order.Height,
                LadingBillNumber = order.LadingBillNumber,
                Length = order.Length,
                NumberType = first.NumberType,
                OPWeight = order.OPWeight,
                OrderChannelUniqueId = order.OrderChannelUniqueId,
                OrderTime = order.OrderTime,
                ReceiverCity = order.ReceiverCity,
                ReceiverCountryCode = order.ReceiverCountryCode,
                ReceiverCountryName = order.ReceiverCountryName,
                ReceiverState = order.ReceiverState,
                ReceiverStreet = order.ReceiverStreet,
                ReceiverZipCode = order.ReceiverZipCode,
                SystemNo = order.SystemNo,
                TrackingNumber = order.TrackingNumber,
                WeightRate = order.WeightRate,
                Width = order.Width
            };
            return info;
        }

        /// <summary>
        /// 提单形式的账单数据，通过提单去关联摊分逻辑，然后通过摊分逻辑，关联所有费用和映射
        /// </summary>
        /// <param name="ladingBillNumberResults"></param>
        /// <returns></returns>
        private async Task SpreadServiceBillByLadingBill(IEnumerable<View_ServiceBillResultOrderInfo> ladingBillNumberResults, int spreadApplyId)
        {
            var currencys = ladingBillNumberResults.Select(i => i.BillCurrency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);

            var ladingBillResultGroups = ladingBillNumberResults.GroupBy(i => i.LadingBillNumber);
            var allLadingBillNumbers = ladingBillResultGroups.Select(i => i.Key).Where(i => !string.IsNullOrWhiteSpace(i));
            var clearDataOk = await _orderCostService.ClearApportionComputeRecord(allLadingBillNumbers, (int)CostType.账单成本);
            if (!clearDataOk)
            {
                throw new Exception("旧数据清除失败");
            }
            if (allLadingBillNumbers?.Any() != true)
            {
                throw new Exception("数据无提单号");
            }

            string billTypeCode = ladingBillNumberResults.First().BillTypeCode;
            int serviceProviderId = ladingBillNumberResults.First().ServiceProviderId;
            var allMappings = await GetMappings(serviceProviderId, billTypeCode);

            //同一个账单费用名称不允许对应多个系统费用，因为不知道该如何摊分费用
            //但一个系统费用可以有多个账单费用映射。摊分时只需将账单费用求和即可
            checkBillDataName(allMappings);
            var multiExpenseGroups = allMappings.GroupBy(m => m.BillDataName).Where(g => g.Count() > 1);
            if (multiExpenseGroups?.Any() == true)
            {
                var names = string.Join(",", multiExpenseGroups.Select(i => i.Key));
                throw new Exception($"不允许同一个账单费用对应多个系统费用的情况，账单费用名称:{names}");
            }

            var allExpenseIds = allMappings.Select(m => m.ExpenseId);
            var allExpenses = await _expenseRepository.QueryAsync(e => e.Id.In(allExpenseIds));

            var allLadingBillOrders = await _orderRepository.GetOrdersByLadingNumbers(allLadingBillNumbers);

            foreach (var ladingBillGroup in ladingBillResultGroups)
            {
                var ladingBillOrders = allLadingBillOrders.Where(o => o.LadingBillNumber == ladingBillGroup.Key);

                //摊分每个系统费用，一个系统费用可能有多个账单费用组成
                var expenseMappingGroups = allMappings.GroupBy(i => i.ExpenseId);

                foreach (var expenseMappingGroup in expenseMappingGroups)
                {
                    int expenseItemId= expenseMappingGroup.First().ExpenseItemId;
                    if(!expenseMappingGroup.All(i=>i.ExpenseItemId== expenseItemId))
                    {
                        throw new Exception($"费用映射包含{expenseMappingGroup.Count()}个相同系统费用(Id:{expenseMappingGroup.Key})，但费用的费项ID不一致！");
                    }

                    var expenseBillDataNames = expenseMappingGroup.Select(m => m.BillDataName);
                    var expenseBillResults = ladingBillGroup.Where(i => expenseBillDataNames.Contains(i.BillDataName));
                    string currency = expenseBillResults.First().BillCurrency;
                    var expense = allExpenses.First(e => e.Id == expenseMappingGroup.Key);
                    bool allResultSameCurrency = expenseBillResults.All(r => r.BillCurrency == currency);
                    if (!allResultSameCurrency)
                    {
                        string billDataNames = string.Join(",", expenseBillDataNames);
                        throw new Exception($"系统费用(${expense.Name})由账单费用({billDataNames})组成，但费用直接货币不一致！");
                    }


                    var first = ladingBillGroup.First();
                    if (!first.BillTime.HasValue)
                    {
                        throw new Exception("账单时间不能为空");
                    }
                    var rate = sysRates?.FirstOrDefault(i => i.Currency == currency && (i.StartDate <= first.BillTime && i.EndDate >= first.BillTime));


                    decimal originAmount = 0m;
                    foreach (var dataName in expenseBillDataNames)
                    {
                        var firstResult = expenseBillResults.FirstOrDefault(i => i.BillDataName == dataName);
                        originAmount += firstResult == null ? 0 : firstResult.BillData;
                    }

                    expenseBillResults.Sum(i => i.BillData);
                    DateTime billTime = first.BillTime.Value;
                    await SpreadExpenseByLadingBill(ladingBillOrders, originAmount, currency, billTime, expense, expenseItemId, rate, spreadApplyId);
                }


            }
        }

        /// <summary>
        /// 检查一套映射中，一个账单费用是否出现多次。无论出现多次是因为对应同一个系统费用，还是多个系统费用，皆不允许，因为无法摊分
        /// </summary>
        /// <param name="allMappings"></param>
        /// <exception cref="Exception"></exception>
        private void checkBillDataName(IEnumerable<Finance_ServiceBillExpenseMapping> allMappings)
        {
            var multiExpenseGroups = allMappings.GroupBy(m => m.BillDataName).Where(g => g.Count() > 1);
            if (multiExpenseGroups?.Any() == true)
            {
                var names = string.Join(",", multiExpenseGroups.Select(i => i.Key));
                throw new Exception($"不允许同一个账单费用对应多个系统费用的情况，账单费用名称:{names}");
            }
        }

        private async Task<bool> SpreadExpenseByLadingBill(IEnumerable<Logistisc_Order> ladingBillOrders, decimal originAmount, string originCurrency, DateTime billTime, Compute_Expense expense,int expenseItemId, Sys_Rate rate, int spreadApplyId)
        {
           
            var firstData = ladingBillOrders.First();

            List<Compute_ExpenseAmount> spreadExpenseAmounts = new List<Compute_ExpenseAmount>();
            List<Logistisc_Order> orders = new List<Logistisc_Order>();

            var totalWeight = ladingBillOrders.Sum(i => i.OPWeight);
            foreach (var r in ladingBillOrders)
            {
                Logistisc_Order order = r;
                r.WeightRate = r.OPWeight / (decimal)totalWeight;
                orders.Add(order);
            }

            decimal settlementAmount = originAmount;
            string settlementCurrency = "CNY";
            decimal settlementRate = 1;
            if (rate != null)
            {
                settlementCurrency = rate.Currency;
                settlementAmount = originAmount * rate.ToCNY;
                settlementRate = rate.ToCNY;
            }

            ComputeAmountResult amountResult = CreateComputeAmountResult(firstData, expense, billTime, originAmount, originCurrency, settlementCurrency, settlementRate, settlementAmount, expenseItemId);
            string ladingbillNumber = ladingBillOrders.FirstOrDefault()?.LadingBillNumber;
            ApportionComputeRecordInput input = new ApportionComputeRecordInput
            {
                orders = orders,
                amountResult = amountResult,
                CostType = CostType.账单成本,
                LadingBillNumber= ladingbillNumber,
            };

            var expenseAmounts = _orderCostService.ApportionComputeRecord(input);
            if (expenseAmounts?.Any() == true)
            {
                spreadExpenseAmounts.AddRange(expenseAmounts);
            }
            if (spreadExpenseAmounts.Count > 0)
            {
                spreadExpenseAmounts.ForEach(i =>
                {
                    i.ServiceBillSpreadApplyId = spreadApplyId;
                });

                var insertOk = await _orderCostService.InsertApportionComputeRecord(spreadExpenseAmounts);
                return insertOk;
            }
            else return false;
        }

        private async Task SpreadServiceBillByOrderNumber(IEnumerable<View_ServiceBillResultOrderInfo> billDataResults, int spreadApplyId)
        {
            var billTypeCode = billDataResults.First().BillTypeCode;
            var serviceProviderId = billDataResults.First().ServiceProviderId;

            var allMappings = await GetMappings(serviceProviderId, billTypeCode);

            var allExpenseIds = allMappings.Select(i => i.ExpenseId);

            var allExpenses = await _expenseRepository.QueryAsync(e => e.Id.In(allExpenseIds));
            if (allExpenses?.Any() != true)
            {
                throw new Exception("找不到系统费用信息");
            }

            var orderNumberDataGroups = billDataResults.GroupBy(i => i.BillDataName);
            foreach (var expenseGroup in orderNumberDataGroups)
            {
                string currentBillDataName = expenseGroup.Key;
                var groupBillDataResults = expenseGroup.ToList();

                var mappings = allMappings?.Where(m => m.BillDataName == currentBillDataName && allExpenseIds.Contains(m.ExpenseId));
                if (mappings?.Any() != true)
                    throw new Exception($"找不到账单费用【{currentBillDataName}】与渠道均摊的费用之间的映射");
                else if (mappings.Count() > 1)
                    throw new Exception($"账单费用【{currentBillDataName}】与渠道均摊的费用存在多条映射,系统费用ID:{string.Join(",", mappings.Select(m => m.ExpenseId))}");

                var mapping = mappings.First();
                var expense = allExpenses.First(e => e.Id == mapping.ExpenseId);

                await SpreadExpenseByOrderNumber(groupBillDataResults, expense, spreadApplyId);

            }

        }


        private void ValidateParam(SpreadServiceBillParam param)
        {
            if (param == null)
                throw new Exception("参数不能为空");
            if (string.IsNullOrWhiteSpace(param.BillName))
                throw new Exception("账单名称不能为空");
        }

        ///按订单号均摊一种费用
        private async Task<bool> SpreadExpenseByOrderNumber(IEnumerable<View_ServiceBillResultOrderInfo> orderNumberResults, Compute_Expense expense, int spreadApplyId)
        {
            var billTypeCode = orderNumberResults.First().BillTypeCode;

            var allSystemNos = orderNumberResults.Select(i => i.SystemNo);
            bool clearDataOk = await _orderCostService.ClearApportionComputeRecord(allSystemNos, CostType.账单成本);
            if (!clearDataOk)
            {
                throw new Exception($"清除旧数据失败");
            }

            var currencys = orderNumberResults.Select(i => i.BillCurrency).Distinct();
            var sysRates = await _rateService.GetRates(currencys);

            List<Compute_ExpenseAmount> spreadExpenseAmounts = new List<Compute_ExpenseAmount>();
            foreach (var r in orderNumberResults)
            {
                Logistisc_Order order = r;
                List<Logistisc_Order> orders = new List<Logistisc_Order> { order };

                if (r.BillTime == null)
                    throw new Exception("账单解析数据中的账单时间不能为空");


                var sysRate = sysRates?.FirstOrDefault(rate => rate.Currency == r.BillCurrency && (rate.StartDate <= r.BillTime.Value && rate.EndDate >= r.BillTime.Value));
                decimal settlementRate = sysRate != null ? sysRate.ToCNY : 1;
                string settlementCurrency = sysRate != null ? "CNY" : r.BillCurrency;
                var amt = new Compute_ExpenseAmount()
                {
                    Number = r.SystemNo,
                    ExpenseId = expense.Id,
                    ExpenseDetailId = 0,//实际账单不用关联费用明细
                    VersionId = 0,//实际账单不用关联版本
                    ExpenseItemId = 0,//expenseItemId,
                    TransactionTime = r.BillTime.Value,
                    Currency = r.BillCurrency,
                    AmountinOriginalCurrency = r.BillData,
                    SettlementRate = settlementRate,
                    SettlementCurrency = r.BillCurrency,
                    SettlementAmount = r.BillData,
                    FeeDescription = expense.Name,
                    BillTypeCode = billTypeCode,
                    CostType = CostType.账单成本,
                    CreateTime = DateTime.Now,
                };
                spreadExpenseAmounts.Add(amt);
            }



            if (spreadExpenseAmounts?.Any() == true)
            {
                spreadExpenseAmounts.ForEach(i =>
                {
                    i.ServiceBillSpreadApplyId = spreadApplyId;
                });
                var insertOk = await _orderCostService.InsertApportionComputeRecord(spreadExpenseAmounts);
                return insertOk;
            }
            else return false;

        }

        private ComputeAmountResult CreateComputeAmountResult(Logistisc_Order firstOrder, Compute_Expense expense, DateTime billTime, decimal originAmount, string billCurrency, string settlementCurrency, decimal settlementRate, decimal settlementAmount,int expenseItemId)
        {
            ComputeAmountResult result = new ComputeAmountResult
            {
                Amount = settlementAmount,
                AmountinOriginalCurrency = originAmount,
                BillCurrency = billCurrency,
                BillNumber = firstOrder.SystemNo,
                Currency = settlementCurrency,
                DetailId = expense.Id,
                ExpenseName = expense.Name,
                ExpenseVersion = billTime,
                SettlementRate = settlementRate,
                VersionId = 0,
                ExpenseId = expense.Id,
                ExpenseItemId = expenseItemId,
                ApportionMethod = expense.ApportionMethod,
            };
            return result;
        }

        public Task<bool> IsServiceBillOrderRangeSpread(string billName)
        {
            var existsNonNumberResult = _ServiceBillResolveResultRepository.IsExistsAsync(i => i.BillName == billName && i.NumberType == NumberType.无);
            return existsNonNumberResult;
        }
    }
}
