﻿using SMP.Data.Finance;
using SMP.Model.Dto.Finance;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Data.Infrastructure;

namespace SMP.Service.Finance
{
    #region Iservices
    public interface ISpreadlogicService
    {
        Task<long> Create(Finance_Spreadlogic model);
        Task<bool> Modify(Finance_Spreadlogic model);
        Task<int> Remove(int Id);
        Task<PageResult<Finance_Spreadlogic>> QueryLogicPage(QueryModel queryModel);

        Task<IEnumerable<Finance_SpreadlogicDetail>> GetSpreadlogicDetails(int SpreadlogicId);
        Task<IEnumerable<Finance_SpreadlogicDetail>> GetSpreadlogicDetails(List<int> SpreadlogicId);
        Task<IEnumerable<SpreadlogicDetailDto>> QueryLogicDetailForWeb(int SpreadlogicId = 0, int ExpenseItem = 0);
        Task<IEnumerable<Finance_Spreadlogic>> GetLogicList();
    }
    #endregion
    #region services
    public class SpreadlogicService : ISpreadlogicService
    {
        private readonly IFinance_SpreadlogicDetailRepository _repository;
        public SpreadlogicService(IFinance_SpreadlogicDetailRepository repository)
        {
            this._repository = repository;
        }
        public async Task<PageResult<Finance_Spreadlogic>> QueryLogicPage(QueryModel queryModel)
        {
            return await _repository.QueryLogicPage(queryModel);
        }
        public async Task<long> Create(Finance_Spreadlogic model)
        {
            var logicId = await _repository.CreateSpreadlogic(model);
            if (model.details != null && model.details.Count() > 0)
            {
                var list = model.details.ToList();
                list.ForEach(x => { x.SpreadlogicId = (int)logicId; });
                await _repository.BulkToDBAsync(list);
            }
            return logicId;
        }
        public async Task<bool> Modify(Finance_Spreadlogic model)
        {
            var result = await _repository.UpdateSpreadlogic(model);
            string sql = $"delete Finance_SpreadlogicDetail where SpreadlogicId={model.Id} ";
            await _repository.ExecuteSqlAsync(sql);
            if (model.details != null && model.details.Count() > 0)
            {
                if (model.details.Any(x => x.SpreadlogicId == 0)) { model.details.ToList().ForEach(x => { x.SpreadlogicId = model.Id; }); }
                await _repository.BulkToDBAsync(model.details.ToList());
            }
            return result;
        }
        public async Task<int> Remove(int Id)
        {
            string sql = string.Format(@"delete Finance_SpreadlogicDetail where SpreadlogicId={0}
delete Finance_Spreadlogic where Id={0}", Id);
            return await _repository.ExecuteSqlAsync(sql);
        }
        public async Task<IEnumerable<Finance_SpreadlogicDetail>> GetSpreadlogicDetails(int SpreadlogicId)
        {
            return await _repository.QueryAsync(x => x.SpreadlogicId == SpreadlogicId);
        }
        public async Task<IEnumerable<Finance_SpreadlogicDetail>> GetSpreadlogicDetails(List<int> SpreadlogicId)
        {
            return await _repository.QueryBySqlAsync($"select * from Finance_SpreadlogicDetail where SpreadlogicId in ({string.Join(',',SpreadlogicId)})");
        }
        public async Task<IEnumerable<SpreadlogicDetailDto>> QueryLogicDetailForWeb(int SpreadlogicId = 0, int ExpenseItem = 0)
        {
            return await _repository.QueryLogicDetailForWeb(SpreadlogicId, ExpenseItem);
        }
        public async Task<IEnumerable<Finance_Spreadlogic>> GetLogicList()
        {
            return await _repository.QueryAsync<Finance_Spreadlogic>(" select *from Finance_Spreadlogic");
        }
    }
    #endregion

}
