﻿using SMP.Model;
using XYY.Common.Standard;
using SMP.Data;
namespace SMP.Service
{
    public interface ISysCurrencyService
    {
        Task<PageResult<Sys_Currency>> QueryCurrency(QueryModel queryModel);
        Task UpdateSysCurrency(Sys_Currency currency);
        Task InsertSysCurrency(Sys_Currency currency); 
        Task<IEnumerable<Sys_Currency>> GetSysCurrencyList();
    }

    public class SysCurrencyService : ISysCurrencyService
    {
        public SysCurrencyService(ISys_CurrencyRepository currencyRepository)
        {
            CurrencyRepository = currencyRepository;
        }

        public ISys_CurrencyRepository CurrencyRepository { get; }
         

        public async Task InsertSysCurrency(Sys_Currency currency)
        {
            Ensure.NotNull(currency, "币种");
            Ensure.NotNullOrEmpty(currency.Name, "币种简码");
            Ensure.NotNullOrEmpty(currency.Description, "币种名称");
            currency.IsEnable = true;
            if (await CurrencyRepository.IsExistsAsync(x => x.Name == currency.Name))
                throw new Exception($"添加币种失败，{currency.Name}已存在系统内");
            await CurrencyRepository.InsertAsync(currency);
        }

        public async Task<PageResult<Sys_Currency>> QueryCurrency(QueryModel queryModel)
        {
            if (queryModel.QueryParamer != null
                && queryModel.QueryParamer.Count > 0
                )
            {
                var serach = queryModel.QueryParamer.FirstOrDefault(x => x.Filed == "search");
                if (serach != null)
                {
                    queryModel.QueryParamer.Remove(serach);
                    string sql = $@"select * from Sys_Currency(nolock) where Name Like '%{serach.Value}%' or Description ='%{serach.Value}%' or Symbol='%{serach.Value}%'";
                    return await CurrencyRepository.QueryPagerAsync(queryModel, sql);
                }
                else
                    return await CurrencyRepository.QueryPagerAsync(queryModel);
            }
            else
                return await CurrencyRepository.QueryPagerAsync(queryModel);

        }

        public async Task UpdateSysCurrency(Sys_Currency currency)
        {
            Ensure.NotNull(currency, "币种");
            Ensure.NotNullOrEmpty(currency.Name, "币种简码");
            Ensure.NotNullOrEmpty(currency.Description, "币种名称");
            if (currency.Id == 0)
                throw new Exception($"未找到{currency.Name}的Id");

            if (await CurrencyRepository.IsExistsAsync(x => x.Name == currency.Name && x.Id != currency.Id))
                throw new Exception($"更新币种失败，{currency.Name}已存在系统内");

            await CurrencyRepository.UpdateAsync(currency);
        }
        public async Task<IEnumerable<Sys_Currency>> GetSysCurrencyList()
        {
            return await CurrencyRepository.QueryAsync();
        }
    }
}