﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Data;
using SMP.Model;
using StackExchange.Redis;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Data.Redis;

namespace SMP.Service
{
    public interface ISysRateService
    {
        Task<PageResult<Sys_Rate>> QueryRate(QueryModel queryModel);
        Task UpdateSysRate(Sys_Rate Rate);
        Task InsertSysRate(Sys_Rate Rate);
        Task DeleteSysRate(Sys_Rate Rate);

        Task<Sys_Rate> QueryRate(DateTime time, int id);

        Task<IEnumerable<Sys_Rate>> GetRates(string currency);
        Task<IEnumerable<Sys_Rate>> GetRates(IEnumerable<string> currencys);
    }

    public class SysRateService : ISysRateService
    {
        ISys_RateRepository _RateRepository;
        ISys_CurrencyRepository currencyRepository;
        private string _cacheKey = "$Rate_";
        IDistributedCache distributedCache;
        IDatabase redisDb = null;
        IServer redisService = null;


        public SysRateService(ISys_RateRepository RateRepository, SMP.Common.WebRegion.RedisHelper helper, ISys_CurrencyRepository currencyRepository, IDistributedCache distributedCache)
        {
            _RateRepository = RateRepository;
            redisDb = helper.GetDatabase();
            redisService = helper.GetServer();
            this.currencyRepository = currencyRepository;
            this.distributedCache = distributedCache;
        }

        public async Task DeleteSysRate(Sys_Rate Rate)
        {
            await ClearCache(Rate.Currency);
            await _RateRepository.DeleteAsync(Rate);
        }

        public async Task InsertSysRate(Sys_Rate Rate)
        {
            if (string.IsNullOrWhiteSpace(Rate.Currency))
                throw new Exception("币种不可为空");


            await ClearCache(Rate.Currency);
            await _RateRepository.InsertAsync(Rate);
        }

        public async Task<PageResult<Sys_Rate>> QueryRate(QueryModel queryModel)
        {
            if (queryModel.QueryParamer != null
               && queryModel.QueryParamer.Count > 0
               )
            {
                var serach = queryModel.QueryParamer.FirstOrDefault(x => x.Filed == "search");
                var date = queryModel.QueryParamer.FirstOrDefault(x => x.Filed == "Date");
                if (date != null)
                {
                    queryModel.QueryParamer.Add(new QueryParamer { Filed = "StartDate", Method = "LeE", Value = date.Value });
                    queryModel.QueryParamer.Add(new QueryParamer { Filed = "EndDate", Method = "GrE", Value = date.Value });
                    queryModel.QueryParamer.Remove(date);
                }
                if (serach != null)
                {
                    queryModel.QueryParamer.Remove(serach);
                    string sql = $@"select * from Sys_Rate(nolock) where Currency like '%{serach.Value}%'";
                    return await _RateRepository.QueryPagerAsync(queryModel, sql);
                }
                else
                    return await _RateRepository.QueryPagerAsync(queryModel);
            }
            else
                return await _RateRepository.QueryPagerAsync(queryModel);
        }

        /// <summary>
        /// 取时间内的汇率
        /// </summary>
        /// <param name="time"></param>
        /// <param name="curreny"></param>
        /// <returns></returns>
        public async Task<Sys_Rate> QueryRate(DateTime time, int id)
        {
            DateTime date = new DateTime(time.Year, time.Month, time.Day);
            Sys_Rate resultRate = null;
            string dataKey = _cacheKey + id + "_" + date.ToString_yyyyMMdd();
            string val = distributedCache.GetString(dataKey);
            if (!string.IsNullOrEmpty(val))
            {
                try
                {
                    resultRate = Newtonsoft.Json.JsonConvert.DeserializeObject<Sys_Rate>(val);
                    //从缓存中取数据
                    return resultRate;
                }
                catch
                {
                    //保存失败时，我们忽略错误，从数据库中获取
                }
            }
            //如果没有被返回的话，我们从数据库中取值并赋值
            string sql = $@"select * from Sys_Rate(nolock) a join Sys_Currency(nolock)b on a.Currency=b.Name
                                where b.Id= @id  and StartDate<=@time  and EndDate>=@time and a.IsEnable=1";
            var rate = (await _RateRepository.QueryBySqlAsync(sql, new { @time = date, id = id })).FirstOrDefault();
            if (rate == null)
            {
                var curreny = await currencyRepository.GetAsync(id);
                return null;
            }

            distributedCache.SetString(dataKey, Newtonsoft.Json.JsonConvert.SerializeObject(rate));
            return rate;

        }

        public async Task ClearCache(string curreny)
        {
            var currenyModel = (await currencyRepository.QueryAsync(x => x.Name == curreny)).FirstOrDefault();
            if (currenyModel != null)
                await ClearCache(currenyModel.Id);

        }

        private async Task ClearCache(int currenyId)
        {
            string key = _cacheKey + currenyId + "*";
            await redisDb.DeleteScanKey(null, key);
        }

        public async Task UpdateSysRate(Sys_Rate Rate)
        {
            await ClearCache(Rate.Currency);
            await _RateRepository.UpdateAsync(Rate);
        }

        public async Task<IEnumerable<Sys_Rate>> GetRates(string currency)
        {
            var rates = await _RateRepository.QueryAsync(r => r.Currency == currency);
            return rates;
        }

        public async Task<IEnumerable<Sys_Rate>> GetRates(IEnumerable<string> currencys)
        {
            if (currencys?.Any() != true)
                return new List<Sys_Rate>();

            var rates = await _RateRepository.QueryAsync(r => currencys.Contains(r.Currency));
            return rates;
        }
    }
}
