﻿using SMP.Data.Compute;
using SMP.Data.Finance;
using SMP.Model.Dto.Reconcile;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.Compute;
using SMP.Service.OrderRangeQuery;
using SMP.Service.Reconcile.Dto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Reconcile
{
    public interface IReconcileService
    {
        //Task<int> SaveReconcileSolution(ReconcileSolutionDto dto);
        //Task<ReconcileSolutionDto> LoadReconcileSolution(LoadReconcileSolutionParam param);
        Task<ReconcileDisplayResult> Reconcile(QueryModel queryModel);
        //Task<IEnumerable<ReconcileSumAmount>> GetSumAmount(ReconcileParam param);
        Task<int> RefreshReconcileData(RefreshReconcileDataParam param);
    }

    public class ReconcileService : IReconcileService
    {
        IFinance_ServiceBillExpenseMappingRepository _ServiceBillExpenseMappingRepository;
        IFinance_ServiceBillResolveSolutionRepository _ServiceBillResolveSolutionRepository;
        IFinance_ServiceBillResolveApplyRepository _ServiceBillResolveApplyRepository;
        //IViewServiceBillExpenseRepository _ViewServiceBillExpenseRepository;
        ICompute_ExpenseRepository _expenseRepository;
        IOrderRangeQueryService _orderRangeQueryService;
        IView_OrderRangeExpenseRepository _viewOrderRangeExpenseRepository;
        IFinance_ServiceBillResolveResultRepository _ServiceBillResolveResultRepository;

        IFinance_ReconcileDataRepository _reconcileDataRepository;

        private const string expenseMergeChar = "✚";
        public ReconcileService(IFinance_ServiceBillResolveSolutionRepository serviceBillResolveSolutionRepository,
             IFinance_ServiceBillResolveApplyRepository serviceBillResolveSolutionApplyRepository,
            //IViewServiceBillExpenseRepository viewServiceBillExpenseRepository, 
            ICompute_ExpenseRepository expenseRepository, IOrderRangeQueryService orderRangeQueryService,
            IView_OrderRangeExpenseRepository viewOrderRangeExpenseRepository, IFinance_ServiceBillResolveResultRepository serviceBillResolveResultRepository, IFinance_ServiceBillExpenseMappingRepository serviceBillExpenseMappingRepository,
            IFinance_ReconcileDataRepository reconcileDataRepository
            )
        {
            _ServiceBillExpenseMappingRepository = serviceBillExpenseMappingRepository;
            _ServiceBillResolveSolutionRepository = serviceBillResolveSolutionRepository;
            _ServiceBillResolveApplyRepository = serviceBillResolveSolutionApplyRepository;
            //_ViewServiceBillExpenseRepository = viewServiceBillExpenseRepository;
            _expenseRepository = expenseRepository;
            _orderRangeQueryService = orderRangeQueryService;
            _viewOrderRangeExpenseRepository = viewOrderRangeExpenseRepository;
            _ServiceBillResolveResultRepository = serviceBillResolveResultRepository;
            _reconcileDataRepository = reconcileDataRepository;
        }


        //private Task PrepareReconcileData(string billName,int mappingSolutionId,bool useOrderRange,int? orderRangeSettingId=null)
        //{
        //    if (string.IsNullOrWhiteSpace(billName))
        //        throw new Exception("账单名字不能为空");
        //    if(mappingSolutionId<=0)
        //        throw new Exception("账单费用映射方案ID无效");

        //    if (useOrderRange)
        //    {

        //    }
        //    else
        //    {
        //        string sql = "INSERT INTO ";
        //    }
        //}

        public async Task<ReconcileDisplayResult> Reconcile(QueryModel queryModel)
        {

            if (queryModel == null || queryModel.QueryParamer?.Any() != true)
                throw new Exception($"参数不能为空");

            var mappingSolutionIdParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "MappingSolutionId");
            if (mappingSolutionIdParam == null)
                throw new Exception($"映射方案参数不能为空");


            var diffTypeParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "DiffType");
            int diffTypeVal = 0;
            ReconcileDiffType diffType = ReconcileDiffType.不限;
            if (diffTypeParam != null)
            {
                queryModel.QueryParamer.Remove(diffTypeParam);

                if (int.TryParse(diffTypeParam.Value, out diffTypeVal))
                {
                    if (Enum.IsDefined(typeof(ReconcileDiffType), diffTypeVal))
                    {
                        diffType = (ReconcileDiffType)diffTypeVal;
                    }
                }
            }



            decimal diffAccurate = 0;
            var diffAccurateParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "DiffAccurate");
            if (diffAccurateParam != null)
            {
                queryModel.QueryParamer.Remove(diffAccurateParam);

                decimal.TryParse(diffAccurateParam.Value, out diffAccurate);
            }

            var billNameParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "BillName");
            if (billNameParam == null)
                throw new Exception($"账单名称参数不能为空");

            Model.Enums.ReconcileType reconcileType = Model.Enums.ReconcileType.按单号对账;
            var reconcileTypeParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "ReconcileType");
            if (reconcileTypeParam != null)
            {
                queryModel.QueryParamer.Remove(reconcileTypeParam);
                if (int.TryParse(reconcileTypeParam.Value, out int reconcileTypeVal))
                {
                    if (reconcileTypeVal == 1)
                        reconcileType = Model.Enums.ReconcileType.汇总对账;
                }
            }
            var orderRangeSettingParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "OrderRangeSettingId");
            int? orderRangeSettingId = null;
            if (reconcileTypeParam != null)
            {
                queryModel.QueryParamer.Remove(orderRangeSettingParam);
                if (int.TryParse(orderRangeSettingParam.Value, out int orderRangeSettingIdVal))
                {
                    orderRangeSettingId = orderRangeSettingIdVal;
                }
            }


            bool calculateSumAmount = false;
            var sumAmountParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "SumAmount");
            if (sumAmountParam != null)
            {
                queryModel.QueryParamer.Remove(sumAmountParam);
                bool.TryParse(sumAmountParam.Value, out calculateSumAmount);
            }

            if (!int.TryParse(mappingSolutionIdParam.Value, out int mappingSolutionId))
            {
                throw new Exception($"映射方案ID无效:{mappingSolutionIdParam.Value}");
            }

            ///是否显示合并费用
            var showMergeExpense = false;
            var showMergeExpenseParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "ShowMergeExpense");
            if (showMergeExpenseParam != null)
            {
                queryModel.QueryParamer.Remove(showMergeExpenseParam);
                bool.TryParse(showMergeExpenseParam.Value, out showMergeExpense);
            }


            string billName = billNameParam.Value;
            if (string.IsNullOrWhiteSpace(billName))
            {
                throw new Exception($"账单名称不能为空");
            }

            var expenseMappings = await _ServiceBillExpenseMappingRepository.GetAllExpenseMapping(mappingSolutionId);
            if (expenseMappings?.Any() != true)
                throw new Exception($"费用映射方案无映射明细，Id:{mappingSolutionId}");





            if (reconcileType == Model.Enums.ReconcileType.按单号对账)
            {
                ReconcileDisplayResult result = new ReconcileDisplayResult { };

                var cols = new List<ReconcileDisplayColumn>();
                foreach (var m in expenseMappings)
                {
                    var col = new ReconcileDisplayColumn { BillDataName = m.BillDataName, SystemDataName = m.ExpenseName, Sort = m.Sort ?? 999 };
                    cols.Add(col);
                }


                result.Columns = cols;
                if (calculateSumAmount)
                {
                    var sumAmounts = await _reconcileDataRepository.GetSumAmount(billName, mappingSolutionId);
                    if (sumAmounts?.Any() == true)
                    {
                        foreach (var col in result.Columns)
                        {
                            if (col.MerginSystemData == false && col.MergingBillData == false)
                            {
                                var sumItem = sumAmounts.FirstOrDefault(s => s.BillDataName == col.BillDataName);
                                col.BillSumAmount = sumItem?.BillSumAmount;
                                col.SystemSumAmount = sumItem?.ExpenseSumAmount;
                            }
                        }
                    }

                }

                if (showMergeExpense)
                {
                    var billDataMergeds = result.Columns.GroupBy(i => i.BillDataName).Where(g => g.Count() > 1);
                    if (billDataMergeds.Any())
                    {
                        foreach (var g in billDataMergeds)
                        {
                            string billDataName = g.Key;
                            string mergedExpenseNames = string.Join(expenseMergeChar, g.Select(i => i.SystemDataName));
                            var mergedAmount = g.Sum(i => i.SystemSumAmount);
                            var col = new ReconcileDisplayColumn { BillSumAmount = g.First().BillSumAmount, SystemSumAmount = mergedAmount, BillDataName = billDataName, SystemDataName = mergedExpenseNames, Sort = 999, MerginSystemData = true };
                            result.Columns.Add(col);
                        }
                    }
                    var systemExpenseMergeds = result.Columns.GroupBy(i => i.SystemDataName).Where(g => g.Count() > 1);
                    if (systemExpenseMergeds.Any())
                    {
                        foreach (var g in systemExpenseMergeds)
                        {
                            string systemExpenseName = g.Key;
                            string mergedBillDataNames = string.Join(expenseMergeChar, g.Select(i => i.BillDataName));
                            var mergedAmount = g.Sum(i => i.BillSumAmount);

                            var col = new ReconcileDisplayColumn { BillSumAmount = mergedAmount, SystemSumAmount = g.First().SystemSumAmount, BillDataName = mergedBillDataNames, SystemDataName = systemExpenseName, Sort = 999, MergingBillData = true };
                            result.Columns.Add(col);
                        }
                    }
                }



                int pageIndex = queryModel.PageIndex;
                if (pageIndex <= 0)
                    pageIndex = 1;

                int pageSize = queryModel.PageSize;
                if (pageSize > 2000)
                {
                    throw new Exception("单页不能超过2000");
                }

                List<string> numbers = null;
                var numberParam = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "Numbers");
                string numberStr = numberParam?.Value;
                if (!string.IsNullOrWhiteSpace(numberStr))
                {
                    numbers = numberStr.Split(",", StringSplitOptions.RemoveEmptyEntries).ToList();
                }


                List<ReconcileGroup> groups = new List<ReconcileGroup>();
                //var pageResult = await _ViewServiceBillExpenseRepository.QueryServiceBillExpensePage(billName, mappingSolutionId, diffType,diffAccurate , numbers, pageIndex, pageSize);
                var pageResult = await _reconcileDataRepository.QueryServiceBillExpensePage(billName, mappingSolutionId, diffType, diffAccurate, numbers, pageIndex, pageSize);
                result.Total = pageResult.TotalCount;
                if (pageResult.Result?.Any() == true)
                {
                    //按一个订单分组，一个组包含多个费用
                    var numberGroups = pageResult.Result.GroupBy(i => i.Number);

                    foreach (var group in numberGroups)
                    {
                        ReconcileGroup rg = new ReconcileGroup();
                        rg.Number = group.Key;
                        var items = new List<ReconcileItem>();
                        foreach (var col in cols)
                        {
                            if (col.MerginSystemData == false && col.MergingBillData == false)
                            {
                                var item = group.FirstOrDefault(i => i.BillDataName == col.BillDataName);

                                decimal? billData = item?.BillData;
                                decimal? expenseAmt = item?.SystemData;
                                var reconcileItem = new ReconcileItem
                                {
                                    BillData = billData,
                                    BillDataName = col.BillDataName,
                                    Sort = col.Sort,
                                    SystemData = expenseAmt,
                                    SystemDataName = col.SystemDataName,
                                    DiffAccurate = diffAccurate
                                };
                                items.Add(reconcileItem);
                            }
                            else
                            {
                                var mergedReconcileItem = new ReconcileItem
                                {
                                    BillDataName = col.BillDataName,
                                    Sort = col.Sort,
                                    SystemDataName = col.SystemDataName,
                                    DiffAccurate = diffAccurate
                                };
                                if (col.MergingBillData == true)
                                {
                                    var systeDataItem = group.FirstOrDefault(i => i.SystemDataName == col.SystemDataName);
                                    mergedReconcileItem.SystemData = systeDataItem?.SystemData;
                                    var names = col.BillDataName.Split(expenseMergeChar);
                                    var mergeBillData = group.Where(i => names.Contains(i.BillDataName)).Sum(i => i.BillData);
                                    mergedReconcileItem.BillData = mergeBillData;
                                }
                                else
                                {
                                    var billDataItem = group.FirstOrDefault(i => i.BillDataName == col.BillDataName);
                                    mergedReconcileItem.BillData = billDataItem?.BillData;
                                    var names = col.SystemDataName.Split(expenseMergeChar);
                                    var mergeSystemData = group.Where(i => names.Contains(i.SystemDataName)).Sum(i => i.SystemData);
                                    mergedReconcileItem.SystemData = mergeSystemData;
                                }
                                items.Add(mergedReconcileItem);
                            }
                        }
                        rg.Items = items;
                        groups.Add(rg);
                    }
                }
                result.Groups = groups;
                return result;
            }
            else
            {
                //int orderRangeSettingId = reconcileSolution.OrderRangeSettingId;
                if (orderRangeSettingId == null)
                    throw new Exception($"以汇总对账方式对账时，必须指定订单搜索范围设置");

                var rangeSetting = await _orderRangeQueryService.GetSettings(orderRangeSettingId.Value);
                if (rangeSetting == null)
                {
                    throw new Exception($"订单范围搜索设置不存在,Id:{orderRangeSettingId.Value}");
                }

                if (rangeSetting.IsExecuted == null || rangeSetting.IsExecuted == false)
                {
                    await _orderRangeQueryService.InsertOrderRange(orderRangeSettingId.Value);
                }

                ReconcileDisplayResult result = new ReconcileDisplayResult { };

                var cols = new List<ReconcileDisplayColumn>();
                List<ReconcileItem> items = new List<ReconcileItem>();
                foreach (var m in expenseMappings)
                {
                    var col = new ReconcileDisplayColumn { BillDataName = m.BillDataName, SystemDataName = m.ExpenseName, Sort = m.Sort ?? 999 };
                    cols.Add(col);
                }
                result.Columns = cols;

                var rangeExpenses = await _viewOrderRangeExpenseRepository.QueryAsync(i => i.RangeSettingId == orderRangeSettingId);

                var billResolveResults = await _ServiceBillResolveResultRepository.QueryAsync(r => r.BillName == billName);


                foreach (var col in cols)
                {
                    if (!col.MergingBillData && !col.MerginSystemData)
                    {
                        var expenseItem = rangeExpenses?.FirstOrDefault(i => i.ExpenseName == col.SystemDataName);
                        var billResolveResult = billResolveResults?.Where(r => r.BillDataName == col.BillDataName);

                        decimal? billData = billResolveResult?.Sum(r => r.Data);
                        decimal? expenseAmt = expenseItem?.SumOriginAmount;

                        var reconcileItem = new ReconcileItem
                        {
                            BillData = billData,
                            BillDataName = col.BillDataName,
                            Sort = col.Sort,
                            SystemData = expenseAmt,
                            SystemDataName = col.SystemDataName
                        };
                        items.Add(reconcileItem);
                    }
                    else
                    {
                        var reconcileItem = new ReconcileItem
                        {
                            BillDataName = col.BillDataName,
                            Sort = col.Sort,
                            SystemDataName = col.SystemDataName
                        };
                        if (col.MergingBillData == true)
                        {
                            var names = col.BillDataName.Split(expenseMergeChar);
                            var billResolveResultLst = billResolveResults?.Where(r => names.Contains(r.BillDataName));
                            var billTotalAmt = billResolveResultLst.Sum(r => r.Data);
                            var expenseItem = rangeExpenses?.FirstOrDefault(i => i.ExpenseName == col.SystemDataName);
                            decimal? expenseAmt = expenseItem?.SumOriginAmount;
                            reconcileItem.BillData = billTotalAmt;
                            reconcileItem.SystemData = expenseAmt;
                        }
                        else
                        {
                            var names = col.SystemDataName.Split(expenseMergeChar);
                            var expenseResultLst = rangeExpenses?.Where(r => names.Contains(r.ExpenseName));
                            var expenseTotalAmt = expenseResultLst.Sum(r => r.SumSettlementAmount);

                            var billResolveResult = billResolveResults?.Where(r => r.BillDataName == col.BillDataName);
                            decimal? billData = billResolveResult?.Sum(r => r.Data);

                            reconcileItem.BillData = billData;
                            reconcileItem.SystemData = expenseTotalAmt;
                        }
                    }
                }
                ReconcileGroup rg = new ReconcileGroup();
                rg.Items = items;
                rg.Number = "-";

                List<ReconcileGroup> groups = new List<ReconcileGroup>();
                groups.Add(rg);
                result.Groups = groups;
                return result;
            }
        }

        //public Task<IEnumerable<ReconcileSumAmount>> GetSumAmount(ReconcileParam param)
        //{
        //    Task<IEnumerable<ReconcileSumAmount>> amounts = _ViewServiceBillExpenseRepository.GetSumAmount(param.BillName, param.ReconcileSolutionId);
        //    return amounts;
        //}

        public async Task<int> RefreshReconcileData(RefreshReconcileDataParam param)
        {
            if (param == null)
                throw new Exception("参数不能为空");
            if (param.MappingSolutionId <= 0)
                throw new Exception("账单费用映射方案ID无效");
            if (string.IsNullOrWhiteSpace(param.BillName))
                throw new Exception("账单名称不能为空");

            int count = await _reconcileDataRepository.RefreshConcileData(param.BillName, param.MappingSolutionId);
            return count;
        }
    }
}
