﻿using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using SMP.Data.Logistisc;
using SMP.Data.Report;
using SMP.Model.Finance;
using SMP.Service.BaseInfo;
using SMP.Service.Report.Dto;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using MiniExcelLibs;
using System.Collections.Concurrent;
using SMP.Data.Compute;

namespace SMP.Service.Report
{
    public interface ICostProfitReportService
    {
        Task<CostProfitReportDto> QueryCostProfitReportPage(QueryModel param);
        Task<byte[]> DownCostProfitReport(QueryModel queryModel);
        Task<byte[]> DownCostProfitReportV2(QueryModel queryModel);
        Task<byte[]> DownCostProfitReportV3(QueryModel queryModel);
    }

    public class CostProfitReportService : ICostProfitReportService
    {
        ILogistisc_OrderRepository _OrderRepository;
        IView_OrderExpenseItemTotalAmountV2Repository _orderExpenseItemTotalAmountVRepository;
        IBaseInfoService _baseInfoService;
        IBranchCompanyService _branchCompanyConnection;
        IView_OrderExpenseAmountRepository _OrderExpenseAmountRepository;
        IView_GeneralCostV2Repository _generalCostV2Repository;
        IView_OrderExpenseItemCategoryAmountRepository _OrderExpenseItemCategoryAmountRepository;
        ICompute_ExpenseItemCategoryRepository _ExpenseItemCategoryRepository;

        public CostProfitReportService(ILogistisc_OrderRepository orderRepository, IView_OrderExpenseItemTotalAmountV2Repository orderExpenseItemCategoryAmountV2Repository, IBaseInfoService baseInfoService, IBranchCompanyService branchCompanyConnection, IView_OrderExpenseAmountRepository orderExpenseAmountRepository, IView_GeneralCostV2Repository generalCostV2Repository, IView_OrderExpenseItemCategoryAmountRepository orderExpenseItemCategoryAmountRepository, ICompute_ExpenseItemCategoryRepository expenseItemCategoryRepository)
        {
            _OrderRepository = orderRepository;
            _orderExpenseItemTotalAmountVRepository = orderExpenseItemCategoryAmountV2Repository;
            _baseInfoService = baseInfoService;
            _branchCompanyConnection = branchCompanyConnection;
            _OrderExpenseAmountRepository = orderExpenseAmountRepository;
            _generalCostV2Repository = generalCostV2Repository;
            _OrderExpenseItemCategoryAmountRepository = orderExpenseItemCategoryAmountRepository;
            _ExpenseItemCategoryRepository = expenseItemCategoryRepository;
        }



        private async Task<CostProfitReportDto> QueryAllCostProfit(QueryModel queryModel)
        {
            if (queryModel == null)
            {
                throw new ArgumentNullException("参数不能为空");
            }
            ReportQueryParamPreHandle.PreHandleQueryModel(ref queryModel);
            var orders = await _OrderRepository.GetAll(queryModel);

            if (orders?.Any() == true)
            {
                CostProfitReportDto result = await QueryCostProfit(orders);
                result.TotalCount = orders.Count();
                return result;
            }
            else
                return null;
        }

        public async Task<CostProfitReportDto> QueryCostProfit(IEnumerable<Logistisc_Order> orders)
        {

            CostProfitReportDto result = new CostProfitReportDto();
            var orderGroups = orders.GroupBy(o => o.SystemNo);
            var systemNos = orderGroups.Select(g => g.Key);

            var branchCompanyOrderAmountsTask = _branchCompanyConnection.GetBranchCompanyOrderAmount(systemNos);
            var orderAmounts = await _orderExpenseItemTotalAmountVRepository.GetOrderAmount(systemNos);


            var branchCompanyOrderAmountsResponse = await branchCompanyOrderAmountsTask;
            var branchCompanyOrderAmounts = branchCompanyOrderAmountsResponse.Data;


            var allCustomers = await _baseInfoService.CacheGetAllCustomers();
            var allChannels = await _baseInfoService.CacheGetAllDeliveryChannels();
            var allPublicChannels = await _baseInfoService.CacheGetAllPublicChannels();

            var allNames = orderAmounts.Select(i => i.ExpenseItemCategoryName).Distinct();
            result.ExpenseItemNames = allNames;

            List<OrderCostProfitDto> orderCostProfitDtos = new List<OrderCostProfitDto>();
            result.ExpenseItemAmounts = orderCostProfitDtos;

            var amountGroups = orderAmounts.GroupBy(i => i.SystemNo);
            foreach (var g in amountGroups)
            {
                var first = g.First();

                string systemNo = first.SystemNo;
                var branchCompanyAmt = branchCompanyOrderAmounts?.FirstOrDefault(a => a.SystemNo == systemNo);

                var customer = allCustomers?.FirstOrDefault(i => i.Id == first.CustomerId);
                var publicChannel = allPublicChannels.FirstOrDefault(i => i.Id == first.PublicChannelId);
                var deliveryChannel = allChannels.FirstOrDefault(i => i.Id.ToString() == first.OrderChannelUniqueId);

                OrderCostProfitDto orderCostProfit = new OrderCostProfitDto
                {
                    SystemNo = systemNo,
                    TrackingNumber = first.TrackingNumber,
                    LadingBillNumber = first.LadingBillNumber,
                    PublicChannelId = first.PublicChannelId,
                    DeliveryChannelId = first.OrderChannelUniqueId,
                    CustomerId = first.CustomerId,
                    OpWeight = first.OPWeight,
                    CustomerName = customer?.CompanyName,
                    PublicChannelName = publicChannel?.Name,
                    DeliveryChannelName = deliveryChannel?.Name,
                    BranchCompanyAmount = branchCompanyAmt?.BranchCompanyAmount,
                    BranchCompanyName = branchCompanyAmt?.BranchCompanyName,
                    BranchCompanyProfit = branchCompanyAmt?.BranchCompanyProfit,
                    CustomerAmount = branchCompanyAmt?.CustomerAmount
                };


                var estimateCosts = g.Where(i => i.CostType == Model.Enums.CostType.预估成本);
                var totalEstimateCost = estimateCosts.Sum(i => i.SettlementAmount);
                orderCostProfit.EstimateTotalCost = totalEstimateCost;

                var billCosts = g.Where(i => i.CostType == Model.Enums.CostType.账单成本);
                decimal? billTotalCost = billCosts?.Any() == true ? billCosts.Sum(i => i.SettlementAmount) : null;
                orderCostProfit.BillTotalCost = billTotalCost;

                orderCostProfit.ExpenseItemAmounts = new List<OrderExpenseItemAmount>();
                foreach (var item in estimateCosts)
                {
                    OrderExpenseItemAmount expenseItemAmt = new OrderExpenseItemAmount
                    {
                        SystemNo = item.SystemNo,
                        CostType = item.CostType,
                        ExpenseItemName = item.ExpenseItemCategoryName,
                        OriginAmount = item.OriginAmount,
                        OriginCurrency = item.OriginCurrency,
                        Rate = item.SettlementRate,
                        SettlementAmount = item.SettlementAmount,
                        SettlementCurrency = item.SettlementCurrency,
                        LastVersionDate = item.LastVersionDate,
                    };
                    orderCostProfit.ExpenseItemAmounts.Add(expenseItemAmt);
                }

                orderCostProfitDtos.Add(orderCostProfit);
            }
            return result;
        }

        public async Task<CostProfitReportDto> QueryCostProfitReportPage(QueryModel queryModel)
        {
            if (queryModel == null)
            {
                throw new ArgumentNullException("参数不能为空");
            }
            ReportQueryParamPreHandle.PreHandleQueryModel(ref queryModel);
            var orderPage = await _OrderRepository.QueryPagerAsync(queryModel);

            if (orderPage?.Result?.Any() == true)
            {
                var orders = orderPage.Result;
                CostProfitReportDto result = await QueryCostProfit(orders);
                result.TotalCount = orderPage.TotalCount;
                return result;
            }
            else
                return null;
        }

        public async Task<byte[]> DownCostProfitReport(QueryModel queryModel)
        {
            queryModel.PageIndex = 1;
            queryModel.PageSize = int.MaxValue;
            var pageResult = await QueryAllCostProfit(queryModel);


            List<string> headNames = new List<string> { "订单号", "跟踪号", "提单号", "下单渠道", "派送渠道", "扣费重量(KG)", "客户名称", "客户费用", "分公司名称", "分公司结算费用", "预计总成本", "实际总成本", "分公司利润", "总公司预计利润", "总公司实际利润" };


            if (pageResult == null)
            {
                pageResult = new CostProfitReportDto
                {
                    ExpenseItemAmounts = new List<OrderCostProfitDto>(),
                    ExpenseItemNames = new List<string>(),
                    TotalCount = 0
                };
            }


            #region npoi
            IWorkbook wk = new XSSFWorkbook();
            ISheet sheet = wk.CreateSheet("Sheet1");

            var expenseItemNames = pageResult.ExpenseItemNames;
            List<string> expenseItemNameAndTimes = new List<string>();
            foreach (var itemName in expenseItemNames)
            {
                expenseItemNameAndTimes.Add(itemName);
                expenseItemNameAndTimes.Add($"{itemName}生效时间");
            }
            headNames.InsertRange(10, expenseItemNameAndTimes);
            var headRow = sheet.CreateRow(0);
            for (int i = 0; i < headNames.Count; i++)
            {
                var cell = headRow.CreateCell(i);
                cell.SetCellValue(headNames[i]);
            }

            for (int r = 0; r < pageResult.ExpenseItemAmounts.Count; r++)
            {
                var amt = pageResult.ExpenseItemAmounts[r];
                var row = sheet.CreateRow(r + 1);

                int idx = 0;
                row.CreateCell(idx++).SetCellValue(amt.SystemNo);
                row.CreateCell(idx++).SetCellValue(amt.TrackingNumber);
                row.CreateCell(idx++).SetCellValue(amt.LadingBillNumber);
                row.CreateCell(idx++).SetCellValue(amt.PublicChannelName);
                row.CreateCell(idx++).SetCellValue(amt.DeliveryChannelName);
                row.CreateCell(idx++).SetCellValue((double)amt.OpWeightKg);
                row.CreateCell(idx++).SetCellValue(amt.CustomerName);
                row.CreateCell(idx++).SetCellValue(amt.CustomerAmount.HasValue ? (double)amt.CustomerAmount : 0);
                row.CreateCell(idx++).SetCellValue(amt.BranchCompanyName);
                row.CreateCell(idx++).SetCellValue(amt.BranchCompanyAmount.HasValue ? (double)amt.BranchCompanyAmount : 0);

                foreach (var expenseItemName in expenseItemNames)
                {
                    var ea = amt.ExpenseItemAmounts?.FirstOrDefault(a => a.ExpenseItemName == expenseItemName);
                    double expenseAmt = ea == null ? 0 : ea.SettlementAmount.HasValue ? (double)ea.SettlementAmount : 0;
                    row.CreateCell(idx++).SetCellValue(expenseAmt);
                    string versionDate = string.Empty;
                    if (ea != null && ea.LastVersionDate.HasValue)
                        versionDate = ea.LastVersionDate.Value.ToString_yyyyMMdd();
                    row.CreateCell(idx++).SetCellValue(versionDate);

                }
                row.CreateCell(idx++).SetCellValue((double)amt.EstimateTotalCost);
                if (amt.BillTotalCost == null)
                {
                    row.CreateCell(idx++).SetCellValue(String.Empty);
                }
                else
                {
                    row.CreateCell(idx++).SetCellValue((double)amt.BillTotalCost);
                }

                row.CreateCell(idx++).SetCellValue(amt.BranchCompanyProfit.HasValue ? (double)amt.BranchCompanyProfit : 0);
                row.CreateCell(idx++).SetCellValue((double)amt.HeadCompanyEstimateProfit);

                if (amt.HeadCompanyBillProfit == null)
                {
                    row.CreateCell(idx++).SetCellValue(String.Empty);
                }
                else
                {
                    row.CreateCell(idx++).SetCellValue((double)amt.HeadCompanyBillProfit);
                }

            }


            MemoryStream ms = new MemoryStream();
            wk.Write(ms);
            var content = ms.ToArray();
            ms.Close();
            ms.Dispose();
            wk.Close();
            return content;
            #endregion

            #region miniExcel
            // string file = Path.GetTempFileName();
            //var fs = System.IO.File.OpenWrite(file); 
            // MiniExcel.SaveAsAsync(fs, pageResult.ExpenseItemAmounts)
            #endregion

        }

        public async Task<byte[]> DownCostProfitReportV2(QueryModel queryModel)
        {
            var allCustomers = await _baseInfoService.CacheGetAllCustomers();
            var allChannels = await _baseInfoService.CacheGetAllDeliveryChannels();
            var allPublicChannels = await _baseInfoService.CacheGetAllPublicChannels();

            ReportQueryParamPreHandle.PreHandleQueryModel(ref queryModel);

            var orderGeneralCost = _generalCostV2Repository.QueryAll(queryModel);
            var orderExpenseAmounts = _OrderExpenseItemCategoryAmountRepository.QueryAll(queryModel);


            var expenseItemNames = orderExpenseAmounts.GroupBy(i => i.ExpenseItemCategoryName).Select(g => g.Key).Where(i => !string.IsNullOrWhiteSpace(i)).ToList();

            var orderExpenseAmountsGroups = orderExpenseAmounts.ToDictionary(i => i.SystemNo + i.ExpenseItemCategoryName);



            bool hasNames = expenseItemNames?.Any() == true;

            List<Dictionary<string, object>> data = new List<Dictionary<string, object>>();
            foreach (var orderCost in orderGeneralCost)
            {
                var customer = allCustomers?.FirstOrDefault(i => i.Id == orderCost.CustomerId);
                var publicChannel = allPublicChannels.FirstOrDefault(i => i.Id.ToString() == orderCost.PublicChannelId);
                var deliveryChannel = allChannels.FirstOrDefault(i => i.Id.ToString() == orderCost.OrderChannelUniqueId);

                string publicChannelName = publicChannel?.Name;
                string channelName = deliveryChannel?.Name;
                string customerName = customer?.CompanyName;
                decimal billWeight = orderCost.OpWeight / 1000m;
                decimal? headCompanyEstimateProfit = orderCost.CustomerAmount - orderCost.TotalEstimateCost;
                decimal? headCompanyActualProfit = orderCost.CustomerAmount - orderCost.TotalBillCost;

                Dictionary<string, object> row = new Dictionary<string, object>();
                row.Add("订单号", orderCost.SystemNo);
                row.Add("跟踪号", orderCost.TrackingNumber);
                row.Add("提单号", orderCost.LadingBillNumber);
                row.Add("下单渠道", publicChannelName);
                row.Add("派送渠道", channelName);
                row.Add("扣费重量(KG)", billWeight);
                row.Add("客户名称", customerName);
                row.Add("客户费用", orderCost.CustomerAmount);
                row.Add("分公司名称", orderCost.BranchCompanyName);
                row.Add("分公司结算费用", orderCost.BranchCompanyAmount);
                row.Add("预计总成本", orderCost.TotalEstimateCost);
                row.Add("实际总成本", orderCost.TotalBillCost);
                row.Add("分公司利润", orderCost.BranchCompanyProfit);
                row.Add("总公司预计利润", headCompanyEstimateProfit);
                row.Add("总公司实际利润", headCompanyActualProfit);

                if (hasNames)
                {
                    foreach (var name in expenseItemNames)
                    {
                        if (name == null) continue;

                        string key = orderCost.SystemNo + name;
                        bool exists = orderExpenseAmountsGroups.TryGetValue(key, out var expense);
                        if (exists && expense != null)
                        {
                            decimal amt = expense.SettlementAmount;
                            DateTime? lastVerDate = expense.LastVersionDate;
                            row.Add(name, amt);
                            row.Add($"版本时间({name})", lastVerDate);
                        }
                        else
                        {
                            row.Add(name, null);
                            row.Add($"版本时间({name})", null);
                        }
                    }
                }
                data.Add(row);
            }

            //var allData = dicts.SelectMany(i => i).ToList();

            #region miniExcel
            string temDir = Path.GetTempPath();
            string temFile = Path.Combine(temDir, Guid.NewGuid().ToString("n") + ".xlsx");
            MiniExcel.SaveAs(temFile, data);

            var bytes = File.ReadAllBytes(temFile);
            return bytes;
            #endregion
        }

        public async Task<byte[]> DownCostProfitReportV3(QueryModel queryModel)
        {
            ReportQueryParamPreHandle.PreHandleQueryModel(ref queryModel);

            var orderGeneralCostTask = _generalCostV2Repository.QueryAllAsync(queryModel);
            var orderExpenseAmountsTask = _OrderExpenseItemCategoryAmountRepository.QueryAllEstimateAmountAsync(queryModel);

            var allCustomers = await _baseInfoService.CacheGetAllCustomers();
            var allChannels = await _baseInfoService.CacheGetAllDeliveryChannels();
            var allPublicChannels = await _baseInfoService.CacheGetAllPublicChannels();
           

            var orderGeneralCost = await orderGeneralCostTask;
            var orderExpenseAmounts = await orderExpenseAmountsTask;

            var allExpenseItemCategorys = await _ExpenseItemCategoryRepository.QueryAllIdNames();

            var allExpenseItemCategoryIds = orderExpenseAmounts.GroupBy(i => i.ExpenseItemCategoryId).Select(g => g.Key).ToList();
            var expenseItemNames = allExpenseItemCategorys.Where(c => allExpenseItemCategoryIds.Contains(c.Id)).Select(c => c.Name).ToList();

            var expenseItemCategoryDic = allExpenseItemCategorys.ToDictionary(i => i.Id, i => i.Name);
            var orderExpenseAmountsGroups = orderExpenseAmounts.ToDictionary(i => i.SystemNo + expenseItemCategoryDic[i.ExpenseItemCategoryId]);

            bool hasNames = expenseItemNames?.Any() == true;

            List<Dictionary<string, object>> data = new List<Dictionary<string, object>>();
            foreach (var orderCost in orderGeneralCost)
            {
                var customer = allCustomers?.FirstOrDefault(i => i.Id == orderCost.CustomerId);
                var publicChannel = allPublicChannels.FirstOrDefault(i => i.Id.ToString() == orderCost.PublicChannelId);
                var deliveryChannel = allChannels.FirstOrDefault(i => i.Id.ToString() == orderCost.OrderChannelUniqueId);

                string publicChannelName = publicChannel?.Name;
                string channelName = deliveryChannel?.Name;
                string customerName = customer?.CompanyName;
                decimal billWeight = orderCost.OpWeight / 1000m;
                decimal? headCompanyEstimateProfit = orderCost.CustomerAmount - orderCost.TotalEstimateCost;
                decimal? headCompanyActualProfit = orderCost.CustomerAmount - orderCost.TotalBillCost;

                Dictionary<string, object> row = new Dictionary<string, object>();
                row.Add("订单号", orderCost.SystemNo);
                row.Add("跟踪号", orderCost.TrackingNumber);
                row.Add("提单号", orderCost.LadingBillNumber);
                row.Add("下单渠道", publicChannelName);
                row.Add("派送渠道", channelName);
                row.Add("扣费重量(KG)", billWeight);
                row.Add("客户名称", customerName);
                row.Add("客户费用", orderCost.CustomerAmount);
                row.Add("分公司名称", orderCost.BranchCompanyName);
                row.Add("分公司结算费用", orderCost.BranchCompanyAmount);
                row.Add("预计总成本", orderCost.TotalEstimateCost);
                row.Add("实际总成本", orderCost.TotalBillCost);
                row.Add("分公司利润", orderCost.BranchCompanyProfit);
                row.Add("总公司预计利润", headCompanyEstimateProfit);
                row.Add("总公司实际利润", headCompanyActualProfit);

                if (hasNames)
                {
                    foreach (var name in expenseItemNames)
                    {
                        string key = orderCost.SystemNo + name;
                        bool exists = orderExpenseAmountsGroups.TryGetValue(key, out var expense);
                        if (exists && expense != null)
                        {
                            decimal amt = expense.SettlementAmount;
                            DateTime? lastVerDate = expense.LastVersionDate;
                            row.Add(name, amt);
                            row.Add($"版本时间({name})", lastVerDate);
                        }
                        else
                        {
                            row.Add(name, null);
                            row.Add($"版本时间({name})", null);
                        }
                    }
                }
                data.Add(row);
            }

            //var allData = dicts.SelectMany(i => i).ToList();

            #region miniExcel
            string temDir = Path.GetTempPath();
            string temFile = Path.Combine(temDir, Guid.NewGuid().ToString("n") + ".xlsx");
            MiniExcel.SaveAs(temFile, data);

            var bytes = File.ReadAllBytes(temFile);
            return bytes;
            #endregion
        }
    }
}
