﻿using RestSharp;
using SMP.Model;
using SMP.Model.Dto.DataInterconnection;
using SMP.Model.Enums;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Core.Standard.Mvc;

namespace SMP.Service.ThirdPartyRequest
{
    public interface IWMSDataInterconnectionService
    {
        Task<ThirdPartyData_LadingBillDto> PullOrderInfo(ThirdPartyData_Input input);
        Task<IEnumerable<Logistics_Channel>> PullChannel();
    }
    public class WMSDataInterconnectionService : IWMSDataInterconnectionService
    {
        private static readonly string _wmsUrl = "http://wms.xingyunyi.cn/";
        private ISys_LogService _sys_LogService;
        private string className = "WMSDataInterconnection";
        public WMSDataInterconnectionService(ISys_LogService sys_LogService)
        {
            this._sys_LogService = sys_LogService;
        }
        /// <summary>
        /// 拉取订单信息
        /// </summary>
        /// <returns></returns>
        public async Task<ThirdPartyData_LadingBillDto> PullOrderInfo(ThirdPartyData_Input input)
        {
            try
            {
                var client = new RestClient(_wmsUrl + "api/Thirdparty/PullOrderInfo");
                var request = new RestRequest();
                request.AddBody(input);
                var response = await client.ExecutePostAsync<ApiJsonModel<ThirdPartyData_LadingBillDto>>(request);
                var result = response.Data?.data;
                return result;
            }
            catch (Exception ex)
            {
                await _sys_LogService.Create(new Sys_Log()
                {
                    LogLevel = LogLevelEnum.ERROR,
                    LogTitle = "拉取订单信息",
                    ClassName = className,
                    MethodName = "PullOrderInfo",
                    ErrorMessage = ex.Message,
                });
                return null;
            }
        }
        /// <summary>
        /// 拉取渠道信息
        /// </summary>
        /// <returns></returns>
        public async Task<IEnumerable<Logistics_Channel>> PullChannel()
        {
            try
            {
                var client = new RestClient(_wmsUrl + "api/Thirdparty/PullChannel");
                var request = new RestRequest();
                var response = await client.ExecuteGetAsync<ApiJsonModel<IEnumerable<Logistics_Channel>>>(request);
                var result = response.Data?.data;
                return result;
            }
            catch (Exception ex)
            {
                await _sys_LogService.Create(new Sys_Log()
                {
                    LogLevel = LogLevelEnum.ERROR,
                    LogTitle = "拉取渠道信息",
                    ClassName = className,
                    MethodName = "PullChannel",
                    ErrorMessage = ex.Message,
                });
                return null;
            }
        }
    }
}
