﻿using Microsoft.AspNetCore.Mvc;
using SMP.Model.Compute;
using SMP.Model.Dto.Finance;
using SMP.Model.Finance;
using SMP.Service.Compute;
using XYY.Common.Standard;

namespace SMP.Api.Base.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class QueryComputeExpenseController : ControllerBase
    {
        public QueryComputeExpenseController(IComputeExpenseService ComputeExpenseService)
        {
            this.ComputeExpenseService = ComputeExpenseService;
        }

        public IComputeExpenseService ComputeExpenseService { get; }
        [HttpGet]
        public async Task<List<Compute_Expense>> Get()
        {
            return await ComputeExpenseService.GetAll();
        }

      
        [HttpPost]
        public async Task<PageResult<Compute_Expense>> Post([FromBody] QueryModel model)
        {
            return await ComputeExpenseService.QueryComputeExpense(model);
        }

        [Route("GetByProvider")]
        [HttpGet]
        public async Task<IEnumerable<Compute_Expense>> GetByProvider(int providerId)
        {
            return await ComputeExpenseService.GetByProviderId(providerId);
        }
        [Route("[action]")]
        [HttpPost]
        public async Task<IEnumerable<QueryExpenseByLadingResult>> GetExpenseWithVersionByLading(QueryExpenseByLadingInput input)
        {
            return await ComputeExpenseService.GetExpenseWithVersionByLading(input);
        }
        [Route("[action]")]
        [HttpGet]
        public async Task<IEnumerable<finances>> InsLadingBillPageExpense(string? LadingBillNumber,int? BindBillId)
        {
            List<finances> finances = new List<finances>();
            if (string.IsNullOrEmpty(LadingBillNumber)) { return finances; }
            QueryExpenseByLadingInput input = new QueryExpenseByLadingInput() { LadingBillNumber= LadingBillNumber, BindBillId= BindBillId??0 };
            var result = await ComputeExpenseService.GetExpenseWithVersionByLading(input);
            if (result!=null&& result.Count()>0) {
                var itemIds = result.Select(x => x.ExpenseItemId).Distinct().ToList();
                foreach (var item in itemIds)
                {
                    finances finance = new finances();
                    finance.IncidentalType = item;
                    finance.ExpenseResult = result.Where(x=>x.ExpenseItemId==item);
                    finances.Add(finance);
                }
            }
            return finances;
        }
        [Route("[action]")]
        [HttpGet]
        public async Task<IEnumerable<Compute_ExpenseItem>> GetAllExpenseItemByTemplateId(int TemplateId)
        {
            return await ComputeExpenseService.GetAllExpenseItemByTemplateId(TemplateId);
        }
    }
}
