﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Model.Finance;
using SMP.Service.Finance;
using XYY.Common.Standard;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.Finance
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class FirstPortController : ControllerBase
    {
        private readonly ILogisticsFirstPortService _logisticsFirstPortService;
        public FirstPortController(ILogisticsFirstPortService logisticsFirstPortService)
        {
            this._logisticsFirstPortService = logisticsFirstPortService;
        }

        [HttpPost]
        public async Task<ApiJsonModel<PageResult<Logistics_FirstPort>>> Query(QueryModel queryModel)
        {
            var result= await _logisticsFirstPortService.QueryPage(queryModel);
            return ApiJsonModel<PageResult<Logistics_FirstPort>>.OK(result);
        }

        [HttpGet]
        public async Task<ApiJsonModel<IEnumerable<Logistics_FirstPort>>> GetList()
        {
            var result = await _logisticsFirstPortService.QueryAll();
            return ApiJsonModel<IEnumerable<Logistics_FirstPort>>.OK(result?.OrderBy(x=>x.Port));
        }

        [HttpPost]
        public async Task<ApiJsonModel> Create(Logistics_FirstPort model)
        {
            if (await _logisticsFirstPortService.IsExists(model.Port, model.PortType, model.Id)) { throw new Exception($"同口岸类型，已存在名为【{model.Port}】的口岸！"); }
            await _logisticsFirstPortService.Create(model);
            return ApiJsonModel.OK();
        }

        [HttpPost]
        public async Task<ApiJsonModel> Modify(Logistics_FirstPort model)
        {
            if (await _logisticsFirstPortService.IsExists(model.Port, model.PortType, model.Id)) { throw new Exception($"同口岸类型，已存在名为【{model.Port}】的口岸！"); }
            await _logisticsFirstPortService.Edit(model);
            return ApiJsonModel.OK();
        }

        [HttpPost]
        public async Task<ApiJsonModel<bool>> Remove(int id)
        {
            var ok = await _logisticsFirstPortService.Delete(id);
            return ApiJsonModel<bool>.OK(ok);
        }
    }
}
