﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Model.Dto.Finance;
using SMP.Model.Finance;
using SMP.Service.Finance;
using System.Diagnostics;
using XYY.Common.Standard;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.Finance
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class LadingBillController : ControllerBase
    {
        private readonly IFinance_Logistics_LadingBillService _BillTypeService;
        public LadingBillController(IFinance_Logistics_LadingBillService BillTypeService)
        {
            this._BillTypeService = BillTypeService;
        }
        [HttpPost]
        public async Task<ApiJsonModel<PageResult<Finance_Logistics_LadingBill>>> Query(QueryModel queryModel)
        {
            var result = await _BillTypeService.QueryPage(queryModel);
            return ApiJsonModel<PageResult<Finance_Logistics_LadingBill>>.OK(result);
        }
        [HttpPost]
        public async Task<ApiJsonModel<long>> Create(LadingBillDto dto)
        {
            long result = 0;
            switch (dto.SystemBillType)
            {
                case Model.Enums.SystemBillTypeEnum.LadingBill:
                    result = await _BillTypeService.Create(dto);
                    break;
                case Model.Enums.SystemBillTypeEnum.CustomerLadingBill:
                    result = await _BillTypeService.CustomerLadingBillCreate(dto);
                    break;
            }
            return ApiJsonModel<long>.OK(result);
        }
        [HttpPost]
        public async Task<ApiJsonModel> Modify(LadingBillDto dto)
        {
            switch (dto.SystemBillType)
            {
                case Model.Enums.SystemBillTypeEnum.LadingBill:
                    await _BillTypeService.Edit(dto);
                    break;
                case Model.Enums.SystemBillTypeEnum.ScopeBill:
                    await _BillTypeService.ScopeBillEdit(dto);
                    break;
                case Model.Enums.SystemBillTypeEnum.CustomerLadingBill:
                    await _BillTypeService.CustomerLadingBillEdit(dto);
                    break;
            }
            return ApiJsonModel.OK();
        }
        [HttpPost]
        public async Task<ApiJsonModel> Remove(int LadingBillId, int SystemBillType)
        {
            switch (SystemBillType)
            {
                case (int)Model.Enums.SystemBillTypeEnum.LadingBill:
                    await _BillTypeService.RemoveLadingBill(LadingBillId);
                    break;
                case (int)Model.Enums.SystemBillTypeEnum.ScopeBill:
                    await _BillTypeService.RemoveScopeBill(LadingBillId);
                    break;
                case (int)Model.Enums.SystemBillTypeEnum.CustomerLadingBill:
                    await _BillTypeService.RemoveCustomerLadingBill(LadingBillId);
                    break;
            }
            return ApiJsonModel.OK();
        }
        [HttpPost]
        public async Task<ApiJsonModel> QueryIncidentalList(int LadingBillId)
        {
            var result = await _BillTypeService.GetIncidentals(LadingBillId);
            return ApiJsonModel.OK(result);
        }
        public async Task<ApiJsonModel> InsBill(int Id)
        {
            var result = await _BillTypeService.InsBill(Id);
            return ApiJsonModel.OK(result);
        }
        [HttpGet]
        public async Task<bool> IsExists(string LadingBillNumber)
        {
            var result = await _BillTypeService.IsExistsAsync(LadingBillNumber);
            return result;
        }
        [HttpGet]
        public async Task<bool> BatchRecompute()
        {
            return await _BillTypeService.BatchRecompute();
        }
        [HttpGet]
        public async Task<ApiJsonModel> BatchRecomputeSeparateOrders(int startId, int endId)
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();
            var result= await _BillTypeService.BatchRecomputeSeparateOrders(startId, endId);
            watch.Stop();
            return ApiJsonModel.OK(new { result = result, sj = watch.ElapsedMilliseconds });
        }
    }
}
