﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Caching.Distributed;
using SMP.Common.Auth;
using System.Security.Claims;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.Login
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class AuthController : ControllerBase
    {
        IConfiguration Configuration { get; set; }
        private readonly IDistributedCache _cache;

        public AuthController(IConfiguration Configuration, IDistributedCache cache)
        {
            this.Configuration = Configuration;
            _cache = cache;
        }
         
        [AllowAnonymous]
        [HttpPost]
        public async Task<TokenDto> Login([FromBody] LoginDto login)
        {
            TokenDto tokenInfo = new TokenDto();
            var userInfo = await Authorized(login.UserName, login.Password);
            string token = JWTHelper.CreateToken(this.Configuration, userInfo);
            //增加Redis白名单
            await _cache.SetStringAsync("jwt" + userInfo.token, "1");
            return new TokenDto
            {
                Success = true,
                Token = token,
                Message = "成功",
                Roles = userInfo.roles,
                Name = userInfo.userNickName
            };
        }
        [AllowAnonymous]
        [HttpPost]
        public async Task<ApiJsonModel> Logout()
        {
            string? token = this.User.Claims.First(x => x.Type == ClaimTypes.NameIdentifier)?.Value;
            if (token != null)
            {
                await _cache.RemoveAsync("jwt" + token);
            }
            return ApiJsonModel.OK();
        }

        private async Task<UserDto> Authorized(string userName, string passWord)
        {
            HttpClient client = new HttpClient();
            client.DefaultRequestHeaders.Add("token", "132A7468DE079C6CEB59F383A661E612");
            var response = await client.PostAsJsonAsync("http://120.24.149.148/api/auth/Login", new { userName, passWord });
            string body = await response.Content.ReadAsStringAsync();
            if (response.StatusCode == System.Net.HttpStatusCode.OK)
            {
                var u = Newtonsoft.Json.JsonConvert.DeserializeObject<ApiJsonModel<UserDto>>(body);
                if (u.data != null)
                {
                    return u.data;
                }
                else
                {
                    throw new Exception("鉴权失败" + u.message);
                }
            }
            else
            {

                throw new Exception("登陆失败" + body);
            }
        }
    }
}
