﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Service.Dtos;
using SMP.Service.OrderRangeQuery;
using SMP.Service.OrderRangeQuery.Dto;
using SMP.Tool.TimingTask.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Mvc;
namespace SMP.Api.Base.Controllers.OrderRangeQuery
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class OrderRangeQueryController : ControllerBase
    {
        IOrderRangeQueryService _orderRangeQueryService;
        public OrderRangeQueryController(IOrderRangeQueryService orderRangeQueryService)
        {
            _orderRangeQueryService = orderRangeQueryService;
        }

        [HttpPost]
        public async Task<ApiJsonModel> SaveSetting(OrderRangeQuerySettingDto dto)
        {
            await _orderRangeQueryService.SaveSetting(dto);
            return ApiJsonModel.OK();
        }

        [HttpPost]
        public async Task<ApiJsonModel> SearchByName(SearchOrderRangeQueryParam dto)
        {
           var result =  await _orderRangeQueryService.SearchSettingByName(dto?.Name);
            return ApiJsonModel.OK(result);
        }

        [HttpGet]
        public async Task<ApiJsonModel> GetAll()
        {
            IEnumerable<OrderRangeQuerySettingItem> result = await _orderRangeQueryService.GetAllSettings();
            return ApiJsonModel.OK(result);
        }


        [HttpPost]
        public async Task<ApiJsonModel> QueryPage(QueryModel queryModel)
        {
            var result = await _orderRangeQueryService.QueryPage(queryModel);
            return ApiJsonModel.OK(result);
        }

        [HttpGet]
        public async Task<ApiJsonModel> LoadRangeSetting(int id)
        {
            OrderRangeQuerySettingDto result = await _orderRangeQueryService.LoadRangeSetting(id);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> QueryResultPage(QueryModel queryModel)
        {
            var result = await _orderRangeQueryService.QueryOrderResultPage(queryModel);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> Execute(ExecuteOrderRangeParam param)
        {
            if (param == null ||param.Id<=0)
                throw new Exception("参数无效");
            var result = await _orderRangeQueryService.Execute(param.Id);
            return ApiJsonModel.OK(result);
        }


        [HttpPost]
        public async Task<ApiJsonModel> Remove(IdParam param)
        {
            bool ok = await _orderRangeQueryService.Remove(param.Id);
            if (ok)
                return ApiJsonModel.OK();
            else
                return ApiJsonModel.Error("删除失败");
        }

    }
}
