﻿using HtmlAgilityPack;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Model;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.Curreny
{
    [ApiController]
    [Route("[controller]")]
    public class GetNowRateController : ControllerBase
    {
        /// <summary>
        /// 获取当前汇率
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public async Task<ApiJsonModel<List<CurrenyDate>>> Get()
        {
            DateTime end = DateTime.Now;
            DateTime start = DateTime.Now.AddMonths(-1);
            return await GetNow(start, end);
        }
        [HttpPost]
        public async Task<ApiJsonModel<List<CurrenyDate>>> GetNow([FromQuery] DateTime start, [FromQuery] DateTime end)
        {
            Dictionary<string, int> names = new Dictionary<string, int>();
            string url = $"https://www.safe.gov.cn/AppStructured/hlw/RMBQuery.do";
            var content = new MultipartFormDataContent();
            content.Add(new StringContent(start.ToString("yyyy-MM-dd")), "startDate");
            content.Add(new StringContent(end.ToString("yyyy-MM-dd")), "endDate");
            content.Add(new StringContent("true"), "queryYN");
            var html = await new HttpClient().PostAsync(url, content);
            var defaultCurreny = GetDefaultCurreny();
            List<CurrenyDate> dates = new List<CurrenyDate>();
            if (html.IsSuccessStatusCode)
            {
                HtmlDocument doc = new HtmlDocument();
                doc.LoadHtml(await html.Content.ReadAsStringAsync());
                HtmlNode node = doc.DocumentNode.SelectSingleNode("/html/body/form/table/tr[4]/td/table/tr[3]/td/table");
                var child = node.ChildNodes.Where(x => x.NodeType == HtmlNodeType.Element).ToList();
                for (int i = 0; i < child.Count; i++)
                {
                    var childNode = child[i];
                    var tdNodes = childNode.ChildNodes.Where(x => x.NodeType == HtmlNodeType.Element).ToList();
                    if (i == 0)
                    {
                        for (var j = 0; j < tdNodes.Count; j++)
                        {
                            names.Add(tdNodes[j].InnerText.Trim(), j);
                        }
                    }
                    else
                    {
                        for (int j = 0; j < names.Count; j++)
                        {
                            var dc = defaultCurreny.FirstOrDefault(x => x.CNName == names.Keys.ElementAt(j));
                            if (dc != null)
                            {
                                DateTime dateTime = DateTime.Parse(tdNodes[0].InnerText.Trim());
                                var dateItem = dates.FirstOrDefault(x => x.Date == dateTime);
                                if (dateItem == null)
                                {
                                    dateItem = new CurrenyDate
                                    {
                                        Rates = new List<CurrenyRate>(),
                                        Date = dateTime
                                    };
                                    dates.Add(dateItem);
                                }
                                dateItem.Rates.Add(new CurrenyRate
                                {
                                    CNName = dc.CNName,
                                    Code = dc.Code,
                                    Rate = tdNodes[j].InnerText.Trim()
                                });
                            }
                        }
                    }
                }
            }
            else
            {
                throw new Exception(await html?.Content?.ReadAsStringAsync() ?? "");
            }
            return ApiJsonModel<List<CurrenyDate>>.OK(dates);
        }

        public class CurrenyDate
        {
            public DateTime Date
            {
                get; set;
            }
            public List<CurrenyRate> Rates
            {
                get; set;
            }
        }

        public class CurrenyRate
        {

            public string Code
            {
                get; set;
            }
            public string CNName
            {
                get; set;
            }

            public string Rate
            {
                get; set;
            }
        }

        public class CurrenyDefault
        {
            public string Code
            {
                get; set;
            }
            public string CNName
            {
                get; set;
            }

            public CurrenyDefault() { }
            public CurrenyDefault(string CnName, string Code)
            {
                this.CNName = CnName;
                this.Code = Code;
            }
        }



        public List<CurrenyDefault> GetDefaultCurreny()
        {
            return new List<CurrenyDefault>
            {
               new CurrenyDefault ("美元","USD" ),
                 new CurrenyDefault  ("欧元","EUR" ),
                 new CurrenyDefault  ("加元","CAD" ),
                new CurrenyDefault   ("澳元","AUD" ),
                  new CurrenyDefault ("英镑","GBP" ),
                 new CurrenyDefault  ("港元","HKD" ),
                 new CurrenyDefault  ("新西兰元","NZD" )
            };
        }
    }
}
