﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Service.Dtos;
using SMP.Service.Reconcile.Dto;
using SMP.Service.ServiceBillExpenseMapping;
using SMP.Service.ServiceBillExpenseMapping.Dto;
using SMP.Tool.TimingTask.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.ServiceBill
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class ServiceBillExpenseMappingController : ControllerBase
    {
        IServiceBillExpenseMappingService _mappingService;
        public ServiceBillExpenseMappingController(IServiceBillExpenseMappingService mappingService)
        {
            _mappingService = mappingService;
        }

        [HttpGet]
        public async Task<ApiJsonModel> GetAllMappingSolutions()
        {
            var solutions = await _mappingService.GetAllMappingSolutions();
            return ApiJsonModel.OK(solutions);
        }

        [HttpPost]
        public async Task<ApiJsonModel> LoadExpenseMappingSolution(LoadExpenseMappingSolutionParam param)
        {
            ServiceBillExpenseMappingSolutionDto result = await _mappingService.LoadMappingSolution(param);
            return ApiJsonModel.OK(result);
        }


        [HttpPost]
        public async Task<ApiJsonModel> QuerySolutionPage(QueryModel queryModel)
        {
            var result = await _mappingService.QuerySolutionPage(queryModel);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> RemoveSolution(IdParam param)
        {
            bool ok = await _mappingService.Remove(param.Id);
            if (ok)
                return ApiJsonModel.OK();
            else
                return ApiJsonModel.Error("删除失败");
        }

        [HttpPost]
        public async Task<ApiJsonModel> SaveMappingSolution(ServiceBillExpenseMappingSolutionDto param)
        {
            int id = await _mappingService.SaveMappingSolution(param);
            return ApiJsonModel.OK(id);

        }

        [HttpPost]
        public async Task<ApiJsonModel> GetBillDataNameAndExpenses(GetBillDataNameAndExpensesParam param)
        {
            ServiceBillDataNameExpense result = await _mappingService.GetBillDataNameAndExpenses(param);
            return ApiJsonModel.OK(result);
        }

        [HttpGet]
        public async Task<ApiJsonModel> GetServiceBillMappings(string billName)
        {
            IEnumerable<NameIdDto> result = await _mappingService.GetServiceBillMappings(billName);
            return ApiJsonModel.OK(result);
        }

        
    }
}
