﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Service.Dtos;
using SMP.Service.ServiceBillResolve;
using SMP.Service.ServiceBillResolve.Dto;
using XYY.Common.Standard;
using XYY.Core.Standard.AliYun;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.ServiceBill
{
    [AllowAnonymous]
    [ApiController]
    [Route("[controller]/[action]")]
    public class ServiceBillResolveController : ControllerBase
    {
        IServiceBillResolveService _serviceBillResolveService;
        IAliYunPostFileSerivce _aliYunPostFileSerivce;

        //AliYunPostFileSerivce

        public ServiceBillResolveController(IServiceBillResolveService serviceBillResolveService, IAliYunPostFileSerivce aliYunPostFileSerivce)
        {
            _serviceBillResolveService = serviceBillResolveService;
            _aliYunPostFileSerivce = aliYunPostFileSerivce;
        }
        [HttpPost]
        public async Task<ApiJsonModel> SaveResolveSolution(ServiceBillResolveSolutionDto dto)
        {
            await _serviceBillResolveService.SaveResolveSolution(dto);
            return ApiJsonModel.OK();
        }


        [HttpPost]
        public async Task<ApiJsonModel> QuerySolutionPage(QueryModel queryModel)
        {
            var result = await _serviceBillResolveService.QuerySolutionPage(queryModel);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> QueryResoveResultPage(QueryResoveResultPageParam param)
        {


            var result = await _serviceBillResolveService.QueryResoveResultPage(param);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> GetBillDataNames(GetResolveFeeTypesParam param)
        {
            List<string> result = await _serviceBillResolveService.GetBillDataNames(param.BillName);
            return ApiJsonModel.OK(result);
        }


        [HttpPost]
        public async Task<ApiJsonModel> QueryResoveApplys(QueryModel queryModel)
        {
            var result = await _serviceBillResolveService.QueryResoveApplyPage(queryModel);
            return ApiJsonModel.OK(result);
        }

        [HttpPost]
        public async Task<ApiJsonModel> LoadSolution(LoadSolutionParam param)
        {
            ServiceBillResolveSolutionDto result = await _serviceBillResolveService.LoadSolution(param);
            return ApiJsonModel.OK(result);
        }


        [HttpPost]
        public async Task<ApiJsonModel> ApplyResolve(ApplyResolveServiceBillParam param)
        {
            if (param == null)
                throw new Exception("参数无效");


            param.FileUrl = param.FileUrl;
            param.SolutionId = param.SolutionId;


            var result = await _serviceBillResolveService.ApplyResolve(param);
            if (result.Success)
                return ApiJsonModel.OK();
            else return ApiJsonModel.Error(result.Message);
        }

        [HttpPost]
        public async Task<ApiJsonModel> GetAllSolutions()
        {
            List<ServiceBillResolveSolutionDto> solutionDtos = await _serviceBillResolveService.GetAllSolutions();
            return ApiJsonModel.OK(solutionDtos);
        }
        [HttpPost]
        public async Task<ApiJsonModel> GetAllEnabledSolutions()
        {
            List<ServiceBillResolveSolutionDto> solutionDtos = await _serviceBillResolveService.GetAllSolutions(true);
            return ApiJsonModel.OK(solutionDtos);
        }


        [HttpPost]
        [RequestSizeLimit(100_000_000)]
        public async Task<ApiJsonModel> UpdateFile()
        {
            var filelist = (await Request.ReadFormAsync()).Files;
            if (filelist != null && filelist.Count > 0)
            {
                var file = filelist[0];

                var originFileName = file.FileName;
                //var ext = Path.GetExtension(originFileName);
                //if (!string.Equals(ext, ".xlsx", StringComparison.CurrentCultureIgnoreCase))
                //{
                //    throw new Exception("目前仅支持Excel 2003+文件(.xlsx)");
                //}
                string fielName = Guid.NewGuid() + file.FileName.Substring(file.FileName.LastIndexOf('.'));
                string url =
                    _aliYunPostFileSerivce.PostHttpFile(file.OpenReadStream(), fielName);
                return ApiJsonModel.OK(new { BillName = originFileName, Url = url });
            }
            else
            {
                return ApiJsonModel.Error("请上传一个文件");
            }
        }

        [HttpPost]
        public async Task<ApiJsonModel> GetSolutionRules(GetSolutionRulesParam param)
        {
            IList<ServiceBillResolveSolutionRuleDto> solutionDtos = await _serviceBillResolveService.GetSolutionRules(param);
            return ApiJsonModel.OK(solutionDtos);
        }

        [HttpPost]
        public async Task<ApiJsonModel> CheckSetBillTimeOnResolve(CheckSetBillTimeOnResolveParam param)
        {
            bool setBillTimeOnResolve = await _serviceBillResolveService.CheckSetBillTimeOnResolve(param);
            string str = setBillTimeOnResolve.ToString();//为什么不直接返回bool，而是string，是因为前端的应用结果遇到false时会提取data值，所以返回字符串方便处理
            return ApiJsonModel.OK(str);
        }




        [HttpPost]
        public async Task<ApiJsonModel> SearchResolvedBillName(SearchResolvedBillNameParam param)
        {
            IEnumerable<string> names = await _serviceBillResolveService.SearchResolvedBillName(param.Name);
            if (names?.Any() == true)
            {
                var result = names.Select(n => new { value = n });
                return ApiJsonModel.OK(result);
            }
            else
            {
                return ApiJsonModel.OK(null);
            }

        }
        [HttpPost]
        public async Task<ApiJsonModel> RemoveSolution(IdParam param)
        {
            bool ok = await _serviceBillResolveService.RemoveSolutionById(param.Id);
            if (ok)
                return ApiJsonModel.OK();
            else
                return ApiJsonModel.Error("删除失败");

        }


    }
}
