﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SMP.Model.Enums;
using SMP.Model.TimeTask;
using SMP.Service.TimeTask;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Mvc;

namespace SMP.Api.Base.Controllers.TimeTask
{
    [ApiController]
    [Route("[controller]/[action]")]
    public class FinanceTaskController : ControllerBase
    {
        private readonly IFinanceTaskService _financeTaskService;
        public FinanceTaskController(IFinanceTaskService financeTaskService)
        {
            this._financeTaskService = financeTaskService;
        }
        [AllowAnonymous]
        [HttpPost]
        public async Task<ApiJsonModel<PageResult<Finance_TimeTask>>> Query(QueryModel queryModel)
        {
            var result = await _financeTaskService.QueryPage(queryModel);
            return ApiJsonModel<PageResult<Finance_TimeTask>>.OK(result);
        }
        [AllowAnonymous]
        [HttpPost]
        public async Task<ApiJsonModel<PageResult<Finance_TimeTypeTaskDetail>>> QueryDetail(QueryModel queryModel)
        {
            var result = await _financeTaskService.QueryDetailPage(queryModel);
            return ApiJsonModel<PageResult<Finance_TimeTypeTaskDetail>>.OK(result);
        }
        [AllowAnonymous]
        [HttpGet]
        public async Task<ApiJsonModel> RetryExecute(int id, int taskTypeEnum)
        {
            await _financeTaskService.RetryExecute(id, taskTypeEnum);
            return ApiJsonModel.OK();
        }
        [AllowAnonymous]
        [HttpPost]
        public async Task<ApiJsonModel> Create(Finance_TimeTask model)
        {
            TimeSpan timeSpan = model.EndTime - model.StartTime;
            if (timeSpan.TotalDays <0) { throw new Exception("开始时间必须小于结束时间！"); }
            if (timeSpan.TotalDays>1) { throw new Exception("开始时间与结束时间最大间隔一天！"); }
            var result= await _financeTaskService.Create(model);
            return ApiJsonModel.OK();
        }
        [AllowAnonymous]
        [HttpGet]
        public async Task<ApiJsonModel<IEnumerable<Finance_TimeTask>>> QueryWaitRead()
        {
            var result = await _financeTaskService.QueryWaitRead();
            return ApiJsonModel<IEnumerable<Finance_TimeTask>>.OK(result);
        }
        [AllowAnonymous]
        [HttpGet]
        public async Task<ApiJsonModel<IEnumerable<Finance_TimeTask>>> QueryFail()
        {
            var result = await _financeTaskService.QueryFail();
            return ApiJsonModel<IEnumerable<Finance_TimeTask>>.OK(result);
        }
        [AllowAnonymous]
        [HttpGet]
        public async Task<ApiJsonModel> ExecuteTaskJob(int taskId,DateTime startTime,DateTime endTime)
        {
            var result = await _financeTaskService.ExecuteTaskJob(taskId,startTime,endTime);
            return ApiJsonModel.OK();
        }

    }
}
