using AspectCore.Extensions.DependencyInjection;
using Agile.Config.Client;
using Microsoft.AspNetCore.Http.Json;
using System.Text.Json;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using System.Text;
using Microsoft.AspNetCore.Authorization;

var builder = WebApplication.CreateBuilder(args);
var client = new ConfigClient();

////עȫ
builder.Configuration.AddAgileConfig(client);

//ע
builder.Host.UseServiceProviderFactory(x => new DynamicProxyServiceProviderFactory());

//עRepository
builder.Services.AddDataService(builder.Configuration, Regions.IDType.Scoped);

//עRepository
builder.Services.AddMVCTransfer();

//עServices
builder.Services.AddAppService(Regions.IDType.Scoped);

//עᰢ
builder.Services.AddAliyunService(builder.Configuration);

//עỺ
builder.Services.AddCache(builder.Configuration, Regions.DistributedCacheType.Memory);

//עû֤
builder.Services.AddAuthentication(JwtBearerDefaults.AuthenticationScheme).AddJwtBearer(options =>
 {
     options.TokenValidationParameters = new TokenValidationParameters()
     {
         ValidateIssuer = true,
         ValidIssuer = builder.Configuration["JWT:Issuer"],
         ValidateAudience = true,
         ValidAudience = builder.Configuration["JWT:Audience"],
         ValidateLifetime = true,
         ValidateIssuerSigningKey = true,
         IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(builder.Configuration["JWT:SecretKey"]))
     };
 });
//нӿڿJWT֤
builder.Services.AddAuthorization(x => x.DefaultPolicy = new AuthorizationPolicyBuilder().RequireAuthenticatedUser().Build());


//ע־ 
builder.Services.AddLog4();

//עcors
string specificOrigins = "all";

builder.Services.AddCors(option => option.AddPolicy(specificOrigins, policy =>
policy.AllowAnyHeader().AllowAnyOrigin().AllowAnyMethod()));
builder.Services.AddEndpointsApiExplorer();
////ȫJson 
builder.Services.AddControllers().AddNewtonsoftJson(options =>
{
    options.SerializerSettings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
    options.SerializerSettings.NullValueHandling = NullValueHandling.Ignore;
    options.SerializerSettings.ContractResolver = new DefaultContractResolver();
});
//עWebApiϢ 
builder.Services.AddSwaggerGen();
var app = builder.Build();

// Configure the HTTP request pipeline.
//if (app.Environment.IsDevelopment())
//{
//עWebApiϢ
app.UseSwagger();
app.UseSwaggerUI();
//}
app.UseHttpLogging();
//app.UseHttpsRedirection();

app.MapControllers();
//app.UseBasicAuthentication();
app.UseAuthentication();//ǰ
app.UseAuthorization();//ں
app.UseCors(specificOrigins); 
app.Use(
          async (context, next) =>
          {
              context.Request.EnableBuffering();
              await next(context);
          }
      );
app.Run();
