﻿using ServiceReference1; 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SMP.Api.Remote
{
    public class FedexApi : IBaseRemoteApi
    {
        public string Name => "Fedex";

        public async Task<bool> IsRemote(Req req)
        {
            RateRequest request = CreateRateRequest(req.ReceiveCity, req.ReceiveState, req.ReceiveZipCode, req.ReceiveCountryCode, req.ReceiveStree);
            RatePortTypeClient client = new RatePortTypeClient(new RatePortTypeClient.EndpointConfiguration(), "https://ws.fedex.com:443/web-services/rate");

            var reply = await client.getRatesAsync(request);
            if (reply.RateReply.HighestSeverity == NotificationSeverityType.SUCCESS ||
                reply.RateReply.HighestSeverity == NotificationSeverityType.NOTE ||
                reply.RateReply.HighestSeverity == NotificationSeverityType.WARNING)
            {
                return ShowRateReply(reply.RateReply);
            }
            else
            {
                return false;
            }
        }

        private RateRequest CreateRateRequest(string receiverCity, string receiverState, string postCode, string countryeCode, string street)
        {
            // Build a RateRequest
            RateRequest request = new RateRequest();
            //
            request.WebAuthenticationDetail = new WebAuthenticationDetail();
            request.WebAuthenticationDetail.UserCredential = new WebAuthenticationCredential();
            request.WebAuthenticationDetail.UserCredential.Key = "hWUfJMn4DWqAJeyr"; // Replace "XXX" with the Key
            request.WebAuthenticationDetail.UserCredential.Password = "IWs80hzmpg2d5RxabvAZyC8ME"; // Replace "XXX" with the Password
                                                                                                   //request.WebAuthenticationDetail.ParentCredential = new WebAuthenticationCredential();
                                                                                                   //request.WebAuthenticationDetail.ParentCredential.Key = "hWUfJMn4DWqAJeyr"; // Replace "XXX" with the Key
                                                                                                   //request.WebAuthenticationDetail.ParentCredential.Password = "IWs80hzmpg2d5RxabvAZyC8ME"; // Replace "XXX"

            request.ClientDetail = new ClientDetail();
            request.ClientDetail.AccountNumber = "670702170"; // Replace "XXX" with the client's account number
            request.ClientDetail.MeterNumber = "252020429"; // Replace "XXX" with the client's meter number 
            //
            request.TransactionDetail = new TransactionDetail();
            request.TransactionDetail.CustomerTransactionId = "***Rate Request using xingyunyi***"; // This is a reference field for the customer.  Any value can be used and will be provided in the response.
            //
            request.Version = new VersionId();
            //
            request.ReturnTransitAndCommit = true;
            request.ReturnTransitAndCommitSpecified = true;
            //
            SetShipmentDetails(request, receiverCity, receiverState, postCode, countryeCode, street);
            //
            return request;
        }
        private static void SetShipmentDetails(RateRequest request, string receiverCity, string receiverState, string postCode, string countryeCode, string street)
        {
            request.RequestedShipment = new RequestedShipment();
            request.RequestedShipment.ShipTimestamp = DateTime.Now; // Shipping date and time 
            request.RequestedShipment.DropoffType = DropoffType.REGULAR_PICKUP; //Drop off types are BUSINESS_SERVICE_CENTER, DROP_BOX, REGULAR_PICKUP, REQUEST_COURIER, STATION


            request.RequestedShipment.ServiceType = "FEDEX_INTERNATIONAL_PRIORITY"; // Service types are STANDARD_OVERNIGHT, PRIORITY_OVERNIGHT, FEDEX_GROUND ...
                                                                                    // request.RequestedShipment.ServiceTypeSpecified = true;
            request.RequestedShipment.PackagingType = "YOUR_PACKAGING"; // Packaging type FEDEX_BOK, FEDEX_PAK, FEDEX_TUBE, YOUR_PACKAGING, ... 
            request.RequestedShipment.PackageCount = "1";
            // request.RequestedShipment.PackagingTypeSpecified = true;
            //
            SetOrigin(request);
            //
            SetDestination(request, receiverCity, receiverState, postCode, countryeCode, street);
            //
            SetPackageLineItems(request);
            //
            request.RequestedShipment.TotalInsuredValue = new Money
            {
                Amount = 5,
                Currency = "USD"
            };
            //
            request.RequestedShipment.PackageCount = "1";
        }


        private static void SetOrigin(RateRequest request)
        {
            request.RequestedShipment.Shipper = new Party();
            request.RequestedShipment.Shipper.Address = new Address();
            request.RequestedShipment.Shipper.Address.StreetLines = new string[1] { "Block A Floor 5, shixia road 8, baiyunhu street, baiyun, guangzhou" };
            request.RequestedShipment.Shipper.Address.City = "GUANGZHOU";
            request.RequestedShipment.Shipper.Address.StateOrProvinceCode = string.Empty;
            request.RequestedShipment.Shipper.Address.PostalCode = "510000";
            request.RequestedShipment.Shipper.Address.CountryCode = "CN";
        }

        private static void SetDestination(RateRequest request, string receiverCity, string receiverState, string postCode, string countryeCode, string street)
        {
            request.RequestedShipment.Recipient = new Party();
            request.RequestedShipment.Recipient.Address = new Address
            {
                StreetLines = new string[1] { street },
                City = receiverCity,
                StateOrProvinceCode = receiverState.Length == 2 ? receiverState : "N",
                PostalCode = postCode,
                CountryCode = countryeCode
            };
        }

        private static void SetPackageLineItems(RateRequest request)
        {
            request.RequestedShipment.RequestedPackageLineItems = new RequestedPackageLineItem[1];
            request.RequestedShipment.RequestedPackageLineItems[0] = new RequestedPackageLineItem
            {
                SequenceNumber = "1", // package sequence number
                GroupPackageCount = "1",
                // package weight
                Weight = new Weight
                {
                    Units = WeightUnits.KG,
                    UnitsSpecified = true,
                    Value = 1,
                    ValueSpecified = true
                },
                // package dimensions
                Dimensions = new Dimensions
                {
                    Length = "10",
                    Width = "13",
                    Height = "4",
                    Units = LinearUnits.IN,
                    UnitsSpecified = true
                },
                // insured value
                InsuredValue = new Money
                {
                    Amount = 10,
                    Currency = "USD"
                }
            };
            // 
        }

        private bool ShowRateReply(RateReply reply)
        {
            Console.WriteLine("RateReply details:");
            foreach (RateReplyDetail rateReplyDetail in reply.RateReplyDetails)
            {

                foreach (RatedShipmentDetail shipmentDetail in rateReplyDetail.RatedShipmentDetails)
                {
                    if (shipmentDetail != null && shipmentDetail.ShipmentRateDetail.Surcharges != null)
                    {
                        foreach (var curcharge in shipmentDetail.ShipmentRateDetail.Surcharges)
                        {
                            if (curcharge.SurchargeType == SurchargeType.OUT_OF_DELIVERY_AREA)
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;
        }
    }
}
